#include "Config.h"


/* Carica le impostazioni del programma:
 *  - options  il puntatore alla struttura che contiene le impostazioni;
 *  - se non trova il file non modifica le opzioni.
 */
void cfg::loadConfig(char *filename, Options *options) {
    //Legge il file config.txt
    ifstream cfg(filename);
    if(!cfg.is_open()) {
        //Impossibile aprire il file => impostazioni standard
        return;
    }
    //Controlla il "BOM" corrispondente alla codifica utf-8
    char bom[]={ 0xEF, 0xBB, 0xBF }, buff[3];
    cfg.read(buff, 3);
    if(bom[0]!=buff[0] || bom[1]!=buff[1] || bom[2]!=buff[2]) {
        //Se non  utf8 usa le impostazioni di default
        return;
    }
    //Legge le impostazioni (racchiuse da virgolette)
    for(int i=0; i<12 && !cfg.eof(); i++) {
        char ch;
        string temp;
        //Cerca l'inizio dell'i-esimo campo
        do {
            cfg.get(ch);
        } while(ch!='"' && !cfg.eof());
        //Legge l'i-esimo campo
        do {
            cfg.get(ch);
            if(ch!='"') temp+=ch;
            else break;
        } while(!cfg.eof());
        //Elabora il contenuto dell'i-esimo campo
        switch(i) {
            case 0:
                //Convertire le animazioni?
                if(temp=="yes") options->anims=1;
                else if(temp=="no") options->anims=2;
                break;

            case 1:
                //Convertire tutti i LODs?
                if(temp=="yes") options->lods=1;
                else if(temp=="no") options->lods=2;
                break;

            case 2:
                //Scrivere i file log?
                if(temp=="yes") options->log=1;
                else if(temp=="no") options->log=2;
                break;

            case 3:
                //Scrivere i file mat?
                if(temp=="yes") options->mat=1;
                else if(temp=="no") options->mat=2;
                break;

            case 4:
                //Usare cartella output indicata dall'utente?
                if(temp=="yes") options->out=1;
                else if(temp=="no") options->out=2;
                break;

            case 5:
                //Cartella output prescelta
                if(!temp.empty()) options->outPath=temp.c_str();
                break;

            case 6:
                //MSTS  installato?
                if(temp=="yes") options->ffed=1;
                else if(temp=="no") options->ffed=2;
                break;

            case 7:
                //Percorso di ffeditc
                if(!temp.empty()) options->ffedPath=temp.c_str();
                break;

            case 8:
                //Usare cartella temp indicata dall'utente?
                if(temp=="yes") options->temp=1;
                else if(temp=="no") options->temp=2;
                break;

            case 9:
                //Cartella temp prescelta
                if(!temp.empty()) options->tempPath=temp.c_str();
                break;

            case 10:
                //Cartelle recenti
                if(!temp.empty()) {
                    int n=0;
                    //Per comodit uso FXString
                    FXString elenco=temp.c_str();
                    //Numero di separatori (== a capo) = numero cartelle -1
                    n=elenco.contains('\t')+1;
                    for(int i=0; i<n; i++) {
                        options->recentInputPaths.push_back( elenco.section('\t',i) );
                    }
                }
                break;

            case 11:
                //Cartelle preferite
                if(!temp.empty()) {
                    int n=0;
                    //Per comodit uso FXString
                    FXString elenco=temp.c_str();
                    //Numero di separatori (== a capo) = numero cartelle -1
                    n=elenco.contains('\t')+1;
                    for(int i=0; i<n; i++) {
                        options->prefInputPaths.push_back( elenco.section('\t',i) );
                    }
                }
                break;
        }
    }
    //Chiude il file
    cfg.close();
}



/* Salva le impostazioni del programma:
 *  - options  il puntatore alla struttura che contiene le impostazioni;
 *  - se non pu il file non lancia errori.
 */
void cfg::saveConfig(char *filename, Options *options) {
    //Apre il file in output
    ofstream cfg(filename);
    if(!cfg.is_open()) {
        //Impossibile aprire il file => impostazioni standard
        return;
    }
    //Scrive il bom
    char bom[]={ 0xEF, 0xBB, 0xBF };
    cfg<<bom[0]<<bom[1]<<bom[2];
    //Scrive il titolo
    cfg<<"+---------------------------------+"<<endl;
    cfg<<"| Please do not modify this file. |"<<endl;
    cfg<<"+---------------------------------+"<<endl<<endl;
    //Scrive le impostazioni:
    // - Animazioni
    cfg<<"$ Convert animations: ";
    options->anims==1 ? cfg<<'"'<<"yes"<<'"'<<endl : cfg<<'"'<<"no"<<'"'<<endl ;
    // - LODs
    cfg<<"$ Convert every LOD: ";
    options->lods==1 ? cfg<<'"'<<"yes"<<'"'<<endl : cfg<<'"'<<"no"<<'"'<<endl ;
    // - File log
    cfg<<"$ Write log file: ";
    options->log==1 ? cfg<<'"'<<"yes"<<'"'<<endl : cfg<<'"'<<"no"<<'"'<<endl ;
    // - File mat
    cfg<<"$ Write material file: ";
    options->mat==1 ? cfg<<'"'<<"yes"<<'"'<<endl : cfg<<'"'<<"no"<<'"'<<endl ;
    // - Output dir
    cfg<<"$ Use non-default output folder: ";
    options->out==1 ? cfg<<'"'<<"yes"<<'"'<<endl : cfg<<'"'<<"no"<<'"'<<endl ;
    cfg<<"$ Output folder path: ";
    cfg<<'"'<<options->outPath.text()<<'"'<<endl;
    // - MSTS dir
    cfg<<"$ MSTS is installed: ";
    options->ffed==1 ? cfg<<'"'<<"yes"<<'"'<<endl : cfg<<'"'<<"no"<<'"'<<endl ;
    cfg<<"$ ffeditc path: ";
    cfg<<'"'<<options->ffedPath.text()<<'"'<<endl;
    // - Temp dir
    cfg<<"$ Use non-default temp folder: ";
    options->temp==1 ? cfg<<'"'<<"yes"<<'"'<<endl : cfg<<'"'<<"no"<<'"'<<endl ;
    cfg<<"$ Temp folder path: ";
    cfg<<'"'<<options->tempPath.text()<<'"'<<endl;

    //Cartelle recenti
    cfg<<"$ Recent folders: ";
    cfg<<'"';
    unsigned n=options->recentInputPaths.size();
    for(unsigned i=0; i<n; i++) {
        cfg<<options->recentInputPaths[i].text();
        if(i!=n-1) cfg<<'\t';
    }
    cfg<<'"'<<endl;

    //Cartelle preferite
    cfg<<"$ Pref folders: ";
    cfg<<'"';
    n=options->prefInputPaths.size();
    for(unsigned i=0; i<n; i++) {
        cfg<<options->prefInputPaths[i].text();
        if(i!=n-1) cfg<<'\t';
    }
    cfg<<'"'<<endl;


    //Chiude il file
    cfg.close();
}
