#include "Convertitore.h"

#ifdef CONSOLE
    #define TOASCII c_str
#else
    #define TOASCII text
#endif

/* Restituisce:
 *  - 1 se fallisce;
 *  - 0 se termina correttamente.
 */
#ifdef CONSOLE
int conv::convert(const string &fileName)
#else
int conv::convert(FXString fileName, FXString filePath, bool useTempDir, Options &options, ProgressDialog &pgsDlg)
#endif
{


    /* Opzioni di lettura */
    #ifdef CONSOLE
    //Nella versione console, decido io le opzioni
    bool readAllLods=false;
    bool readAnims=true;
    bool matFile=true;
    bool logFile=true;
    #else
    bool readAllLods= options.lods==1 ? true : false;
    bool readAnims= options.anims==1 ? true : false;
    bool matFile= options.mat==1 ? true : false;
    bool logFile= options.log==1 ? true : false;
    #endif


    /* Percorsi e nome del file senz estensione */
    #ifdef CONSOLE
    string inPath, outPath, name, fullPath;
    unsigned pos=fileName.find_last_of('\\');
    if(pos!=string::npos) {
        inPath.assign(fileName, 0, pos);
        name.assign(fileName, pos+1, fileName.length()-pos-2);
    }
    else {
        name.assign(fileName, 0, fileName.length()-2);
    }
    outPath=inPath;
    #else
    FXString inPath=(useTempDir ? options.tempPath : filePath);
    FXString outPath=( options.out==1 ? options.outPath : filePath );
    FXString name=FXPath::stripExtension(fileName);
    FXString fullPath;
    #endif


    /* Apre il file log */
    #ifdef CONSOLE
    //Nella versione console, il log  la finestra di prompt
    ostream &log=cout;
    #else
    ofstream file;
    fullPath = outPath + '\\' + name + "_log.txt";
    if(logFile) {
        file.open(fullPath.text());
        if(!file.is_open()) logFile=false;
    }
    ostream &log=file;
    #endif


    /* Oggetti e variabili per la conversione */
    #ifdef CONSOLE
    TSshape tsShape(log, logFile);
    Mesh mesh(tsShape, log, logFile);
    File3ds file3ds(mesh, log, logFile);
    #else
    TSshape tsShape(log, logFile, pgsDlg);
    Mesh mesh(tsShape, log, logFile, pgsDlg);
    File3ds file3ds(mesh, log, logFile);//, pgsDlg);
    pgsDlg.setTotal(100); //La barra entra in funzione solo per la lettura dei LOD
    pgsDlg.setProgress(0);
    #endif
    unsigned stato=0;


    /* Inizia la conversione */
    try {
        //Apre il file
        #ifdef CONSOLE
        fullPath=fileName;
        #else
        fullPath= inPath + '\\' + (useTempDir ? "ShapeConverterTemp.s" : fileName);
        #endif
        if(logFile) log<<"Opening file '"<<fileName.TOASCII()<<"' in '"<<inPath.TOASCII()<<"'...";
        tsShape.openFile(fullPath.TOASCII());
        if(logFile) log<<" done!"<<endl;

        //Legge il file
        if(logFile) log<<"Parsing shape file...";
        #ifndef CONSOLE
        pgsDlg.setMidLabel("Parsing shape file...");
        if(!pgsDlg.check()) throw 0;
        #endif
        tsShape.readGeneral();
        tsShape.readLods(readAllLods);
        if(readAnims) {
            //Legge le animazioni
            tsShape.readAnims();
        }
        if(logFile) log<<" done!"<<endl;

        //Chiude il file
        if(logFile) log<<"Closing the file...";
        tsShape.closeFile();
        if(logFile) log<<" done!"<<endl<<endl;

        //Parte comune a tutti i lods: materiali e lista oggetti
        if(logFile) log<<"Structuring materials...";
        #ifndef CONSOLE
        pgsDlg.setMidLabel("Structuring materials...");
        if(!pgsDlg.check()) throw 0;
        #endif
        mesh.importMat();
        if(logFile) log<<" done!"<<endl<<"Structuring objects...";
        #ifndef CONSOLE
        pgsDlg.setMidLabel("Structuring objects...");
        if(!pgsDlg.check()) throw 0;
        #endif
        mesh.importObj();
        if(logFile) log<<" done!"<<endl<<"Structuring matrices...";
        #ifndef CONSOLE
        pgsDlg.setMidLabel("Structuring matrices...");
        if(!pgsDlg.check()) throw 0;
        #endif
        mesh.importMatrices();
        if(logFile) log<<" done!"<<endl<<"Structuring animations...";
        #ifndef CONSOLE
        pgsDlg.setMidLabel("Structuring animations...");
        if(!pgsDlg.check()) throw 0;
        #endif
        mesh.importKeys();
        if(logFile) log<<" done!"<<endl<<endl;

        //Esporta la lista dei materiali, se richiesto
        if(matFile) {
            if(logFile) log<<"Writing material list...";
            #ifdef CONSOLE
            fullPath = outPath + name;
            #else
            fullPath = outPath + '\\' + name;
            #endif
            fullPath+="_mat.txt";
            mesh.writeMatFile(fullPath.TOASCII());
            if(logFile) log<<" done!"<<endl<<endl;
        }

        //Controllo nomi mat, img, obj
        if(logFile) log<<"Checking materials, images and objects names..."<<endl;
        #ifndef CONSOLE
        pgsDlg.setMidLabel("Checking materials, images and objects names...");
        if(!pgsDlg.check()) throw 0;
        #endif
        mesh.checkMatNames();
        mesh.checkObjNames();
        if(logFile) log<<"done!"<<endl<<endl;

        //Numero di lods da analizzare
        u8 numLods=0;
        if(readAllLods) {
            //L'utente ha chiesto di convertirli tutti
            numLods=(u8)tsShape.getTSlods()->getLung();
            if(logFile) log<<"We are going to convert all LODs: there are "<<(int)numLods<<" LODs in this file."<<endl;
        }
        else {
            //L'utente vuole convertire solo il primo
            if(logFile) log<<"We are going to convert only first LOD."<<endl;
            #ifndef CONSOLE
            pgsDlg.setMidLabel("Converting only first LOD...");
            if(!pgsDlg.check()) throw 0;
            #endif
            numLods=1;
        }


        //Parte relativa ad ogni lod
        for(u8 i=0; i<numLods; i++) {
            if(logFile) log<<"Converting LOD no."<<(int)i<<"..."<<endl;
            #ifndef CONSOLE
            FXString lodLabel, tempLabel;
            if(numLods!=1) {
                lodLabel.format("LOD %d of %d: ", i+1, numLods);
                pgsDlg.setMidLabel(lodLabel);
                if(!pgsDlg.check()) throw 0;
            }
            else lodLabel="First LOD: ";
            #endif
            //Elenco facce, suddivise per oggetto e materiale
            if(logFile) log<<"\tSetting up faces list...";
            #ifndef CONSOLE
            tempLabel=lodLabel+"setting up faces list...";
            pgsDlg.setMidLabel(tempLabel);
            if(!pgsDlg.check()) throw 0;
            #endif
            mesh.importFaces(i);
            if(logFile) log<<" done!"<<endl;
            //Importazione vertici, vertici_uv e normali
            if(logFile) log<<"setting up vertices and normals lists...";
            #ifndef CONSOLE
            tempLabel=lodLabel+"setting up vertices and normals lists...";
            pgsDlg.setMidLabel(tempLabel);
            if(!pgsDlg.check()) throw 0;
            #endif
            mesh.importVertices(i);
            if(logFile) log<<" done!"<<endl;

            //Controllo normali facce
            if(logFile) log<<"\tChecking normals..."<<endl;
            #ifndef CONSOLE
            tempLabel=lodLabel+"checking normals...";
            pgsDlg.setMidLabel(tempLabel);
            if(!pgsDlg.check()) throw 0;
            #endif
            mesh.checkNormals();
            if(logFile) log<<"\tdone!"<<endl;


            //Esportazione in 3ds: apertura file
            #ifdef CONSOLE
            fullPath = outPath + name + '_';
            #else
            fullPath = outPath + '\\' + name + '_';
            tempLabel=lodLabel+"writing 3ds file...";
            pgsDlg.setMidLabel(tempLabel);
            #endif
            fullPath += int2string(i, true).c_str();
            fullPath += ".3ds";
            if(logFile) log<<"\tOpening the file '"<<fullPath.TOASCII()<<"'...";
            file3ds.open(fullPath.TOASCII());
            if(logFile) log<<" done!"<<endl;

            //Esportazione in 3ds: calcolo dimensioni
            if(logFile) log<<"\tComputing the size of the file...";
            file3ds.sizeCalc();
            if(logFile) log<<" done!"<<endl;

            //Esportazione in 3ds: scrittura file
            if(logFile) log<<"\tWriting file header...";
            file3ds.scriviTestata();
            if(logFile) log<<" done!"<<endl<<"\tWriting materials section...";
            file3ds.scriviMateriali();
            if(logFile) log<<" done!"<<endl<<"\tWriting objects section...";
            file3ds.scriviOggetti();
            if(logFile) log<<" done!"<<endl<<"\tWriting animations section...";
            file3ds.scriviAnimazioni();
            if(logFile) log<<" done!"<<endl;

            //Esportazione in 3ds: chiusura file
            if(logFile) log<<"\tClosing the file...";
            file3ds.close();
            if(logFile) log<<"done!"<<endl;


            //Svuotamento liste geometria di mesh
            if(logFile) log<<"\tDeleting geometry...";
            mesh.eraseGeom();
            if(logFile) log<<" done!"<<endl;
            if(logFile) log<<"End of LOD."<<endl<<endl;
        }
    }
    catch (bad_alloc xa) {
        //cout<<"Errore di allocazione memoria non definito!"<<endl;
        if(logFile) log<<"Error! Bad_alloc"<<endl;
        stato=1;
    }
    catch (Errore err) {
        if(logFile) {
            log<<"Error! Error code: ";
            log<<err.getErrClass()<<'/';
            log<<err.getErrOper()<<'/';
            log<<err.getErrType()<<'/';
            log<<err.getNumElem()<<'/';
            log<<err.getFileStat()<<'/';
            log<<err.getErrFileOp()<<endl;
        }
        /*err.a=errore.getErrClass();
        err.b=errore.getErrOper();
        err.c=errore.getErrType();
        err.d=errore.getNumElem();
        err.e=errore.getFileStat();
        err.f=errore.getErrFileOp();*/
        stato=1;
    }
    catch(int interrupt) {
        if(logFile) log<<"STOP! Cancelled by user."<<endl;
        stato=3;
    }
    catch (...) {
        //cout<<"Errore non previsto!"<<endl;
        if(logFile) log<<"Error! Undefined."<<endl;
        stato=1;
    }

    //Cancella tsShape
    if(logFile) log<<"Deleting shape structure...";
    tsShape.eraseAll();
    if(logFile) log<<" done!"<<endl;

    //Cancella mesh
    if(logFile) log<<"Deleting mesh structure...";
    mesh.eraseAll();
    if(logFile) log<<" done!"<<endl;


    //Fine
    return stato;
}

