#include "OptionsDialog.h"


/** CLASSE OptionsDialog: FOX's MACROS **/

/* Macro, message map:
 *  - message type: what appened;
 *  - message id: source;
 *  - message handler: what to do.
 */
FXDEFMAP(OptionsDialog) OptionsDialogMap[]={
   //_________Message_Type_____________________ID__________________________Message_Handler_______________
   FXMAPFUNC(   SEL_COMMAND,           OptionsDialog::ID_OUTCB,        OptionsDialog::onCmdOutCheckBtn  ),
   FXMAPFUNC(   SEL_COMMAND,           OptionsDialog::ID_OUTBROW,      OptionsDialog::onCmdOutBrowse    ),
   FXMAPFUNC(   SEL_COMMAND,           OptionsDialog::ID_FFEDCB,       OptionsDialog::onCmdFfedCheckBtn ),
   FXMAPFUNC(   SEL_COMMAND,           OptionsDialog::ID_FFEDBROW,     OptionsDialog::onCmdFfedBrowse   ),
   FXMAPFUNC(   SEL_COMMAND,           OptionsDialog::ID_TEMPCB,       OptionsDialog::onCmdTempCheckBtn ),
   FXMAPFUNC(   SEL_COMMAND,           OptionsDialog::ID_TEMPBROW,     OptionsDialog::onCmdTempBrowse   ),
   FXMAPFUNC(   SEL_COMMAND,           OptionsDialog::ID_OUTSEE,       OptionsDialog::onCmdSeeBtn       ),
   FXMAPFUNC(   SEL_COMMAND,           OptionsDialog::ID_FFEDSEE,      OptionsDialog::onCmdSeeBtn       ),
   FXMAPFUNC(   SEL_COMMAND,           OptionsDialog::ID_TEMPSEE,      OptionsDialog::onCmdSeeBtn       ),
   FXMAPFUNC(   SEL_UPDATE,            OptionsDialog::ID_PREFLIST,     OptionsDialog::onUpdPrefList     ),
   FXMAPFUNC(   SEL_COMMAND,           OptionsDialog::ID_PREFDEL,      OptionsDialog::onCmdDelPref      ),
   FXMAPFUNC(   SEL_COMMAND,           OptionsDialog::ID_PREFCLR,      OptionsDialog::onCmdDelPref      )
};


/* Macro, message map implementation:
 *  - 1, the name of the class;
 *  - 2, the name of the parent class;
 *  - 3, pointer to message map;
 *  - 4, no. of messages in message map.
 */
FXIMPLEMENT(OptionsDialog, FXDialogBox, OptionsDialogMap, ARRAYNUMBER(OptionsDialogMap))




/** CLASSE OptionsDialog: COSTRUTTORE E DISTRUTTORE **/

/* This function builds the window:
 *  - 1, pointer to the FXWindow owner;
 *  - 2, the window title;
 *  - 5, window decorations;
 *  - 6, initial position, w;
 *  - 7, initial position, h;
 *  - 8, initial size, w;
 *  - 9, initial size, h.
 */
OptionsDialog::OptionsDialog(FXWindow* owner, FXIcon *_iconsBig[], FXIcon *_iconsSmall[], Options *_opt) : FXDialogBox(owner, "Options", DECOR_TITLE|DECOR_CLOSE|DECOR_BORDER, 0, 0, 400, 330, 0,0,0,0) {
    //Here we can build the window

    /// Icons
    iconsBig=_iconsBig;
    iconsSmall=_iconsSmall;
    setIcon(iconsBig[ICON_WINDOW]);
    setMiniIcon(iconsSmall[ICON_WINDOW]);

    /// DataTargets
    //NB: opzioni passate per indirizzo! => Non vanno modificate
    //Solo le prefdir vanno modificate direttamente
    options=_opt;
    anims.connect(options->anims);
    lods.connect(options->lods);
    log.connect(options->log);
    mat.connect(options->mat);
    out=options->out;
    outPath=options->outPath;
    ffed=options->ffed;
    ffedPath=options->ffedPath;
    temp=options->temp;
    tempPath=options->tempPath;

    /// Dialog frame
    dialogFrame=new FXVerticalFrame(this, LAYOUT_FILL_X|LAYOUT_FILL_Y,0,0,0,0, 0,0, 0,0, 0,0);
        /// Main frame
        mainFrame=new FXHorizontalFrame(dialogFrame, LAYOUT_FILL_X|LAYOUT_FILL_Y,0,0,0,0, 5,5, 5,5, 5,5);
            /// Switcher
            switcher=new FXSwitcher(mainFrame,LAYOUT_FILL_X|LAYOUT_FILL_Y|LAYOUT_RIGHT|FRAME_RAISED);

                /// Options 1: program options
                progOpt=new FXVerticalFrame(switcher, LAYOUT_FILL_X|LAYOUT_FILL_Y|FRAME_NONE,0,0,0,0, 5,5, 5,5, 5,5);
                    /// Output folder
                    outBox=new FXGroupBox(progOpt,"Output folder:", GROUPBOX_TITLE_LEFT|FRAME_GROOVE|LAYOUT_FILL_COLUMN|LAYOUT_FILL_X,0,0,0,0, 2,2, 5,5, 5,5);
                    outCheckBtn=new FXCheckButton(outBox, "Use input folder as standard output folder.", this, OptionsDialog::ID_OUTCB, ICON_BEFORE_TEXT|FRAME_NONE|LAYOUT_CENTER_X,0,0,0,0,5,5,5,5);
                    outBtnFrame=new FXHorizontalFrame(outBox, LAYOUT_CENTER_X,0,0,0,0,0,0,0,0);
                    outChooseBtn=new FXButton(outBtnFrame, "Browse...", NULL, this, OptionsDialog::ID_OUTBROW, BUTTON_NORMAL,0,0,0,0,10,10,5,5);
                    if(out==2) {
                        //Stato attuale = usa la stessa cartella di input
                        outCheckBtn->setCheck(true);
                        outChooseBtn->disable();
                    }
                    outSeeBtn=new FXButton(outBtnFrame, "See current...", NULL, this, OptionsDialog::ID_OUTSEE, BUTTON_NORMAL,0,0,0,0,10,10,5,5);
                    /// Ffeditc path
                    ffedBox=new FXGroupBox(progOpt,"MS Train Simulator root directory:", GROUPBOX_TITLE_LEFT|FRAME_GROOVE|LAYOUT_FILL_COLUMN|LAYOUT_FILL_X,0,0,0,0, 2,2, 5,5, 5,5);
                    ffedCheckBtn=new FXCheckButton(ffedBox, "MS Train Simulator isn't installed on this computer.", this, OptionsDialog::ID_FFEDCB, ICON_BEFORE_TEXT|FRAME_NONE|LAYOUT_CENTER_X,0,0,0,0,5,5,5,5);
                    ffedBtnFrame=new FXHorizontalFrame(ffedBox, LAYOUT_CENTER_X,0,0,0,0,0,0,0,0);
                    ffedChooseBtn=new FXButton(ffedBtnFrame, "Browse...", NULL, this, OptionsDialog::ID_FFEDBROW, BUTTON_NORMAL,0,0,0,0,10,10,5,5);
                    if(ffed==2) {
                        //Stato attuale = ts non  installato
                        ffedCheckBtn->setCheck(true);
                        ffedChooseBtn->disable();
                    }
                    ffedSeeBtn=new FXButton(ffedBtnFrame, "See current...", NULL, this, OptionsDialog::ID_FFEDSEE, BUTTON_NORMAL,0,0,0,0,10,10,5,5);
                    /// Temp path
                    tempBox=new FXGroupBox(progOpt,"Temp directory:", GROUPBOX_TITLE_LEFT|FRAME_GROOVE|LAYOUT_FILL_COLUMN|LAYOUT_FILL_X,0,0,0,0, 2,2, 5,5, 5,5);
                    tempCheckBtn=new FXCheckButton(tempBox, "Use default temp directory", this, OptionsDialog::ID_TEMPCB, ICON_BEFORE_TEXT|FRAME_NONE|LAYOUT_CENTER_X,0,0,0,0,5,5,5,5);
                    tempBtnFrame=new FXHorizontalFrame(tempBox, LAYOUT_CENTER_X,0,0,0,0,0,0,0,0);
                    tempChooseBtn=new FXButton(tempBtnFrame, "Browse...", NULL, this, OptionsDialog::ID_TEMPBROW, BUTTON_NORMAL,0,0,0,0,10,10,5,5);
                    if(temp==2) {
                        //Stato attuale = usa default
                        tempCheckBtn->setCheck(true);
                        tempChooseBtn->disable();
                    }
                    tempSeeBtn=new FXButton(tempBtnFrame, "See current...", NULL, this, OptionsDialog::ID_TEMPSEE, BUTTON_NORMAL,0,0,0,0,10,10,5,5);

                /// Options 2: conversion options
                convOpt=new FXMatrix(switcher, 2, MATRIX_BY_COLUMNS|LAYOUT_FILL_X|FRAME_NONE,0,0,0,0, 5,5, 5,5, 10,8);
                    /// Animation conversion
                    animsBox=new FXGroupBox(convOpt,"Animations:", GROUPBOX_TITLE_LEFT|FRAME_GROOVE|LAYOUT_FILL_COLUMN|LAYOUT_FILL_X,0,0,0,0, 2,2, 5,5, 5,5);
                    new FXRadioButton(animsBox, "Convert", &anims, FXDataTarget::ID_OPTION+1);
                    new FXRadioButton(animsBox, "Don't convert", &anims, FXDataTarget::ID_OPTION+2);
                    /// LODs conversion
                    lodsBox=new FXGroupBox(convOpt,"LODs:", GROUPBOX_TITLE_LEFT|FRAME_GROOVE|LAYOUT_FILL_COLUMN|LAYOUT_FILL_X,0,0,0,0, 2,2, 5,5, 5,5);
                    new FXRadioButton(lodsBox, "Convert every LOD", &lods, FXDataTarget::ID_OPTION+1);
                    new FXRadioButton(lodsBox, "Convert only first LOD", &lods, FXDataTarget::ID_OPTION+2);
                    /// Log file
                    logBox=new FXGroupBox(convOpt,"Log file:",GROUPBOX_TITLE_LEFT|FRAME_GROOVE|LAYOUT_FILL_COLUMN|LAYOUT_FILL_X,0,0,0,0, 2,2, 5,5, 5,5);
                    new FXRadioButton(logBox, "Write the log file", &log, FXDataTarget::ID_OPTION+1);
                    new FXRadioButton(logBox, "Don't write it", &log, FXDataTarget::ID_OPTION+2);
                    /// Mat file
                    matBox=new FXGroupBox(convOpt,"Materials file:",GROUPBOX_TITLE_LEFT|FRAME_GROOVE|LAYOUT_FILL_COLUMN|LAYOUT_FILL_X,0,0,0,0, 2,2, 5,5, 5,5);
                    new FXRadioButton(matBox, "Write the materials file", &mat, FXDataTarget::ID_OPTION+1);
                    new FXRadioButton(matBox, "Don't write it", &mat, FXDataTarget::ID_OPTION+2);

                /// Options 3: pref folders
                prefOpt=new FXVerticalFrame(switcher, LAYOUT_FILL_X|LAYOUT_FILL_Y|FRAME_NONE,0,0,0,0, 5,5, 5,5, 5,5);
                    /// Folder list
                    prefList=new FXIconList(prefOpt, this, OptionsDialog::ID_PREFLIST, LIST_SINGLESELECT|LAYOUT_FILL_X|LAYOUT_FILL_Y);
                    prefList->appendHeader("Directory", NULL, 250); //NULL  l'icona
                    unsigned prefs=options->prefInputPaths.size();
                    for(unsigned i=0; i<prefs; i++) {
                        prefList->appendItem(options->prefInputPaths[i], NULL, iconsSmall[ICON_FOLDER_STAR]);
                    }
                    FXHorizontalFrame *btns=new FXHorizontalFrame(prefOpt,FRAME_NONE|LAYOUT_CENTER_X,0,0,0,0,5,5,0,0);
                    prefDelBtn=new FXButton(btns, "Delete", NULL, this, OptionsDialog::ID_PREFDEL, BUTTON_NORMAL,0,0,0,0,10,10,5,5);
                    prefClrBtn=new FXButton(btns, "Delete all", NULL, this, OptionsDialog::ID_PREFCLR, BUTTON_NORMAL,0,0,0,0,10,10,5,5);

            /// Buttons frame
            btnFrame=new FXVerticalFrame(mainFrame, LAYOUT_FILL_Y|PACK_UNIFORM_WIDTH|FRAME_RAISED,0,0,0,0, 5,5, 5,5, 5,5);
                /// Switcher buttons
                FXVerticalFrame *temp=new FXVerticalFrame(btnFrame,FRAME_SUNKEN,0,0,0,0,0,0,0,0); //Per avere l'effetto bordo scavato sui buttons
                switchBtn[0]=new FXButton(temp, "\tEdit folder options", iconsBig[ICON_OPT_FOLDER], switcher, FXSwitcher::ID_OPEN_FIRST+0, FRAME_RAISED|ICON_ABOVE_TEXT|LAYOUT_FILL_X,0,0,0,0,2,2,2,2);
                temp = new FXVerticalFrame(btnFrame,FRAME_SUNKEN,0,0,0,0,0,0,0,0);
                switchBtn[1]=new FXButton(temp, "\tEdit conversion options", iconsBig[ICON_OPT_APP], switcher, FXSwitcher::ID_OPEN_FIRST+1, FRAME_RAISED|ICON_ABOVE_TEXT|LAYOUT_FILL_X,0,0,0,0,2,2,2,2);
                temp = new FXVerticalFrame(btnFrame,FRAME_SUNKEN,0,0,0,0,0,0,0,0);
                switchBtn[2]=new FXButton(temp, "\tEdit favourit folders", iconsBig[ICON_OPT_PREF], switcher, FXSwitcher::ID_OPEN_FIRST+2, FRAME_RAISED|ICON_ABOVE_TEXT|LAYOUT_FILL_X,0,0,0,0,2,2,2,2);

        /// Lower frame
        lowerFrame=new FXHorizontalFrame(dialogFrame, LAYOUT_FILL_X,0,0,0,0, 5,5, 5,5, 5,5);
            /// Buttons
            saveBtn=new FXButton(lowerFrame, "Save\tSave changes", NULL, this, ID_ACCEPT, BUTTON_NORMAL|LAYOUT_RIGHT,0,0,0,0,10,10,5,5);
            cancelBtn=new FXButton(lowerFrame, "Cancel\tDiscard changes", NULL, this, ID_CANCEL, BUTTON_NORMAL|LAYOUT_RIGHT,0,0,0,0,10,10,5,5);

    /// End of OptionsDialog
}

/* Distruttore:
 *  - only MenuPanes, Cursors, Fonts, Images, Icons, Bitmaps, and Visuals.
 *  - see 'http://www.fox-toolkit.com/faq.html#DELETINGSTUFF'.
 */
OptionsDialog::~OptionsDialog() {
    //Nothing to do (icons belong to MyWindow class)
}




/** CLASSE OptionsDialog: CALLBACKS functions **/

/* Gestisce il checkButton della cartella di output:
 *  - no comment.
 */
long OptionsDialog::onCmdOutCheckBtn(FXObject *sender, FXSelector sel, void *ptr) {
    if(outCheckBtn->getCheck()) {
        outChooseBtn->disable();
        out=2;
    }
    else {
        outChooseBtn->enable();
        out=1;
    }
    return 1;
}


/* Gestisce il chooseBtn della cartella di output:
 *  - no comment.
 */
long OptionsDialog::onCmdOutBrowse(FXObject *sender, FXSelector sel, void *ptr) {
    //Crea il dialogo
    FXDirDialog outDlg(this, "Choose your preferred output folder");
    //Cartella iniziale
    outDlg.setDirectory(outPath);
    //Icona
    outDlg.setMiniIcon(iconsSmall[ICON_WINDOW]);
    //Esegue e visualizza il dialogo
    if(outDlg.execute()==1) {
        //Salva la nuova cartella scelta
        outPath=outDlg.getDirectory();
    }
    return 1;
}


/* Gestisce il checkButton della cartella di msts:
 *  - no comment.
 */
long OptionsDialog::onCmdFfedCheckBtn(FXObject *sender, FXSelector sel, void *ptr) {
    if(ffedCheckBtn->getCheck()) {
        ffedChooseBtn->disable();
        FXMessageBox::warning(this, MBOX_OK, "ShapeConverter", "You can convert only unicode shape files.");
        ffed=2;
    }
    else {
        ffedChooseBtn->enable();
        ffed=1;
    }
    return 1;
}


/* Gestisce il chooseBtn della cartella di msts:
 *  - no comment.
 */
long OptionsDialog::onCmdFfedBrowse(FXObject *sender, FXSelector sel, void *ptr) {
    //Crea il dialogo
    FXDirDialog ffedDlg(this, "Choose MSTS root folder");
    //Cartella iniziale
    FXString mstsPath=ffedPath.rbefore('\\',3);
    ffedDlg.setDirectory(mstsPath);
    //Icona
    ffedDlg.setMiniIcon(iconsSmall[ICON_WINDOW]);
    //Esegue e visualizza il dialogo
    if(ffedDlg.execute()==1) {
        //Salva la nuova cartella scelta
        ffedPath=ffedDlg.getDirectory();
        ffedPath.append("\\UTILS\\FFEDIT\\ffeditc_unicode.exe");
        //Controlla se esiste il file exe (todo)
        //Usare: FXStat::exists(name) oppure costruirla con FindFile di win
        if(win::fileExist(ffedPath)!=1) {
            FXMessageBox::warning(this, MBOX_OK, "ShapeConverter", "Invalid path! Can't find 'ffeditc_unicode.exe'.");
            ffedCheckBtn->setCheck(true, true);
        }
    }
    return 1;
}


/* Gestisce il checkButton della cartella temporanea:
 *  - no comment.
 */
long OptionsDialog::onCmdTempCheckBtn(FXObject *sender, FXSelector sel, void *ptr) {
    if(tempCheckBtn->getCheck()) {
        tempChooseBtn->disable();
        temp=2;
    }
    else {
        tempChooseBtn->enable();
        temp=1;
    }
    return 1;
}


/* Gestisce il chooseBtn della cartella temporanea:
 *  - no comment.
 */
long OptionsDialog::onCmdTempBrowse(FXObject *sender, FXSelector sel, void *ptr) {
    //Crea il dialogo
    FXDirDialog tempDlg(this, "Choose your preferred temp folder");
    //Cartella iniziale
    tempDlg.setDirectory(tempPath);
    //Icona
    tempDlg.setMiniIcon(iconsSmall[ICON_WINDOW]);
    //Esegue e visualizza il dialogo
    if(tempDlg.execute()==1) {
        //Salva la nuova cartella scelta
        tempPath=tempDlg.getDirectory();
    }
    return 1;
}


/* Gestisce i button per visualizzare la cartella attuale:
 *  - no comment.
 */
long OptionsDialog::onCmdSeeBtn(FXObject *sender, FXSelector sel, void *ptr) {
    FXString dir;
    if(FXSELID(sel)==ID_OUTSEE) dir=outPath;
    else if(FXSELID(sel)==ID_FFEDSEE) dir=ffedPath.rbefore('\\',3);
    else if(FXSELID(sel)==ID_TEMPSEE) dir=tempPath;
    dir.prepend("Current directory:\n");
    FXMessageBox::information(this, MBOX_OK, "ShapeConverter", dir.text() );
    return 1;
}


/* Gestisce l'attivazione dei buttons per cancellare le cartelle preferite:
 *  - no comment.
 */
long OptionsDialog::onUpdPrefList(FXObject *sender, FXSelector sel, void *ptr) {
    if(prefList->getNumRows()>0) {
        prefClrBtn->enable();
        if( prefList->isItemSelected( prefList->getCurrentItem() ) ) prefDelBtn->enable();
        else prefDelBtn->disable();
    }
    else {
        prefClrBtn->disable();
        prefDelBtn->disable();
    }
    return 1;
}


/* Cancella una o pi cartella preferita:
 *  - no comment.
 */
long OptionsDialog::onCmdDelPref(FXObject *sender, FXSelector sel, void *ptr) {
    int item=prefList->getCurrentItem();
    switch( FXSELID(sel) ) {
        case OptionsDialog::ID_PREFDEL:
            if(item>=0) {
                if(FXMessageBox::question(this, MBOX_OK_CANCEL, "ShapeConverter", "Do you really want to delete this bookmark\nfrom the list?")==MBOX_CLICKED_CANCEL) break;
                //Cancella l'elemento selezionato
                prefList->removeItem(item);
                options->prefInputPaths.erase( options->prefInputPaths.begin()+item );
            }
            break;
        case OptionsDialog::ID_PREFCLR:
            if(FXMessageBox::question(this, MBOX_OK_CANCEL, "ShapeConverter", "Do you really want to delete all the bookmarks\nfrom the list?")==MBOX_CLICKED_CANCEL) break;
            //Cancella tutti
            prefList->clearItems();
            options->prefInputPaths.clear();
            break;
    }
    return 1;
}




/** CLASSE OptionsDialog: AGGIORNAMENTO OPZIONI **/

/* Aggiorna le opzioni di MyWindow:
 *  - no comment.
 */
void OptionsDialog::getOptions(Options &options) {
    options.anims=*(FXuchar*)anims.getData();
    options.lods=*(FXuchar*)lods.getData();
    options.log=*(FXuchar*)log.getData();
    options.mat=*(FXuchar*)mat.getData();
    options.out=out;
    options.outPath=outPath;
    if(win::fileExist(ffedPath)==1) {
        options.ffed=ffed;
        options.ffedPath=ffedPath;
    }
    else {
        options.ffed=2;
        options.ffedPath="";
    }
    options.ffedPath=ffedPath;
    options.temp=temp;
    options.tempPath=tempPath;
}
