#include "ProgressDialog.h"


/** CLASSE ProgressDialog: FOX's MACROS **/

/* Macro, message map:
 *  - message type: what appened;
 *  - message id: source;
 *  - message handler: what to do.
 */
FXDEFMAP(ProgressDialog) ProgressDialogMap[]={
   //_________Message_Type_____________________ID__________________________Message_Handler_______________
   //FXMAPFUNC(   SEL_COMMAND,           FXDialogBox::ID_ACCEPT,      ProgressDialog::onCmdCancel      ), //usare questo per chiudere la dialog alla pressione di annulla
   FXMAPFUNC(   SEL_COMMAND,           ProgressDialog::ID_CANCEL,   ProgressDialog::onCmdCancel      )
};


/* Macro, message map implementation:
 *  - 1, the name of the class;
 *  - 2, the name of the parent class;
 *  - 3, pointer to message map;
 *  - 4, no. of messages in message map.
 */
FXIMPLEMENT(ProgressDialog, FXDialogBox, ProgressDialogMap, ARRAYNUMBER(ProgressDialogMap))




/** CLASSE ProgressDialog: COSTRUTTORE E DISTRUTTORE **/

/* This function builds the window:
 *  - 1, pointer to the FXWindow owner;
 *  - 2, the window title;
 *  - 5, window decorations;
 *  - 6, initial position, w;
 *  - 7, initial position, h;
 *  - 8, initial size, w;
 *  - 9, initial size, h.
 */
ProgressDialog::ProgressDialog(FXWindow* owner, FXString title) : FXDialogBox(owner, title, DECOR_TITLE|DECOR_BORDER, 0, 0, 300, 150, 0,0,0,0) {
    corr=0, err=0, tot=0;
    //Here we can build the window
    dialogFrame=new FXVerticalFrame(this, LAYOUT_FILL_X|LAYOUT_FILL_Y|FRAME_NONE,0,0,0,0, 5,5, 5,5, 5,5);
    topLabel=new FXLabel(dialogFrame,"Converting file x of y...",NULL,LAYOUT_CENTER_X|LAYOUT_FILL_X|LAYOUT_FILL_Y);
    midLabel=new FXLabel(dialogFrame,"Ready",NULL,LAYOUT_CENTER_X|LAYOUT_FILL_X|LAYOUT_FILL_Y);
    progressBar=new FXProgressBar(dialogFrame, NULL, 0, PROGRESSBAR_HORIZONTAL|PROGRESSBAR_PERCENTAGE|FRAME_GROOVE|LAYOUT_FILL_X|LAYOUT_FIX_HEIGHT, 0,0,0,30);
    counterStr="%d converted / %d failed / %d total";
    botLabel=new FXLabel(dialogFrame,temp.format(counterStr.text(),corr,err,tot),NULL,LAYOUT_CENTER_X|LAYOUT_FILL_X|LAYOUT_FILL_Y);
    new FXSeparator(dialogFrame);
    cancelBtn=new FXButton(dialogFrame, "Stop", NULL, this, ProgressDialog::ID_CANCEL, BUTTON_NORMAL|LAYOUT_CENTER_X,0,0,0,0,10,10,5,5);
    cancelled=false;


    /// End of ProgressDialog
}

/* Distruttore:
 *  - only MenuPanes, Cursors, Fonts, Images, Icons, Bitmaps, and Visuals.
 *  - see 'http://www.fox-toolkit.com/faq.html#DELETINGSTUFF'.
 */
ProgressDialog::~ProgressDialog() {
    //Nothing to do
}




/** CLASSE ProgressDialog: CALLBACKS functions **/

/* Annulla operazione:
 *  - no comment.
 */
long ProgressDialog::onCmdCancel(FXObject *sender, FXSelector sel, void *ptr) {
    this->getApp()->endWaitCursor();
    if( FXMessageBox::question(this, MBOX_YES_NO, "ShapeConverter", "Do you really want to stop this task?")==MBOX_CLICKED_NO) {
        cancelled=false;
        this->getApp()->beginWaitCursor();
    }
    else {
        cancelled=true;
    }
    return 1;
}




/** CLASSE ProgressDialog: ALTRE FUNZIONI **/

/* Aggiorna la finestra ed eventualmente annulla l'operazione:
 *  - no comment.
 */
bool ProgressDialog::check(void) {
    this->getApp()->runModalWhileEvents(this);
    //L'utente vuole interrompere
    if(cancelled) {
        //L'utente annulla la conversione
        this->hide();
        return false;
    }
    //L'utente non clicca cancel
    return true;
}


/* Mostra i risultati dellla conversione:
 *  - no comment.
 */
void ProgressDialog::showResults(void) {
    counterStr="%d of %d files were converted successfully.\nFor further information, see log files.";
    FXMessageBox::information(this, MBOX_OK, "ShapeConverter", temp.format(counterStr.text(),corr,tot).text());
}




/** CLASSE ProgressDialog: ALTRE FUNZIONI **/

/* Inizializza la barra per la prossima operazione:
 *  - no comment.
 */
void ProgressDialog::startOperation(float _step, char *label, bool restart) {
    //Riparto da zero?
    if(restart) counter=0.0f;
    //C' una stringa da aggiungere?
    if(label!=NULL) {
        midLabel->setText(label);
    }
    //Passo
    step=_step;
    //Controllo se l'utente vuole annullare
    if( !check() ) throw 0; //L'eccezione viene gestita in Convertitore.cpp
}


/* Incrementa di un passo la barra per l'operazione come da inizializzazione:
 *  - no comment.
 */
void ProgressDialog::incrementStep(void) {
    counter+=step;
    progressBar->setProgress( unsigned(round(counter)) );
}


/* Incrementa di n passi la barra per l'operazione come da inizializzazione:
 *  - no comment.
 */
void ProgressDialog::incrementStep(unsigned n) {
    counter+=n*step;
    progressBar->setProgress( unsigned(round(counter)) );
}
