#include "WinUtils.h"


/* Ricerca tutti i files nella cartella:
 *  - path  il percorso della cartella;
 *  - fileList  il puntatore al vettore in cui saranno contenuti i nomi dei files trovati;
 *  - query contiene la stringa di ricerca (ammessi wilcards come * ecc.);
 *  - restituisce il numero di files trovati (>=0) oppure -1 in caso di errore.
 */
int win::findAllFiles(FXString path, std::vector<FXString> *fileList, FXchar query[]) {
    WIN32_FIND_DATA FindFileData;
    HANDLE hFind = INVALID_HANDLE_VALUE;
    DWORD dwError;
    int numFiles=0;

    path.append(query);
    hFind=FindFirstFile( (char*)path.text(), &FindFileData );

    if(hFind == INVALID_HANDLE_VALUE) {
        dwError = GetLastError();
        if(dwError == ERROR_FILE_NOT_FOUND) {
            //Non ci sono file
            return 0;
        }
        else {
            //Errore
            return -1;
        }
    }
    else {
        do {
            fileList->push_back(FindFileData.cFileName);
            numFiles++;
        } while(FindNextFile(hFind, &FindFileData) != 0);
        dwError = GetLastError();
        FindClose(hFind);
        if(dwError != ERROR_NO_MORE_FILES) {
            //Errore
            return -1;
        }
    }
    return numFiles;
}


/* Richiede il percorso delle cartelle speciali di Windows:
 *  - path  il reference alla stringa che conterr la cartella;
 *  - folder specifica quale cartella cercare (vedi enum nel file header).
 */
void win::getDirPath(FXString &path, int folder) {
    TCHAR thePath[MAX_PATH];
    int csidl;
    switch(folder) {
        case DIR_DESKTOP:
            csidl=CSIDL_DESKTOPDIRECTORY;
            break;
        case DIR_DOCUMENTS:
            csidl=CSIDL_PERSONAL;
            break;
        default:
            return;
    }
    SHGetFolderPath( NULL, csidl, NULL, SHGFP_TYPE_CURRENT, thePath );
    path=thePath;
}


/* Controlla se il file esiste. Restituisce:
 *  - 1 se esiste;
 *  - 0 se non esiste;
 *  - -1 in caso di altri errori.
 */
int win::fileExist(FXString fullpath) {
    WIN32_FIND_DATA FindFileData;
    HANDLE hFind = INVALID_HANDLE_VALUE;
    DWORD dwError;

    hFind=FindFirstFile( (char*)fullpath.text(), &FindFileData );
    if(hFind == INVALID_HANDLE_VALUE) {
        dwError = GetLastError();
        if(dwError == ERROR_FILE_NOT_FOUND) {
            //Non esiste il file
            return 0;
        }
        else {
            //Errore
            return -1;
        }
    }
    FindClose(hFind);
    return 1;
}


/* Controlla se la cartella esiste. Restituisce:
 *  - 1 se esiste;
 *  - 0 se non esiste;
 *  - -1 in caso di altri errori.
 */
int win::dirExist(FXString path) {
    WIN32_FIND_DATA FindFileData;
    HANDLE hFind = INVALID_HANDLE_VALUE;
    DWORD dwError;

    hFind=FindFirstFile( (char*)path.text(), &FindFileData );
    if(hFind == INVALID_HANDLE_VALUE) {
        dwError = GetLastError();
        if(dwError == ERROR_PATH_NOT_FOUND) {
            //Non esiste il file
            return 0;
        }
        else {
            //Errore
            return -1;
        }
    }
    FindClose(hFind);
    return 1;
}


/* Cancella un file:
 *  - no comment.
 */
void win::deleteFile(FXString fullpath) {
    DeleteFile(fullpath.text());
}


/* Esegue un file o apre il file passato in param con l'applicazione associata. Restituisce:
 *  - 1 se va a buon fine;
 *  - 0 altrimenti.
 */
int win::execute(FXString filepath, FXString param) {
    int ris=(int)ShellExecute( NULL, "open", filepath.text(), param.text(), NULL, SW_SHOW );
    if(ris>32) return 1;
    else return 0;
}
