#include "CharUtils.h"


/** FUNZIONI DI CONTROLLO DEI CARATTERI **/

/* Verifica se il carattere in ingresso  un terminatore di stringa:
 *  - restituisce vero se lo ;
 *  - altrimenti restituisce falso.
 */
bool isStrTerm(char ch) {
    switch(ch) {
        case ' ':
        case '"':
        case '\n':
        case '\r':
        case '\b':
        case '\t':
            return true;
        default:
            return false;
    }
}


/* Verifica se il carattere in ingresso  un numero:
 *  - restituisce vero se  un numero (0-9);
 *  - altrimenti restituisce falso.
 */
bool isNum(char ch) {
    if(ch>=0x30 && ch<=0x39)
        return true;
    else
        return false;
}


/* Verifica se il carattere in ingresso  una virgola:
 *  - restituisce vero se  una virgola (, o .);
 *  - altrimenti restituisce falso.
 */
bool isDot(char ch) {
    if(ch==0x2C || ch==0x2E)
        return true;
    else
        return false;
}


/* Verifica se il carattere in ingresso  un segno di 10^:
 *  - restituisce vero se  una segno di 10^ (e o E);
 *  - altrimenti restituisce falso.
 */
bool isExp(char ch) {
    if(ch==0x65 || ch==0x45)
        return true;
    else
        return false;
}


/* Verifica se il carattere in ingresso  un carattere di segno:
 *  - restituisce vero se  un carattere di segno (+ or -);
 *  - altrimenti restituisce falso.
 */
bool isSign(char ch) {
    if(ch==0x2B || ch==0x2D)
        return true;
    else
        return false;
}


/* Verifica se il carattere in ingresso  un carattere di formattazione:
 *  - restituisce vero se lo ;
 *  - altrimenti restituisce falso.
 */
bool isForm(char ch) {
    switch(ch) {
        case ' ':
        case '\n':
        case '\r':
        case '\t':
            return true;
        default:
            return false;
    }
}



/** FUNZIONI DI CONVERSIONE STR=>NUM **/

/* Trasforma la stringa in ingresso in un intero signed 32bit:
 *  - salva il segno;
 *  - controllo lunghezza numero: lancia eccezione se il numero supera i limiti.
 */
/* vedi file header */


/* Trasforma la stringa in ingresso in un float:
 *  - salva il segno;
 *  - controllo lunghezza numero (max 32bit float => 6 cifre di precisione);
 *  - arrotonda e/o tronca automaticamente in caso di superamento limiti;
 *  - salva 0 in caso di errori.
 */
void string2float(string mantInt, string mantDec, string exp, char segnoMant, char segnoExp, f32 *num) {
    u8 lung;
    s16 esponente=0;
    char ch;
    f32 temp, ris=0;
    f32 powPos[]={1,1e+1,1e+2,1e+3,1e+4,1e+5,1e+6,1e+7,1e+8,1e+9,1e+10};
    f32 powNeg[]={1,1e-1,1e-2,1e-3,1e-4,1e-5,1e-6,1e-7,1e-8,1e-9,1e-10};
    //Mantissa - Parte intera
    lung=mantInt.length();
    if(lung>6) {
        //Il numero va oltre i limiti (6 cifre di precisione)
        lung=6;
    }
    for(u8 i=1; i<=lung; i++) { //i=1 => ultimo ch della str => pow=1
        ch=mantInt[lung-i];
        temp=((s16)ch-0x30)*powPos[i-1];
        ris+=temp;
    }
    //Mantissa - Parte decimale
    lung=mantDec.length();
    if(lung>6) {
        //Il numero va oltre i limiti (6 cifre di precisione)
        lung=6;
    }
    for(u8 i=lung; i>=1; i--) { //i=lung => primo ch della str => pow=0.1
        ch=mantDec[lung-i];
        temp=((s16)ch-0x30)*powNeg[lung-i+1];
        ris+=temp;
    }
    //Esponente
    string2int(exp, '+', &esponente); //Lo converte in intero (positivo per comodit)
    s16 decine=esponente/10; //Numero di moltiplicazioni per 1e10
    s16 unita=esponente%10; //Esponente minore di 10
    if(segnoExp=='-') {
        for(u8 i=0; i<decine; i++) {
            //Moltiplica decine-volte ris per 1e-10
            ris*=1e-10;
        }
    }
    else {
        for(u8 i=0; i<decine; i++) {
            //Moltiplica decine-volte ris per 1e+10
            ris*=1e+10;
        }
    }
    if(segnoExp=='-') {
        ris*=powNeg[unita];
    }
    else {
        ris*=powPos[unita];
    }
    //Cambia il segno se negativo
    if(segnoMant=='-') {
        ris*=-1;
    }
    //Salva il risultato all'indirizzo fornito
    *num=ris;
}



/** FUNZIONI PER CONTROLLO DIMENSIONE NUMERI **/

/* Verifica se il numero eccede i limiti dell's32 o dell'u32:
 *  - sign indica se il numero  signed ( <0) o unsigned ( >=0);
 *  - lancia un'eccezione se il numero supera +/-2,147,483,647 oppure +4,294,967,295.
 */
void checkIntLenght(string num, s8 sign) {
    u8 lung;
    char cifreS[]={2,1,4,7,4,8,3,6,4,7};
    char cifreU[]={4,2,9,4,9,6,7,2,9,5};
    char *cifre;

    //Determina se deve lavorare con un signed o un unsigned
    if(sign<0) {
        //Intero signed 32bit
        cifre=cifreS;
    }
    else {
        //Intero unsgned 32bit
        cifre=cifreU;
    }

    //Controllo della lunghezza della stringa
    lung=num.length();
    if(lung>10) {
        //Il numero va sicuramente oltre i limiti dell'intero signed 16bit
        throw Errore(ERR_OP_INT_READ, ERR_TP_EXCEED_UNITS);
    }
    else if(lung<10) {
        //Il numero  sicuramente minore dei limiti
        return;
    }

    //Se lung==10 occorre controllare le cifre
    for(u8 i=0; i<lung; i++) {
        if(num[i]>cifre[i]) {
            //Il numero eccede sicuramente i limiti!
            throw Errore(ERR_OP_INT_READ, ERR_TP_EXCEED_UNITS);
        }
        else if(num[i]<cifre[i]) {
            //Il numero non eccede sicuramente i limiti!
            return;
        }
        //Se num[i]==cifre[i] occorre controllare la cifra sucessiva
    }
    //Se arriva qui, significa che ha 10 cifre ed  uguale al limite
    return;
}




/** FUNZIONI DI CONVERSIONE NUM=>STR **/

/* Trasforma il numero intero in ingresso in una stringa:
 *  - no comment.
 */
string int2string(u8 num, bool noCentinaia) {
    //La stringa contenente il numero
    string str;
    u8 centinaia=num/100;
    u8 decine=(num%100)/10;
    u8 unita=(num%10);
    if(!noCentinaia) str+='0'+centinaia;
    str+='0'+decine;
    str+='0'+unita;
    return str;
}

