#include "File3ds.h"


/** CLASSE File3ds: COSTRUTTORE & DISTRUTTORE **/

/* Costruttore:
 * - inizializza le variabili.
 */
File3ds::File3ds(Mesh &_mesh, ostream &_log, bool _logFile) : mesh(_mesh), log(_log) {
    writedBytes=0;
    logFile=_logFile;
}


/* Distruttore:
 * - dealloca gli array contenenti la lunghezza del file
 */
File3ds::~File3ds(void) {
    if(lungSezMats!=NULL) delete [] lungSezMats;
    if(lungSezObjs!=NULL) delete [] lungSezObjs;
    if(lungSezAnims!=NULL) delete [] lungSezAnims;
}



/** CLASSE File3ds: APERTURA E CHIUSURA FILE **/

/* Apre un file:
 *  - in caso di errore restituisce false.
 */
void File3ds::open(const char filename[]) {
    file.open(filename, ios::binary);
    if(!file) {
        Errore err(FILE_ST_CLOSED, ERR_FO_OPEN_FAIL);
        err.setErrClass(ERR_CL_3DS);
        err.setErrOper(ERR_OP_FILE_OPENING);
        throw err;
    }
    writedBytes=0;
}


/* Chiude il file aperto:
 */
void File3ds::close(void) {
    file.close();
}



/** CLASSE File3ds: CALCOLO DIMENSIONI SEZIONI DEL FILE **/

/* Calcolo della lunghezza delle sezioni del file:
 *  - come descritto nei commenti delle variabili private.
 */
void File3ds::sizeCalc(void) {
    //Allocazione array
    u8 numMats=mesh.getNumMats();
    u8 numObjs=mesh.getNumObjs();
    try {
        lungSezMats=new u32[numMats*3];
        lungSezObjs=new u32[numObjs*6];
        lungSezAnims=new u32[numObjs*4];
    }
    catch(bad_alloc xa) {
        //Errore di allocazione
        throw Errore(ERR_CL_3DS, ERR_OP_SIZE_CALC, ERR_TP_BAD_ALLOC);
    }

    //Sezione materiali
    for(u8 i=0; i<numMats; i++) {
        //Indirizzo dell'i-esimo materiale
        Material *oneMat=mesh.getMat(i);
        //MAT_NAME - colonna 0
        u8 lungName=oneMat->getMatName().length();
        lungSezMats[3*i+0]=6+lungName+1; //= header + stringa + '\0'
        //MAT_TEXNAME - colonna 1
        lungName=oneMat->getImageName().length();
        lungSezMats[3*i+1]=6+8+6+lungName+1+8; //= header + INT_PERCENTAGE + (header + stringa + '\0') + MAT_MAP_TILING
        //TOT=MAT_ENTRY - colonna 2
        //= header + MAT_NAME + MAT_AMBIENT + MAT_DIFFUSE + MAT_SPECULAR + MAT_SHININESS + MAT_TRANSPARENCY + MAT_SHADING + MAT_TEXMAP
        lungSezMats[3*i+2]=6+lungSezMats[3*i+0]+24*3+14*3+8+lungSezMats[3*i+1];
    }

    //Sezione oggetti
    for(u8 i=0; i<numObjs; i++) {
        //Indirizzo dell'i-esimo oggetto
        Object *oneObj=mesh.getObj(i);
        //POINT_ARRAY - colonna 0
        u16 numPoints=oneObj->getNumPoints();
        lungSezObjs[6*i+0]=6+2+numPoints*12; //= header + uShort + n * (3 float)
        //TEX_VERTS - colonna 1
        u16 numUvPoints=oneObj->getNumUvPoints();
        lungSezObjs[6*i+1]=6+2+numUvPoints*8; //= header + uShort + n * (2 float)
        //FACE_ARRAY - colonna 2
        u8 numSubObjs=oneObj->getNumSubObjs();
        u16 numFaces=0;
        for(u8 j=0; j<numSubObjs; j++) {
            //Indirizzo del j-esimo subobject dell'i-esimo oggetto
            SubObject *oneSO=oneObj->getSubObj(j);
            //Numero complessivo di facce dell'oggetto
            numFaces+=oneSO->getNumFaces();
        }
        //Lunghezza chunk
        lungSezObjs[6*i+2]=6+2+numFaces*8; //= header + uShort + n * (4 uShort)
        //MSH_MAT_GROUP totale - colonna 3
        u32 lung=0;
        for(u8 j=0; j<numSubObjs; j++) {
            //Indirizzo del j-esimo subobject dell'i-esimo oggetto
            SubObject *oneSO=oneObj->getSubObj(j);
            //Numero di facce del subobject
            numFaces=oneSO->getNumFaces();
            //Indirizzo del materiale applicato al subobject
            Material *oneMat=mesh.getMat(oneSO->getNumMat());
            //Lunghezza del nome del materiale applicato
            u8 lungName=oneMat->getMatName().length();
            //Lunghezza chunk
            lung+=6+lungName+1+2+numFaces*2; //= header + uShort + (stringa + '\0') + n * uShort
        }
        //Lunghezza totale gruppo chunks MSH_MAT_GROUPs
        lungSezObjs[6*i+3]=lung; //= n volte il chunk MSH_MAT_GRUOP //=0; per eliminarlo
        //N_TRL_OBJECT - colonna 4
        //= header + POINT_ARRAY + TEX_VERTS + FACE_ARRAY + n * MSH_MAT_GROUP + MESH_MATRIX + MESH_COLOR
        lungSezObjs[6*i+4]=6+lungSezObjs[6*i+0]+lungSezObjs[6*i+1]+lungSezObjs[6*i+2]+lungSezObjs[6*i+3]+54+7;
        //TOT=NAMED_OBJECT - colonna 5
        u8 lungName=oneObj->getObjName().length();
        lungSezObjs[6*i+5]=6+lungName+1+lungSezObjs[6*i+4]; //= header + (stringa + '\0') + N_TRL_OBJECT
    }

    //Sezione animazioni
    for(u8 i=0; i<numObjs; i++) {
        //Indirizzo dell'i-esimo oggetto
        Object *oneObj=mesh.getObj(i);
        //Lunghezza nome oggetto
        u8 lungName=oneObj->getObjName().length();
        //Si definiscono: trackheader = uShort + uLong + uLong + uLong = 14
        //                keyheader = uLong + uShort = 6
        //NODE_HDR - colonna 0
        lungSezAnims[4*i+0]=6+lungName+1+2*3; //= header + (stringa + '\0') + uShort + uShort + uShort
        //POS_TRACK_TAG - colonna 1
        //Numero linKeys
        u8 numLinKeys=oneObj->getNumLinKeys();
        if(numLinKeys==0) lungSezAnims[4*i+1]=0;
        else lungSezAnims[4*i+1]=6+14+numLinKeys*(6+12); //= header + trackheader + n * (keyheader + 3*float)
        //ROT_TRACK_TAG - colonna 2
        //Numero linKeys
        u8 numRotKeys=oneObj->getNumRotKeys();
        if(numRotKeys==0) lungSezAnims[4*i+2]=0;
        else lungSezAnims[4*i+2]=6+14+numRotKeys*(6+16); //= header + trackheader + n * (keyheader + 4*float)
        //OBJECT_NODE_TAG - colonna 3
        // = header + NODE_ID + NODE_HDR + PIVOT + POS_TRACK_TAG + ROT_TRACK_TAG + SCL_TRACK_TAG
        lungSezAnims[4*i+3]=6+8+lungSezAnims[4*i+0]+18+lungSezAnims[4*i+1]+lungSezAnims[4*i+2]+(6+14+6+12);
    }


    //Lunghezza totale del file (M3DMAGIC)
    //= chunkHeader + M3D_VERSION + MDATA + KFDATA;
    //   --> MDATA = chunkHeader + MESH_VERSION + MASTER_SCALE + n * MAT_ENTRY + n * NAMED_OBJECT =
    //             = chunkHeader + MESH_VERSION + MASTER_SCALE + lungMatSect + lungObjSect =
    //             = lungTotMDATA
    //   --> KFDATA = chunkHeader + KFHDR + KFSEG + KFCURTIME + n * OBJECT_NODE_TAG =
    //              = chunkHeader + KFHDR + KFSEG + KFCURTIME + lungKeySect =
    //              = longTotKFDATA

    //Lunghezza sezione materiali (MAT_ENTRYs)
    u32 lungMatSect=0;
    for(u8 i=0; i<numMats; i++) {
        lungMatSect+=lungSezMats[3*i+2]; //colonna 2
    }
    //cout<<"Lunghezza attesa sez mat: "<<lungMatSect<<"bytes"<<endl;

    //Lunghezza sezione materiali (NAMED_OBJECTs)
    u32 lungObjSect=0;
    for(u8 i=0; i<numObjs; i++) {
        lungObjSect+=lungSezObjs[6*i+5]; //colonna 5
    }
    //cout<<"Lunghezza attesa sez ogg: "<<lungObjSect<<"bytes"<<endl;

    //Lunghezza sezione keyframes (OBJECT_NODE_TAGs)
    u32 lungKeySect=0;
    for(u8 i=0; i<numObjs; i++) {
        lungKeySect+=lungSezAnims[4*i+3]; //colonna 3
    }
    //cout<<"Lunghezza attesa sez key: "<<lungKeySect<<"bytes"<<endl;

    //Lunghezza chunk MDATA
    lungTotMDATA=6+10+10+lungMatSect+lungObjSect;
    //Lunghezza chunk KFDATA
    lungTotKFDATA=6+21+14+10+lungKeySect;
    //Lunghezza totale file 3ds
    lungTotFile=6+10+lungTotMDATA+lungTotKFDATA;
    cout<<endl<<"Lunghezza attesa del file: "<<lungTotFile<<"bytes"<<endl;
}



/** CLASSE File3ds: SCRITTURA FILE **/

/* Scrive la parte introduttiva del file 3ds:
 *  - header del chunk M3DMAGIC;
 *  - chunk M3D_VERSION;
 *  - header del chunk MDATA;
 *  - chunk MESH_VERSION;
 *  - chunk MASTER_SCALE.
 */
void File3ds::scriviTestata(void) {
    //M3DMAGIC
    scriviHead(0x4D4D, lungTotFile);
    //M3D_VERSION
    scriviHead(0x0002, 10);
    scriviUL(0x00000003);
    //MDATA
    scriviHead(0x3D3D, lungTotMDATA);
    //+ MESH_VERSION
        scriviHead(0x3D3E, 10);
        scriviUL(0x00000003);
    //+ MASTER_SCALE
        scriviHead(0x0100, 10);
        scriviF32(1.0f);
    //cout<<"Bytes scritti - fine testata: "<<writedBytes<<endl;
}


/* Scrive la parte del file 3ds relativa ai materiali (parte del chunk MDATA):
 *  - chunk MAT_ENTRY e tutti i suoi figli per tutti i materiali.
 */
void File3ds::scriviMateriali(void) {
    //Numero di materiali da scrivere
    u8 numMats=mesh.getNumMats();

    //Un chunk MAT_ENTRY (incluso in MDATA) per ogni materiale
    for(u8 i=0; i<numMats; i++) {
        //Indirizzo dell'i-esimo materiale
        Material *oneMat=mesh.getMat(i);
        //MAT_ENTRY
        scriviHead(0xAFFF, lungSezMats[3*i+2]);
        //+ MAT_NAME
            scriviHead(0xA000, lungSezMats[3*i+0]);
            scriviString(oneMat->getMatName());
        //+ MAT_AMBIENT
            scriviHead(0xA010, 24);
            //+ COLOR_24
                scriviHead(0x0011, 9);
                scriviByte( oneMat->getColorR() );
                scriviByte( oneMat->getColorG() );
                scriviByte( oneMat->getColorB() );
            //+ LIN_COLOR_24
                scriviHead(0x0012, 9);
                scriviByte( oneMat->getColorR() );
                scriviByte( oneMat->getColorG() );
                scriviByte( oneMat->getColorB() );
        //+ MAT_DIFFUSE
            scriviHead(0xA020, 24);
            //+ COLOR_24
                scriviHead(0x0011, 9);
                scriviByte( oneMat->getColorR() );
                scriviByte( oneMat->getColorG() );
                scriviByte( oneMat->getColorB() );
            //+ LIN_COLOR_24
                scriviHead(0x0012, 9);
                scriviByte( oneMat->getColorR() );
                scriviByte( oneMat->getColorG() );
                scriviByte( oneMat->getColorB() );
        //+ MAT_SPECULAR
            scriviHead(0xA030, 24);
            //+ COLOR_24
                scriviHead(0x0011, 9);
                scriviByte(255);
                scriviByte(255);
                scriviByte(255);
            //+ LIN_COLOR_24
                scriviHead(0x0012, 9);
                scriviByte(255);
                scriviByte(255);
                scriviByte(255);
        //+ MAT_SHININESS
            scriviHead(0xA040, 14);
            //+ INT_PERCENTAGE
                scriviHead(0x0030, 8);
                scriviUS(100);
        //+ MAT_SHIN2PCT
            scriviHead(0xA041, 14);
            //+ INT_PERCENTAGE
                scriviHead(0x0030, 8);
                scriviUS(100);
        //+ MAT_TRANSPARENCY
            scriviHead(0xA050, 14);
            //+ INT_PERCENTAGE
                scriviHead(0x0030, 8);
                scriviUS(0);
        //+ MAT_SHADING
            scriviHead(0xA100, 8);
            scriviUS(1);
        //+ MAT_TEXMAP
            scriviHead(0xA200, lungSezMats[3*i+1]);
            //+ INT_PERCENTAGE
                scriviHead(0x0030, 8);
                scriviUS(100);
            //+ MAT_MAPNAME
                scriviHead(0xA300, lungSezMats[3*i+1]-8-8-6);
                scriviString(oneMat->getImageName());
            //+ MAT_MAP_TILING
                scriviHead(0xA351, 8);
                scriviUS(0x40);
        //- end of chunk MAT_ENTRY
    }
    //cout<<"Bytes scritti - fine sez materiali: "<<writedBytes<<endl;
}


/* Scrive la parte del file 3ds relativa agli oggetti (parte del chunk MDATA):
 *  - chunk NAMED_OBJECT e tutti i suoi figli per tutti i materiali.
 */
void File3ds::scriviOggetti(void) {
    //Numero di oggetti da scrivere
    u8 numObjs=mesh.getNumObjs();
    //cout<<"numero di oggetti: "<<(u16)numObjs<<endl;

    //Un chunk NAMED_OBJECT (incluso in MDATA) per ogni materiale
    for(u8 i=0; i<numObjs; i++) {
        //Indirizzo dell'i-esimo oggetto
        Object *oneObj=mesh.getObj(i);
        //NAMED_OBJECT
        scriviHead(0x4000, lungSezObjs[6*i+5]);
        scriviString(oneObj->getObjName());
        //cout<<"\tnome oggetto: "<<oneObj->getObjName()<<endl;
        //+ N_TRL_OBJECT
            scriviHead(0x4100, lungSezObjs[6*i+4]);
            //+ POINT_ARRAY
                scriviHead(0x4110, lungSezObjs[6*i+0]);
                u16 numPoints=oneObj->getNumPoints();
                scriviUS(numPoints);
                //cout<<"\t\t=> numero di punti: "<<numPoints<<endl;
                for(u16 j=0; j<numPoints; j++) {
                    //Trasformazione in coordinate globali
                    // NB: va fatto qui e non mentre le salva perch altrimenti risulta impossibile
                    // controllare le normali (la normale non pu essere trasformata con la stessa
                    // matrice dei lati: perch?
                    Vett3<f32> point;
                    trasfVett(oneObj->getAssMatr(), oneObj->getPoint(j), &point);
                    //Scrive un punto
                    scriviVett3(&point); //Va rototraslato!
                }
            //+ TEX_VERTS
                scriviHead(0x4140, lungSezObjs[6*i+1]);
                u16 numUvPoints=oneObj->getNumUvPoints();
                scriviUS(numUvPoints);
                //cout<<"\t\t=> numero di punti uv: "<<numPoints<<endl;
                for(u16 j=0; j<numUvPoints; j++) {
                    //Scrive un punto
                    scriviVett2(oneObj->getUvPoint(j));
                }
            //+ FACE_ARRAY
                scriviHead(0x4120, lungSezObjs[6*i+2]+lungSezObjs[6*i+3]);
                u8 numSubObjs=oneObj->getNumSubObjs();
                u16 numFaces=0;
                for(u8 j=0; j<numSubObjs; j++) {
                    //Indirizzo del j-esimo subobject dell'i-esimo oggetto
                    SubObject *oneSO=oneObj->getSubObj(j);
                    //Numero complessivo di facce dell'oggetto
                    numFaces+=oneSO->getNumFaces();
                }
                //Scrive il numero di facce dell'oggetto
                scriviUS(numFaces);
                //cout<<"\t\t=> numero di subobjects: "<<(u16)numSubObjs<<endl;
                //cout<<"\t\t=> numero di facce: "<<numFaces<<endl;
                for(u8 j=0; j<numSubObjs; j++) {
                    //Indirizzo del j-esimo subobject dell'i-esimo oggetto
                    SubObject *oneSO=oneObj->getSubObj(j);
                    //Numero di facce del subobject
                    u16 numFacesSO=oneSO->getNumFaces();
                    for(u16 k=0; k<numFacesSO; k++) {
                        //Scrive una faccia
                        scriviUS(oneSO->getVertIdx(k, 1));
                        scriviUS(oneSO->getVertIdx(k, 2));
                        scriviUS(oneSO->getVertIdx(k, 3));
                        scriviUS(0x0007);
                    }
                }
                //+ MSH_MAT_GROUP (figlio di FACE_ARRAY)
                    //Un chunk per ogni subobject
                    u16 contaFaces=0;
                    for(u8 j=0; j<numSubObjs; j++) {
                        //Indirizzo del j-esimo subobject dell'i-esimo oggetto
                        SubObject *oneSO=oneObj->getSubObj(j);
                        //Indirizzo del materiale applicato al subobject
                        Material *oneMat=mesh.getMat(oneSO->getNumMat());
                        //Numero di facce del subobject
                        u16 numFacesSO=oneSO->getNumFaces();
                        //Testata del chunk
                        scriviHead(0x4130, 6+oneMat->getMatName().length()+1+2+numFacesSO*2);
                        //Scrive il nome del materiale
                        scriviString(oneMat->getMatName());
                        //Scrive il numero di facce del subobject
                        scriviUS(numFacesSO);
                        //Contatore facce subobject precedenti
                        for(u16 k=0; k<numFacesSO; k++) {
                            //Scrive una faccia
                            //NB: le facce scritte in FACE_ARRAY sono gi ordinate per materiale!
                            scriviUS(contaFaces+k);
                        }
                        //Aggiorna il contatore per il prossimo subobject
                        contaFaces+=numFacesSO;
                        //cin>>maxIdx;
                    }
            //- end of chunk FACE ARRAY
            //+ MESH_MATRIX
                scriviHead(0x4160, 54);
                scriviMatrPiv(oneObj->getAssMatr());
            //+ MESH_COLOR
                scriviHead(0x4165, 7);
                scriviByte(i);
        //- end of chunk N_TRL_OBJECT
        //- end of chunk NAMED_OBJECT
    }
    //cout<<"Bytes scritti - fine sez oggetti: "<<writedBytes<<endl;
}

/* Scrive la parte del file 3ds relativa alle animazioni:
 *  - chunk KFDATA e tutti i suoi figli.
 */
void File3ds::scriviAnimazioni(void) {
    //KFDATA
    scriviHead(0xB000, lungTotKFDATA);
        //+ KFHDR
            scriviHead(0xB00A, 21);
            scriviUS(0x0005);
            scriviString("MAXSCENE");
            scriviUL(8);
        //+ KFSEG
            scriviHead(0xB008, 14);
            scriviUL(0);
            if(mesh.getMaxKeyNum()>7) scriviUL(mesh.getMaxKeyNum()+1);
            else scriviUL(8);
        //+ KFCURTIME
            scriviHead(0xB009, 10);
            scriviUL(0);

    //Un chunk OBJECT_NODE_TAG (0xB002) per ogni oggetto
    //Numero di oggetti
    u8 numObjs=mesh.getNumObjs();
    for(u8 i=0; i<numObjs; i++) {
         //Indirizzo dell'i-esimo oggetto
        Object *oneObj=mesh.getObj(i);
        //+ OBJECT_NODE_TAG
            scriviHead(0xB002, lungSezAnims[4*i+3]);
            //+ NODE_ID
                scriviHead(0xB030, 8);
                scriviUS((u16)i);
            //+ NODE_HDR
                scriviHead(0xB010, lungSezAnims[4*i+0]);
                scriviString(oneObj->getObjName());
                scriviUS(0x4000);
                scriviUS(0);
                scriviSS(oneObj->getParent());
            //+ PIVOT
                scriviHead(0xB013, 18);
                scriviF32(0.0f);
                scriviF32(0.0f);
                scriviF32(0.0f);
        //Numero di linKeys dell'i-esimo oggetto
        u8 numLinKeys=oneObj->getNumLinKeys();
        if(numLinKeys!=0) {
            //+ POS_TRACK_TAG (una sola volta per ogni oggetto, se ha linKeys)
                scriviHead(0xB020, lungSezAnims[4*i+1]);
                //Trackheader
                scriviUS(0x0000);
                scriviUL(0x0000);
                scriviUL(0x0000);
                scriviUL(numLinKeys);
            for(u8 j=0; j<numLinKeys; j++) {
                //Indirizzo del j-esimo linKey dell'i-esimo oggetto
                LinKey *oneLinKey=oneObj->getLinKey(j);
                //Per ogni linKey dell'i-esimo oggetto scrive un frame
                //Keyheader
                scriviUL(oneLinKey->getNumKey());
                scriviUS(0x0000);
                //LinKey
                scriviF32(oneLinKey->getX());
                scriviF32(oneLinKey->getY());
                scriviF32(oneLinKey->getZ());
            }
        }
        //Numero di rotKeys dell'i-esimo oggetto
        u8 numRotKeys=oneObj->getNumRotKeys();
        if(numRotKeys!=0) {
            //+ ROT_TRACK_TAG (una sola volta per ogni oggetto, se ha rotKeys)
                scriviHead(0xB021, lungSezAnims[4*i+2]);
                //Trackheader
                scriviUS(0x0000);
                scriviUL(0x0000);
                scriviUL(0x0000);
                scriviUL(numRotKeys);
            for(u8 j=0; j<numRotKeys; j++) {
                //Indirizzo del j-esimo rotKey dell'i-esimo oggetto
                RotKey *oneRotKey=oneObj->getRotKey(j);
                //Trasforma il quaternione in eaa come richiesto dal formato 3ds
                Vett4<f32> eaa;
                quat2eaa(oneRotKey, &eaa);
                //cout<<"RK no."<<(u16)j<<endl;
                //cout<<'\t'<<eaa.getX()<<'\t'<<eaa.getY()<<'\t'<<eaa.getZ()<<'\t'<<eaa.getW()<<endl;
                //Per ogni linKey dell'i-esimo oggetto scrive un frame
                //Keyheader
                scriviUL(oneRotKey->getNumKey());
                scriviUS(0x0000);
                //RotKey
                /*scriviF32(oneRotKey->getW()); //Angolo
                scriviF32(oneRotKey->getX());
                scriviF32(oneRotKey->getY());
                scriviF32(oneRotKey->getZ());*/
                scriviF32( - eaa.getW() ); //Angolo (con segno opposto!)
                scriviF32( eaa.getX() );
                scriviF32( eaa.getY() );
                scriviF32( eaa.getZ() );
            }
        }
        //Un key di scala per posizionare correttamente gli oggetti
        //+ SCL_TRACK_TAG (una sola volta per ogni oggetto, un solo keyframe)
                scriviHead(0xB022, 6+14+6+12);
                //Trackheader
                scriviUS(0x0000);
                scriviUL(0x0000);
                scriviUL(0x0000);
                scriviUL(1); //1 solo kf
                //Keyheader
                scriviUL(0); //kf no.0
                scriviUS(0x0000);
                //ScaleKey
                //Deve essere specchiato solo su y, oltre alla rotazione di 90 su x
                scriviF32( 1.0f); //X
                if(i==0) scriviF32(-1.0f); //Y, specchio solo sull'oggetto root
                else scriviF32( 1.0f); //Y
                scriviF32( 1.0f); //Z
    }
    //cout<<"Bytes scritti - fine testata: "<<writedBytes<<endl;
}



/** CLASSE File3ds: SCRITTURA DATI ELEMENTARI --- PRIVATO **/

/* Funzioni base per la scrittura dei dati sul file binario:
 *  - 1 byte.
 */
void File3ds::scriviByte(u8 out) {
    //Scrive un byte sul file 3ds
    file.put(out);
    //Aggiorna il contatore
    writedBytes++;
}


/* Funzioni base per la scrittura dei dati sul file binario:
 *  - Interi.
 */
void File3ds::scriviUS(u16 out) {
    //Scrive un intero senza segno da 16bit (=2B) sul file 3ds
    u8 buf[2];
    buf[0]=out>>8;
    buf[1]=out;
    file.put(buf[1]);
    file.put(buf[0]);
    //Aggiorna il contatore
    writedBytes+=2;
}

void File3ds::scriviUL(u32 out) {
    //Scrive un intero senza segno da 32bit (=4B) sul file 3ds
    u8 buf[4];
    buf[0]=out>>24;
    buf[1]=out>>16;
    buf[2]=out>>8;
    buf[3]=out;
    file.put(buf[3]);
    file.put(buf[2]);
    file.put(buf[1]);
    file.put(buf[0]);
    //Aggiorna il contatore
    writedBytes+=4;
}

void File3ds::scriviSS(s16 out) {
    //Scrive un intero con segno da 16bit (=2B) sul file 3ds
    u8 buf[2];
    buf[0]=out>>8;
    buf[1]=out;
    file.put(buf[1]);
    file.put(buf[0]);
    //Aggiorna il contatore
    writedBytes+=2;
}

void File3ds::scriviSL(s32 out) {
    //Scrive un intero con segno da 32bit (=4B) sul file 3ds
    u8 buf[4];
    buf[0]=out>>24;
    buf[1]=out>>16;
    buf[2]=out>>8;
    buf[3]=out;
    file.put(buf[3]);
    file.put(buf[2]);
    file.put(buf[1]);
    file.put(buf[0]);
    //Aggiorna il contatore
    writedBytes+=4;
}


/* Funzioni base per la scrittura dei dati sul file binario:
 *  - Float.
 */
void File3ds::scriviF32(f32 out) {
    //Scrive un float (=4B) sul file 3ds (codifica IEEE standard)
    file.write((const char*)&out, sizeof(f32));
    //Aggiorna il contatore
    writedBytes+=4;
}


/* Funzioni base per la scrittura dei dati sul file binario:
 *  - Vettori e matrici.
 */
void File3ds::scriviVett3(Vett3<f32> *out) {
    //Scrive un vettore con 3 componenti f32 sul file 3ds
    f32 temp[3];
    temp[0]=out->getX();
    temp[1]=out->getY();
    temp[2]=out->getZ();
    file.write((const char*) temp, 3*sizeof(f32));
    //file.write((const char*) &(temp), sizeof(f32));
    //file.write((const char*) &(temp), sizeof(f32));
    writedBytes+=12; //4*3;
}

void File3ds::scriviVett2(Vett2<f32> *out) {
    //Scrive un vettore con 2 componenti f32 sul file 3ds
    f32 temp[2];
    temp[0]=out->getX();
    temp[1]=out->getY();
    file.write((const char*) temp, 2*sizeof(f32));
    //file.write((const char*)&(out->getY()), sizeof(f32));
    //Aggiorna il contatore
    writedBytes+=8; //4*2;
}

void File3ds::scriviMatrPiv(Matr4x4<f32> *out) {
    //Scrive una matrice 4x3 con componenti f32 sul file 3ds
    //Trascura l'ultima colonna
    //NB: deve essere scritta per righe
    //Prima riga
    f32 temp[3];
    temp[0]=out->get11();
    temp[1]=out->get12();
    temp[2]=out->get13();
    file.write((const char*) temp, 3*sizeof(f32));
    //Seconda riga
    temp[0]=out->get21();
    temp[1]=out->get22();
    temp[2]=out->get23();
    file.write((const char*) temp, 3*sizeof(f32));
    //Terza riga
    temp[0]=out->get31();
    temp[1]=out->get32();
    temp[2]=out->get33();
    file.write((const char*) temp, 3*sizeof(f32));
    //Quarta riga
    temp[0]=out->get41();
    temp[1]=out->get42();
    temp[2]=out->get43();
    file.write((const char*) temp, 3*sizeof(f32));
    /*//Prima riga
    file.write((const char*)&(out->get11()), sizeof(f32));
    file.write((const char*)&(out->get12()), sizeof(f32));
    file.write((const char*)&(out->get13()), sizeof(f32));
    //Seconda riga
    file.write((const char*)&(out->get21()), sizeof(f32));
    file.write((const char*)&(out->get22()), sizeof(f32));
    file.write((const char*)&(out->get23()), sizeof(f32));
    //Terza riga
    file.write((const char*)&(out->get31()), sizeof(f32));
    file.write((const char*)&(out->get32()), sizeof(f32));
    file.write((const char*)&(out->get33()), sizeof(f32));
    //Quarta riga
    file.write((const char*)&(out->get41()), sizeof(f32));
    file.write((const char*)&(out->get42()), sizeof(f32));
    file.write((const char*)&(out->get43()), sizeof(f32));*/
    //Aggiorna il contatore
    writedBytes+=48; //4*12;
}


/* Funzioni base per la scrittura dei dati sul file binario:
 *  - Stringhe.
 */
void File3ds::scriviString(string out) {
    //Scrive una stringa di lunghezza <=max sul file 3ds
    //NB: aggiunge il carattere nullo al termine!
    //Per la scrittura di caratteri si possono usare gli operatori standard dello stream
    file<<out;
    file<<'\0';
    writedBytes+=out.length()+1;
}


/* Funzioni base per la scrittura dei dati sul file binario:
 *  - Testa del chunk;
 *  - lunghezza comprende la lunghezza dell'header!
 */
void File3ds::scriviHead(u16 head, u32 lunghezza) {
    //Scrive la testata del chunk sul file 3ds
    scriviUS(head);
    scriviUL(lunghezza);
}
