#ifndef FILE3DS_H_INCLUDED
#define FILE3DS_H_INCLUDED

#include <iostream>
#include <fstream>
#include "Tipi.h"
#include "Errore.h"
#include "Vettori.cpp" //Per evitare errore 'undefined reference' includere il cpp invece dell'h
#include "Mesh.h"


using namespace std;

/* DEFINIZIONE DEI TIPI
byte => u8
unsigned short int => u16
unsigned long int => u32
short int => s16
long int => s32
*/


/** CLASSE PER SCRIVERE FILE .3ds **/

/* Questa classe contiene le funzioni per:
 *  - scrivere un file binario;
 *  - scrivere i chunks del file 3ds a partire da un oggetto 'mesh' ottenuto convertendo il file shape.
 */
class File3ds {
    public:
        //Costruttore e distruttore
        File3ds(Mesh &_mesh, ostream &_log, bool _logFile);
        ~File3ds(void);
        //Apre e chiude il file
        void open(const char filename[]);
        void close(void);
        //Calcolo lunghezza sezioni del file
        void sizeCalc(void);
        //Scrittura file
        void scriviTestata(void);
        void scriviMateriali(void);
        void scriviOggetti(void);
        void scriviAnimazioni(void);

    private:
        //Scrittura dati elementari
        void scriviByte(u8 out);
        void scriviUS(u16 out);
        void scriviUL(u32 out);
        void scriviSS(s16 out);
        void scriviSL(s32 out);
        void scriviF32(f32 out);
        void scriviVett3(Vett3<f32> *out);
        void scriviVett2(Vett2<f32> *out);
        void scriviMatrPiv(Matr4x4<f32> *out);
        void scriviString(string out);
        void scriviHead(u16 head, u32 lunghezza);
        //Reference all'oggetto mesh
        Mesh &mesh;
        //Stream per log
        ostream &log;
        bool logFile;
        //Stream per la scrittura del file
        ofstream file;
        //Numero di bytes scritti (per controllo)
        u32 writedBytes;
        //Dimensioni delle sezioni del file, in bytes
        //Lunghezza totale del file
        u32 lungTotFile;
        //Lunghezza totale chunk MDATA;
        u32 lungTotMDATA;
        //Lunghezza totale chunk KFDATA;
        u32 lungTotKFDATA;
        //NB: nel seguito, si intende che le righe siano tutte una dopo l'altra su una sola riga! Quindi:
        // matrice (m x n), elemento (i,j) della matrice => elemento (n*i,j) dell'array!
        // Inoltre i chunk qui di seguito sono scritti in ordine diverso rispetto alla posizione nell'array!
        //Lunghezza della sezione materiali (puntatore per array: una riga per ogni materiale; una colonna
        // per ogni sezione: TOT=MAT_ENTRY, MAT_NAME, MAT_TEXMAP)
        u32 *lungSezMats;
        //Lunghezza della sezione oggetti (puntatore per array: una riga per ogni oggetto; una colonna
        // per ogni sezione: TOT=NAMED_OBJECT, N_TRL_OBJECT, POINT_ARRAY, TEX_VERTS, FACE_ARRAY, MSH_MAT_GROUP)
        u32 *lungSezObjs;
        //Lunghezza della sezione oggetti (puntatore per array: una riga per ogni oggetto; una colonna
        // per ogni sezione: TOT=OBJECT_NODE_TAG, NODE_HDR, POS_TRACK_TAG, ROT_TRACK_TAG)
        u32 *lungSezAnims;
};


#endif // FILE3DS_H_INCLUDED
