#ifndef LISTE_CPP_INCLUDED
#define LISTE_CPP_INCLUDED

#include "Liste.h"

/** *** *** *** *** *** *** *** *** *** *** *** *** *** ***
 **
 ** TESTA DELLA LISTA: CLASSE Lista<tipo>
 **
 ** *** *** *** *** *** *** *** *** *** *** *** *** *** ***
 **/


/** CLASSE Lista<tipo>: COSTRUTTORE & DISTRUTTORE **/

/* Costruttore:
 * - inizializza i puntatori curr ed elem.
 * - inizializza errCode.
 */
template <class tipo>
Lista<tipo>::Lista(void) {
    elem=NULL;
    curr=NULL;
    errCode=ERR_CL_DEFAULT;
    lung=0;
}

/* Costruttore:
 * - inizializza i puntatori curr ed elem.
 * - inizializza errCode con il valore ricevuto.
 */
template <class tipo>
Lista<tipo>::Lista(ErrClass _errCode) {
    elem=NULL;
    curr=NULL;
    errCode=_errCode;
    lung=0;
}


/* Distruttore:
 * - svuota la lista (se non  gi stata svuotata).
 */
template <class tipo>
Lista<tipo>::~Lista(void) {
    eraseAll();
}




/** CLASSE Lista<tipo>: CREAZIONE ELEMENTO **/

/* Aggiunge un nuovo elemento:
 * - controlla se la lista  vuota;
 * - aggiorna curr con il puntatore al contenuto dell'elemento appena creato;
 * - restituisce il puntatore al nuovo elemento.
 */
template <class tipo>
tipo *Lista<tipo>::addElem(void) {
    if(elem!=NULL) {
        //La lista non  vuota => ottiene il puntatore dell'ultimo elemento
        getLast();
    }

    try {
        curr=new ElementoLista<tipo>(curr);
        lung++;
    }
    catch(bad_alloc xa) {
        throw Errore(errCode, ERR_OP_MEM_ALLOC, ERR_TP_BAD_ALLOC, getLast()->getNum());
    }
    catch(Errore err) {
        err.setErrClass(errCode);
        err.setNumElem(getLast()->getNum());
        throw err;
    }

    if(elem==NULL) {
        //La lista era vuota
        elem=curr;
    }
    checkCurr();
    return curr->getOgg();
}




/** CLASSE Lista<tipo>: RICERCA DI UN ELEMENTO **/

/* Richiesta dell'elemento corrente:
 * - funzione PRIVATA;
 * - restituisce il puntatore all'elemento corrente.
 */
template <class tipo>
ElementoLista<tipo> *Lista<tipo>::getCurr(void) {
    checkCurr();
    return curr;
}


/* Richiesta del primo elemento:
 * - funzione PRIVATA;
 * - restituisce il puntatore al primo elemento.
 */
template <class tipo>
ElementoLista<tipo> *Lista<tipo>::getFirst(void) {
    curr=elem;
    checkCurr();
    return curr;
}


/* Richiesta dell'ultimo elemento:
 * - funzione PRIVATA;
 * - restituisce il puntatore all'ultimo elemento.
 */
template <class tipo>
ElementoLista<tipo> *Lista<tipo>::getLast(void) {
    if(curr!=NULL) {
        //La lista non  vuota
        while(curr->getNext()!=NULL) {
            //Cerca l'ultimo elemento
            curr=curr->getNext();
        }
    }
    checkCurr();
    return curr;
}


/* Richiesta dell'elemento numero n:
 * - restituisce il puntatore al contenuto dell'n-esimo elemento.
 */
template <class tipo>
tipo *Lista<tipo>::getElem(size_list n) {
    if(curr!=NULL) {
        //La lista non  vuota
        if(curr->getNum()>n) {
            //L'elemento richiesto  prima di curr;
            while(curr->getPrev()!=NULL) {
                curr=curr->getPrev();
                if(curr->getNum()==n) break;
            }
        }
        else if(curr->getNum()<n) {
            //L'elemento richiesto  dopo curr;
            while(curr->getNext()!=NULL) {
                curr=curr->getNext();
                if(curr->getNum()==n) break;
            }
        }

    }
    //Se curr->getNum==n arriva direttamente qui
    //Se curr==NULL (lista vuota) arriva direttamente qui
    checkCurr();
    return curr->getOgg();
}



/** CLASSE Lista<tipo>: SVUOTAMENTO LISTA **/

/* Svuota la lista:
 * - innesca lo svuotamento basato sui distruttori.
 */
template <class tipo>
void Lista<tipo>::eraseAll(void) {
    ElementoLista<tipo> *firstToDelete=elem;
    if(elem!=NULL) {
        //La lista non  vuota => inizia lo svuotamento
        do {
            curr=firstToDelete;
            firstToDelete=curr->getNext();
            delete curr;
        } while(firstToDelete!=NULL);
    }
    //Re-inizializza i puntatori
    curr=NULL;
    elem=NULL;
    lung=0;
}



/** CLASSE Lista<tipo>: CONTROLLO **/

/* Controlla la lunghezza della lista:
 * - restituisce il numero di elementi memorizzati effettivamente e non il numero
 *   dell'ultimo elemento;
 * - controlla la congruenza dei numeri di ogni elemento.
 */
template <class tipo>
size_list Lista<tipo>::check(void) {
    size_list i=0, j=0, k=0;
    ElementoLista<tipo> *lastVisited=NULL;
    if(elem!=NULL) {
        //La lista non  vuota => inizia il controllo
        curr=elem; //Riavvolge la lista
        //Percorre la lista in avanti
        do {
            if(curr->getNum()!=i) throw Errore(errCode, ERR_OP_LIST_CHECK, ERR_TP_FORWARD_FAIL, i);
            i++;
            lastVisited=curr;
            curr=curr->getNext();
        } while(curr!=NULL);
        //Percorre la lista all'indietro
        j=i-1; //i viene incrementato prima del controllo di curr!=NULL
        curr=lastVisited; //Contiene l'indirizzo dell'ultimo elemento esistente
        do {
            if(curr->getNum()!=j) throw Errore(errCode, ERR_OP_LIST_CHECK, ERR_TP_BACKWARD_FAIL, j);
            j--;
            k++;
            curr=curr->getPrev();
        } while(curr!=NULL);
        //Controlla che all'indietro abbia lo stesso numero di elementi
        if(k!=i) throw Errore(errCode, ERR_OP_LIST_CHECK, ERR_TP_CHECK_FAIL);
        //Controlla il contatore 'lung' della lista
        if(lung!=i) throw Errore(errCode, ERR_OP_LIST_CHECK, ERR_TP_CHECK_FAIL_2);
        //Riposiziona il 'cursore' della lista all'inizio
        curr=elem;
    }
    //Se arriva qui non ci sono stati errori => restituisce il numero di elementi presenti
    //NB: i  il numero progressivo dell'ultimo oggetto gi incrementato di 1 => lung=i (e non i+1)
    return i;
}


/* Controlla il contenuto di curr:
 * - se curr==NULL lancia un'eccezione;
 * - la funzione  PRIVATA.
 */
template <class tipo>
void Lista<tipo>::checkCurr(void) {
    if(curr==NULL) {
        //Errore!
        throw Errore(errCode, ERR_OP_POINTER_CHECK, ERR_TP_NULL_POINTER);
    }
}



/** CLASSE Lista<tipo>: ERRORI **/

/* Salva il codice di errore per la classe:
 */
template <class tipo>
void Lista<tipo>::setErrCode(ErrClass _errCode) {
   errCode=_errCode;
}


/* Restituisce il codice di errore per la classe:
 */
template <class tipo>
ErrClass Lista<tipo>::getErrCode(void) {
    return errCode;
}



/** CLASSE Lista<tipo>: LUNGHEZZA **/

/* Restituisce la lunghezza della lista:
 */
template <class tipo>
size_list Lista<tipo>::getLung(void) {
    return lung;
}




/** *** *** *** *** *** *** *** *** *** *** *** *** *** ***
 **
 ** ELEMENTO DELLA LISTA: CLASSE ElementoLista<tipo>
 **
 ** *** *** *** *** *** *** *** *** *** *** *** *** *** ***
 **/


/** CLASSE ElementoLista<tipo>: COSTRUTTORE & DISTRUTTORE **/

/* Costruttore:
 * - inizializza i puntatori succ ed prev.
 * - inizializza errCode.
 */
template <class tipo>
ElementoLista<tipo>::ElementoLista(ElementoLista<tipo> *last) {
    next=NULL;
    prev=last;
    if(last!=NULL) {
        //Non  il primo della lista
        if(last->getNum()==MAX_LIST_LENGTH) {
            //Ha raggiunto il limite di elementi memorizzabili
            //NB: se last==NULL non entra nell'if e non lancia ERR_TP_NULL_POINTER
            throw Errore(ERR_OP_LIST_ADD, ERR_TP_EXCEED_UNITS);
        }
        num=prev->getNum()+1;
        prev->next=this;
    }
    else {
        //E' il primo elemento
        num=0;
    }
}


/* Distruttore: (per memoria)
 * - nothing to do.
 */



/** CLASSE ElementoLista<tipo>: NUMERO ELEMENTO **/

/* Restituisce il numero dell'elemento:
 */
template <class tipo>
size_list ElementoLista<tipo>::getNum(void) {
    return num;
}



/** CLASSE ElementoLista<tipo>: SCORIMENTO LISTA **/

/* Elemento precedente:
 * - restituisce il puntatore all'elemento precedente;
 * - pu restituire NULL;
 * -  una funzione PRIVATA.
 */
template <class tipo>
ElementoLista<tipo> *ElementoLista<tipo>::getPrev(void) {
    return prev;
}


/* Elemento successivo:
 * - restituisce il puntatore all'elemento successivo;
 * - pu restituire NULL;
 * -  una funzione PRIVATA.
 */
template <class tipo>
ElementoLista<tipo> *ElementoLista<tipo>::getNext(void) {
    return next;
}



/** CLASSE ElementoLista<tipo>: INTERROGAZIONE CONTENUTO **/

/* Restituisce il puntatore al contenuto dell'elemento:
 * - restituisce il puntatore al contenuto dell'elemento;
 * -  una funzione PRIVATA.
 */
template <class tipo>
tipo *ElementoLista<tipo>::getOgg(void) {
    return &ogg;
}

#endif // LISTE_CPP_INCLUDED
