#ifndef LISTE_H_INCLUDED
#define LISTE_H_INCLUDED

#include <iostream>
#include "Tipi.h"
#include "Errore.h"

typedef u16 size_list; //per comodit
#define MAX_LIST_LENGTH 0xFFFF


using namespace std;



template<class tipo> class ElementoLista; //Dichiarazione anticipata


/** CLASSI PER LISTE: TESTA DELLA LISTA **/

/* Questa classe costituisce la testa della lista:
 * - sfrutta curr per abbreviare i tempi di ricerca degli elementi (accessi consecutivi);
 * - tutte le funzioni restituiscono SEMPRE un puntatore valido (se si richiede l'indirizzo di
 *   un elemento inesistente lanciano un'eccezione grazie alla funzione checkCurr();
 * - pu contenere al pi 65,536 elementi (dal no.0 al no. 65,535 - intero unsigned 16bit).
 */
template <class tipo>
class Lista {

    public:
        //Costruttore e distruttore
        Lista(void);
        Lista(ErrClass _errCode);
        ~Lista(void);
        //Creazione di un nuovo elemento
        tipo *addElem(void); //Restituisce puntatore all'elemento appena creato (aggiunto in coda)
        //Ricerca di un elemento
        tipo *getElem(size_list n); //Restituisce il puntatore dell'elemento numero n
        //Svuotamento
        void eraseAll(void); //Svota la lista
        //Controllo della lunghezza della lista
        size_list check(void); //Restituisce il numero di elementi memorizzati effettivamente
        //Codice di errore
        void setErrCode(ErrClass _errCode); //Assegne il codice
        ErrClass getErrCode(void); //Restituisce il codice assegnato a questa classe
        //Restituisce la lunghezza della lista
        size_list getLung(void);

    private:
        //Puntatori
        ElementoLista<tipo> *elem;
        ElementoLista<tipo> *curr;
        //Codice di errore
        ErrClass errCode;
        //Funzioni private
        void checkCurr(void); //Controlla che curr!=NULL; se false, lancia una eccezione
        ElementoLista<tipo> *getCurr(void); //Restituisce il puntatore dell'elemento corrente
        ElementoLista<tipo> *getFirst(void); //Restituisce il puntatore del primo elemento della lista
        ElementoLista<tipo> *getLast(void); //Restituisce il puntatore dell'ultimo elemento della lista
        //Numero di elementi della lista
        size_list lung;
};



/** CLASSI PER LISTE: ELEMENTO DELLA LISTA **/

/* Questa classe costituisce il generico elemento della lista:
 * - il contenuto  privato e accessibile solo attraverso le apposite funzioni;
 * - possono esistere al massimo 65,535 elementi;
 * - il contenuto e le funzioni sono specificati nelle classi 'tipo';
 * - le funzioni per muoversi nella lista sono private perch non devono essere usate
 *   all'esterno. Occorre usare quelle della testa della lista, che hanno il controllo sul
 *   puntatore restituito.
 */
template <class tipo>
class ElementoLista {

    public:
        //Costruttore e distruttore
        ElementoLista(ElementoLista<tipo> *last);
        ~ElementoLista(void) { };
        //Restituisce il numero dell'elemento
        size_list getNum(void);

    private:
        //Puntatori
        ElementoLista<tipo> *next;
        ElementoLista<tipo> *prev;
        //Contenuto
        tipo ogg;
        //Numero dell'elemento
        size_list num;
        //Funzioni per muoversi nella lista
        ElementoLista<tipo> *getPrev(void);
        ElementoLista<tipo> *getNext(void);
        //Restituisce il puntatore al contenuto
        tipo* getOgg(void);
        //Dichiarazione friend per la testa
        friend class Lista<tipo>;
};


#endif // LISTE_H_INCLUDED
