#ifndef MATHUTILS_CPP_INCLUDED
#define MATHUTILS_CPP_INCLUDED

#include "MathUtils.h"


/** OPERAZIONI CON I VETTORI **/

/* Norma di un vettore
 */
template<class tipo>
tipo norma(Vett3<tipo> *v) {
    tipo norma;
    norma=sqrt( v->getX() * v->getX() + v->getY() * v->getY() + v->getZ() * v->getZ() );
    return norma;
}

/* Normalizza un vettore
 */
template<class tipo>
void normalizza(Vett3<tipo> *v) {
    tipo norma;
    norma=sqrt( v->getX() * v->getX() + v->getY() * v->getY() + v->getZ() * v->getZ() );
    if(abs(norma)>0.0001f) v->set( v->getX()/norma, v->getY()/norma, v->getZ()/norma );
}


/* Prodotto scalare o interno:
 */
template<class tipo>
tipo prodScal(Vett3<tipo> *v, Vett3<tipo> *w) {
    tipo scal;
    scal= v->getX() * w->getX() + v->getY() * w->getY() + v->getZ() * w->getZ() ;
    return scal;
}


/* Prodotto scalare o interno normalizzato:
 *  - restituisce il coseno dell'angolo compreso tra i vettori.
 */
template<class tipo>
tipo prodScalNorm(Vett3<tipo> *v, Vett3<tipo> *w) {
    tipo scal;
    scal=prodScal(v, w);
    scal/=norma(v);
    scal/=norma(w);
    return scal;
}


/* Prodotto vettoriale:
 */
template<class tipo>
void prodVett(Vett3<tipo> *v, Vett3<tipo> *w, Vett3<tipo> *ris) {
    tipo temp;
    temp= v->getY() * w->getZ() - v->getZ() * w->getY() ;
    ris->setX(temp);
    temp= v->getZ() * w->getX() - v->getX() * w->getZ() ;
    ris->setY(temp);
    temp= v->getX() * w->getY() - v->getY() * w->getX() ;
    ris->setZ(temp);
}



/** OPERAZIONI CON LE MATRICI 4x4 **/

/* Prodotto righe per colonne di due matrici 4x4
 *  - esegue il prodotto C=AxB.
 */
template<class tipo>
void prodRxC(Matr4x4<tipo> *a, Matr4x4<tipo> *b, Matr4x4<tipo> *c) {
    tipo temp;
    //Prima riga:
    // -> Prima colonna: c11
        temp = a->get11() * b->get11() + a->get12() * b->get21() + a->get13() * b->get31() + a->get14() * b->get41() ;
        c->set11(temp);
    // -> Seconda colonna: c12
        temp = a->get11() * b->get12() + a->get12() * b->get22() + a->get13() * b->get32() + a->get14() * b->get42() ;
        c->set12(temp);
    // -> Terza colonna: c13
        temp = a->get11() * b->get13() + a->get12() * b->get23() + a->get13() * b->get33() + a->get14() * b->get43() ;
        c->set13(temp);
    // -> Quarta colonna: c14
        temp = a->get11() * b->get14() + a->get12() * b->get24() + a->get13() * b->get34() + a->get14() * b->get44() ;
        c->set14(temp);

    //Seconda riga:
    // -> Prima colonna: c21
        temp = a->get21() * b->get11() + a->get22() * b->get21() + a->get23() * b->get31() + a->get24() * b->get41() ;
        c->set21(temp);
    // -> Seconda colonna: c22
        temp = a->get21() * b->get12() + a->get22() * b->get22() + a->get23() * b->get32() + a->get24() * b->get42() ;
        c->set22(temp);
    // -> Terza colonna: c23
        temp = a->get21() * b->get13() + a->get22() * b->get23() + a->get23() * b->get33() + a->get24() * b->get43() ;
        c->set23(temp);
    // -> Quarta colonna: c24
        temp = a->get21() * b->get14() + a->get22() * b->get24() + a->get23() * b->get34() + a->get24() * b->get44() ;
        c->set24(temp);

    //Terza riga:
    // -> Prima colonna: c31
        temp = a->get31() * b->get11() + a->get32() * b->get21() + a->get33() * b->get31() + a->get34() * b->get41() ;
        c->set31(temp);
    // -> Seconda colonna: c32
        temp = a->get31() * b->get12() + a->get32() * b->get22() + a->get33() * b->get32() + a->get34() * b->get42() ;
        c->set32(temp);
    // -> Terza colonna: c33
        temp = a->get31() * b->get13() + a->get32() * b->get23() + a->get33() * b->get33() + a->get34() * b->get43() ;
        c->set33(temp);
    // -> Quarta colonna: c34
        temp = a->get31() * b->get14() + a->get32() * b->get24() + a->get33() * b->get34() + a->get34() * b->get44() ;
        c->set34(temp);

    //Quarta riga:
    // -> Prima colonna: c41
        temp = a->get41() * b->get11() + a->get42() * b->get21() + a->get43() * b->get31() + a->get44() * b->get41() ;
        c->set41(temp);
    // -> Seconda colonna: c42
        temp = a->get41() * b->get12() + a->get42() * b->get22() + a->get43() * b->get32() + a->get44() * b->get42() ;
        c->set42(temp);
    // -> Terza colonna: c43
        temp = a->get41() * b->get13() + a->get42() * b->get23() + a->get43() * b->get33() + a->get44() * b->get43() ;
        c->set43(temp);
    // -> Prima colonna: c44
        temp = a->get41() * b->get14() + a->get42() * b->get24() + a->get43() * b->get34() + a->get44() * b->get44() ;
        c->set44(temp);
    /*//Stampa per correzione
    cout<<endl<<"Matrice A:"<<endl;
    cout<<scientific<<setprecision(6);
    cout<<a->get11()<<'\t'<<a->get12()<<'\t'<<a->get13()<<'\t'<<a->get14()<<endl;
    cout<<a->get21()<<'\t'<<a->get22()<<'\t'<<a->get23()<<'\t'<<a->get24()<<endl;
    cout<<a->get31()<<'\t'<<a->get32()<<'\t'<<a->get33()<<'\t'<<a->get34()<<endl;
    cout<<a->get41()<<'\t'<<a->get42()<<'\t'<<a->get43()<<'\t'<<a->get44()<<endl;
    cout<<endl<<"Matrice B:"<<endl;
    cout<<scientific<<setprecision(6);
    cout<<b->get11()<<'\t'<<b->get12()<<'\t'<<b->get13()<<'\t'<<b->get14()<<endl;
    cout<<b->get21()<<'\t'<<b->get22()<<'\t'<<b->get23()<<'\t'<<b->get24()<<endl;
    cout<<b->get31()<<'\t'<<b->get32()<<'\t'<<b->get33()<<'\t'<<b->get34()<<endl;
    cout<<b->get41()<<'\t'<<b->get42()<<'\t'<<b->get43()<<'\t'<<b->get44()<<endl;
    cout<<endl<<"Matrice C:"<<endl;
    cout<<scientific<<setprecision(6);
    cout<<c->get11()<<'\t'<<c->get12()<<'\t'<<c->get13()<<'\t'<<c->get14()<<endl;
    cout<<c->get21()<<'\t'<<c->get22()<<'\t'<<c->get23()<<'\t'<<c->get24()<<endl;
    cout<<c->get31()<<'\t'<<c->get32()<<'\t'<<c->get33()<<'\t'<<c->get34()<<endl;
    cout<<c->get41()<<'\t'<<c->get42()<<'\t'<<c->get43()<<'\t'<<c->get44()<<endl;*/
}



/* Copia due matrici 4x4
 *  - esegue la copia out=in.
 */
template<class tipo>
void copyMatr(Matr4x4<tipo> *in, Matr4x4<tipo> *out) {
    //Prima riga:
    out->set11(in->get11());
    out->set12(in->get12());
    out->set13(in->get13());
    out->set14(in->get14());
    //Seconda riga:
    out->set21(in->get21());
    out->set22(in->get22());
    out->set23(in->get23());
    out->set24(in->get24());
    //Terza riga:
    out->set31(in->get31());
    out->set32(in->get32());
    out->set33(in->get33());
    out->set34(in->get34());
    //Quarta riga:
    out->set41(in->get41());
    out->set42(in->get42());
    out->set43(in->get43());
    out->set44(in->get44());
    cout<<scientific<<setprecision(6);
    /*//Stampa
    cout<<scientific<<setprecision(6);
    cout<<out->get11()<<'\t'<<out->get12()<<'\t'<<out->get13()<<'\t'<<out->get14()<<endl;
    cout<<out->get21()<<'\t'<<out->get22()<<'\t'<<out->get23()<<'\t'<<out->get24()<<endl;
    cout<<out->get31()<<'\t'<<out->get32()<<'\t'<<out->get33()<<'\t'<<out->get34()<<endl;
    cout<<out->get41()<<'\t'<<out->get42()<<'\t'<<out->get43()<<'\t'<<out->get44()<<endl;*/
}



/** OPERAZIONI CON LE MATRICI 3x3 **/

/* Prodotto righe per colonne di due matrici 3x3
 *  - esegue il prodotto C=AxB.
 */
template<class tipo> void prodRxC(Matr3x3<tipo> *a, Matr3x3<tipo> *b, Matr3x3<tipo> *c) {
    tipo temp;
    //Prima riga:
    // -> Prima colonna: c11
        temp = a->get11() * b->get11() + a->get12() * b->get21() + a->get13() * b->get31() ;
        c->set11(temp);
    // -> Seconda colonna: c12
        temp = a->get11() * b->get12() + a->get12() * b->get22() + a->get13() * b->get32() ;
        c->set12(temp);
    // -> Terza colonna: c13
        temp = a->get11() * b->get13() + a->get12() * b->get23() + a->get13() * b->get33() ;
        c->set13(temp);

    //Seconda riga:
    // -> Prima colonna: c21
        temp = a->get21() * b->get11() + a->get22() * b->get21() + a->get23() * b->get31() ;
        c->set21(temp);
    // -> Seconda colonna: c22
        temp = a->get21() * b->get12() + a->get22() * b->get22() + a->get23() * b->get32() ;
        c->set22(temp);
    // -> Terza colonna: c23
        temp = a->get21() * b->get13() + a->get22() * b->get23() + a->get23() * b->get33() ;
        c->set23(temp);

    //Terza riga:
    // -> Prima colonna: c31
        temp = a->get31() * b->get11() + a->get32() * b->get21() + a->get33() * b->get31() ;
        c->set31(temp);
    // -> Seconda colonna: c32
        temp = a->get31() * b->get12() + a->get32() * b->get22() + a->get33() * b->get32() ;
        c->set32(temp);
    // -> Terza colonna: c33
        temp = a->get31() * b->get13() + a->get32() * b->get23() + a->get33() * b->get33() ;
        c->set33(temp);
}

/* Determinante di una matrice 3x3
 *  - applica la regola di Sarrus
 */
template<class tipo> tipo det(Matr3x3<tipo> *a) {
    tipo det= a->get11() * a->get22() * a->get33()
            + a->get12() * a->get23() * a->get31()
            + a->get13() * a->get21() * a->get32()
            - a->get13() * a->get22() * a->get31()
            - a->get11() * a->get23() * a->get32()
            - a->get12() * a->get21() * a->get33();
    return det;
}




/** OPERAZIONI GEOMETRICHE CON MATRICI e VETTORI **/

/* Applica una trasformazione, identificata dalla matrice, al vettore
 *  - moltiplica il vettore [vett 1] con la matrice trasf
 */
template<class tipo>
void trasfVett(Matr4x4<tipo> *trasf, Vett3<tipo> *vettIn, Vett3<tipo> *vettOut) {
    //Crea il vettore per l'operazione
    Vett4<tipo> vett;
    vett.set(vettIn->getX(), vettIn->getY(), vettIn->getZ(), 1.0f);
    //Moltiplica il vettore per la matrice
    tipo x, y, z;
    x = vett.getX() * trasf->get11() + vett.getY() * trasf->get21() +vett.getZ() * trasf->get31() + vett.getW() * trasf->get41();
    y = vett.getX() * trasf->get12() + vett.getY() * trasf->get22() +vett.getZ() * trasf->get32() + vett.getW() * trasf->get42();
    z = vett.getX() * trasf->get13() + vett.getY() * trasf->get23() +vett.getZ() * trasf->get33() + vett.getW() * trasf->get43();
    vettOut->set(x, y, z);
    /*//Stampa
    cout<<endl<<"Funzione trasfVett"<<endl<<"Matrice:"<<endl;
    cout<<scientific<<setprecision(6);
    cout<<trasf->get11()<<'\t'<<trasf->get12()<<'\t'<<trasf->get13()<<'\t'<<trasf->get14()<<endl;
    cout<<trasf->get21()<<'\t'<<trasf->get22()<<'\t'<<trasf->get23()<<'\t'<<trasf->get24()<<endl;
    cout<<trasf->get31()<<'\t'<<trasf->get32()<<'\t'<<trasf->get33()<<'\t'<<trasf->get34()<<endl;
    cout<<trasf->get41()<<'\t'<<trasf->get42()<<'\t'<<trasf->get43()<<'\t'<<trasf->get44()<<endl;
    cout<<"Vettore in ingresso:"<<endl;
    cout<<scientific<<setprecision(6);
    cout<<vettIn->getX()<<'\t'<<vettIn->getY()<<'\t'<<vettIn->getZ()<<endl;
    cout<<"Vettore funzione:"<<endl;
    cout<<scientific<<setprecision(6);
    cout<<vett.getX()<<'\t'<<vett.getY()<<'\t'<<vett.getZ()<<'\t'<<vett.getW()<<endl;
    cout<<"Vettore in uscita:"<<endl;
    cout<<scientific<<setprecision(6);
    cout<<vettOut->getX()<<'\t'<<vettOut->getY()<<'\t'<<vettOut->getZ()<<endl;*/
}


/* Ricava l'angolo e l'asse di rotazione dalla matrice di trasformazione
 */
template<class tipo> void matr2angleAxis(Matr4x4<tipo> *matr, Vett4<tipo> *vett) {
    //Usa due metodi: il primo vale se l'angolo  diverso da 180, il secondo vale sempre ma non considera il segno dell'angolo
    // quindi  adatto al caso angolo=180
    //Matrice 3x3, corrispondente alla parte di rotazione della matrice di trasformazione, e suo quadrato
    Matr3x3<f96> A, B;
    A.set( matr->get11(), matr->get12(), matr->get13(),
           matr->get21(), matr->get22(), matr->get23(),
           matr->get31(), matr->get32(), matr->get33() );
    prodRxC( &A, &A, &B);
    //Calcolo di determinanti e tracce per il calcolo degli autovalori
    f96 detA = det(&A);
    f96 trA = A.get11() + A.get22() + A.get33() ;
    f96 trB = B.get11() + B.get22() + B.get33() ;
    //Coefficienti del polinomio caratteristico di terzo grado: ax^3 + bx^2 + cx +d = 0
    f96 a=-1;
    f96 b=trA;
    f96 c=0.5*(trB - trA*trA);
    f96 d=detA;
    //Coefficienti per la soluzione del polinomio caratteristico:
    f96 q = ( 3*a*c - b*b ) / ( 9*a*a );
    f96 r = ( 9*a*b*c - 27*a*a*d - 2*b*b*b ) / ( 54*a*a*a );
    //Discriminante del polinomio di terzo grado:
    f96 delta=q*q*q+r*r;
    //if(delta<0) throw Errore(todo);
    //Altri coefficienti
    f96 s = cbrt( r + sqrt(delta) ); //pow( r + sqrt(delta) , 1.0f/3 );
    f96 t = cbrt( r - sqrt(delta) ); //pow( r - sqrt(delta) , 1.0f/3 ); NB: serve if(arg<0) pow(-arg)
    //Soluzioni del polinomio (autovalori di A) NB: solo la parte reale
    f96 x1 = s + t - b/(3*a);
    //f96 x2 = -0.5*(s+t) - b/(3*a); //segue la parte immaginaria
    //Il terzo autovalore ha la stessa parte reale del secondo
    f96 re = -0.5*(s+t) - b/(3*a); // => cos
    f96 im = sqrt(3)/2 * (s-t); // => sin
    //Ora si pu calcolare l'angolo, a partire dal secondo e terzo autovettore
    //f96 angolo=acos(x2);
    f96 angolo=atan2(im, re);
    f96 sign, v1, v2, v3;;
    if(abs(angolo-piGreco)>1e-4) {
        //L'angolo  diverso da 180; il segno viene automaticamente invertito!
        if(abs(angolo)>1e-4) {
            //Si pu dividere per sen(angolo)
            tipo traccia=matr->get11() + matr->get22() + matr->get33();
            if(traccia>3.0f) traccia=3.0f;
            angolo = acos( ( traccia - 1 )/2 );
            v1 = -( matr->get32() - matr->get23() ) / ( 2 * sin(angolo) ) ;
            v2 = -( matr->get13() - matr->get31() ) / ( 2 * sin(angolo) ) ;
            v3 = -( matr->get21() - matr->get12() ) / ( 2 * sin(angolo) ) ;
            //cout<<endl<<"v1="<<v1<<", v2="<<v2<<", v3="<<v3<<", ang="<<angolo;
        }
        else {
            //Evita l'errore di divisione per zero (angolo==0)
            v1=v2=v3=0.0f;
        }
    }
    else {
        //Il metodo precedente fallisce
        //Per trovare l'asse di rotazione si deve trovare l'autovettore corrispondente al primo autovettore
        // C = prime due righe e colonne della matrice A
        f96 detC= (A.get11()-x1) * (A.get22()-x1) - A.get12() * A.get21() ;
        f96 detD= (A.get22()-x1) * (A.get33()-x1) - A.get32() * A.get23() ;
        f96 detE= (A.get11()-x1) * (A.get33()-x1) - A.get13() * A.get31() ;
        if(abs(detC)>1e-6) {
            v1 = -(A.get22()-x1) * A.get13() / detC + A.get12() * A.get23() / detC ;
            v2 =  A.get21() * A.get13() / detC - (A.get11()-x1) * A.get23() / detC ;
            v3 = 1 ;
            if(abs(v1)<1e-6) v1=0.0f;
            if(abs(v2)<1e-6) v2=0.0f;
        }
        else if(abs(detD)>1e-6) {
            v1 = 1 ;
            v2 = -(A.get33()-x1) * A.get21() / detD + A.get23() * A.get31() / detD ;
            v3 =  A.get32() * A.get21() / detD - (A.get22()-x1) * A.get31() / detD ;
            if(abs(v2)<1e-6) v2=0.0f;
            if(abs(v3)<1e-6) v3=0.0f;
        }
        else if(abs(detE)>1e-6) {
            v1 =  (A.get33()-x1) * A.get12() / detE - A.get12() * A.get32() / detE ;
            v2 = 1 ;
            v3 = -A.get31() * A.get12() / detE + (A.get11()-x1) * A.get32() / detE ;
            if(abs(v1)<1e-6) v1=0.0f;
            if(abs(v3)<1e-6) v3=0.0f;
        }
        else { v1=v2=v3=0.0f; }
    }
    //else throw Errore(todo);
    Vett3<f96> asse;
    asse.set(v1, v2, v3);
    f96 norm=norma(&asse);
    if(norm>1e-6) { v1/=norm; v2/=norm; v3/=norm; }
    if(angolo<0) sign=-1.0f; //3ds e ts hanno orientamenti opposti! ANNULLATO!
    else sign=+1.0f;
    vett->set(v1*sign, v2*sign, v3*sign, abs(angolo));
    //vett->set(v1, v2, v3, angolo);
}


/* Ricava la matrice di dcm dall'angolo+asse di rotazione
 */
template<class tipo> void angleAxis2matr(Vett4<tipo> *vett, Matr3x3<tipo> *dcm) {
    //if(vett->getY()<0 || vett->getZ()<0 || vett->getY()<0) segno=-1.0f;
    tipo angolo=vett->getW();
    tipo a=cos( angolo );
    tipo b=1-a;
    tipo c=sin( vett->getW() );
    tipo e1=vett->getX();//abs(vett->getX());
    tipo e2=vett->getY();//abs(vett->getY());
    tipo e3=vett->getZ();//abs(vett->getZ());
    dcm->set( a+b*e1*e1,    b*e1*e2+c*e3,   b*e1*e3-c*e2,
              b*e1*e2-c*e3, a+b*e2*e2,      b*e2*e3+c*e1,
              b*e1*e3+c*e2, b*e2*e3-c*e1,   a+b*e3*e3);
}


/* Ricava l'angolo e l'asse di rotazione dal quaternione
 */
template<class tipo> void quat2angleAxis(Vett4<tipo> *quat, Vett4<tipo> *vett) {
    //Vettore corrispondente al quaternione, escluso il temine scalare:
    Vett3<tipo> asse;
    asse.set( quat->getX(), quat->getY(), quat->getZ() );
    normalizza(&asse);
    if(quat->getW()>1.0f) quat->setW(1.0f); //Contro l'errore numerico
    else if(quat->getW()<-1.0f) quat->setW(-1.0f); //Contro l'errore numerico
    tipo angolo=2*acos( quat->getW() );
    //cout<<"quat: "<<quat->getX()<<", "<<quat->getY()<<", "<<quat->getZ()<<", "<<quat->getW()<<endl;
    //cout<<"QuatW: "<<quat->getW()<<endl;
    //cout<<"Angolo: "<<angolo<<endl;
    vett->set( asse.getX(), asse.getY(), asse.getZ(), angolo );
}


/* Ricava il quaternione dal vettore angolo+asse
 */
template<class tipo> void angleAxis2quat(Vett4<tipo> *vett, Vett4<tipo> *quat) {
    tipo teta=vett->getW();
    quat->set( vett->getX() * sin(teta/2) ,
               vett->getY() * sin(teta/2) ,
               vett->getZ() * sin(teta/2) ,
               cos(teta/2) );
}


/* Ricava il quaternione dalla matrice di trasformazione
 */
/*template<class tipo> void matr2quat(Matr4x4<tipo> *matr, Vett4<tipo> *quat) {
    //Ricava il vettore asse-angolo dalla matrice
    Vett4<tipo> vett;
    matr2angleAxis(matr, &vett);
    //Trasforma il vettore ottenuto in quaternione
    angleAxis2quat(&vett, quat);
}*/





/* Differenza tra quaternioni:
 *  - esegue c=b*con(a), che equivale ad eseguire la differenza tra la rotazione a e la rotazione b;
 *  - il coniugato di un quaternione si ottiene cambiando segno alla parte vettoriale (cfr wiki).
 */
template<class tipo> void diffQuat(Vett4<tipo> *a, Vett4<tipo> *b, Vett4<tipo> *c) {
    //Coniugato di a:
    Vett4<tipo> a_star;
    a_star.set( -a->getX(), -a->getY(), -a->getZ(), +a->getW() );
    //cout<<"a_star: "<<a->getX()<<", "<<a->getY()<<", "<<a->getZ()<<", "<<a->getW()<<endl;
    //cout<<"b: "<<b->getX()<<", "<<b->getY()<<", "<<b->getZ()<<", "<<b->getW()<<endl;
    prodQuat( &a_star, b, c);
}


/* Ricava il quaternione e il vettore traslazione dalla matrice di trasformazione
 *  - NB: nella guida matrfaq_latest.html di j3d.org la matrice memorizzata  la trasposta
 *    di quella di TS e 3ds.
 */
template<class tipo> void matr2quat(Matr4x4<tipo> *matr, Vett4<tipo> *quat, Vett3<tipo> *trasl) {
    tipo s, x, y, z, w;
    //Calcolo della traccia della matrice
    tipo trace = matr->get11() + matr->get22() + matr->get33() + matr->get44() ; //NB: get44()==1
    //Controllo sulla traccia
    if(trace>1e-5) {
        //Si procede al calcolo immediato
        s = 2 * sqrt(trace);
        x = ( matr->get23() - matr->get32() ) / s;
        y = ( matr->get31() - matr->get13() ) / s;
        z = ( matr->get12() - matr->get21() ) / s;
        w = 0.25 * s;
    }
    else {
        //La traccia della matrice dcm  nulla, si cerca allora l'elemento maggiore della diagonale principale
        if( matr->get11() > matr->get22() && matr->get11() > matr->get33() ) {
            //L'elemento 1,1  il maggiore della diagonale
            s = 2 * sqrt( 1 + matr->get11() - matr->get22() - matr->get33() );
            x = 0.25 * s;
            y = ( matr->get12() + matr->get21() ) / s;
            z = ( matr->get31() + matr->get13() ) / s;
            w = ( matr->get23() - matr->get32() ) / s;
        }
        else if( matr->get22() > matr->get33() ) {
            //L'elemento 2,2  il maggiore della diagonale
            s = 2 * sqrt( 1 + matr->get22() - matr->get11() - matr->get33() );
            x = ( matr->get12() + matr->get21() ) / s;
            y = 0.25 * s;
            z = ( matr->get23() + matr->get32() ) / s;
            w = ( matr->get31() - matr->get13() ) / s;
        }
        else {
            //L'elemento 3,3  il maggiore della diagonale
            s = 2 * sqrt( 1 + matr->get33() - matr->get11() - matr->get22() );
            x = ( matr->get31() + matr->get13() ) / s;
            y = ( matr->get23() + matr->get32() ) / s;
            z = 0.25 * s;
            w = ( matr->get12() - matr->get21() ) / s;
        }
    }
    //Infine salva il quaternione...
    quat->set( x, y, z, w );
    //.. e il vettore traslazione
    if(trasl!=NULL) {
        trasl->set( matr->get41(), matr->get42(), matr->get43() );
    }
}


/* Ricava la matrice di trasformazione dal quaternione e dal vettore traslazione
 *  - NB: nella guida matrfaq_latest.html di j3d.org la matrice memorizzata  la trasposta
 *    di quella di TS e 3ds.
 */
template<class tipo> void quat2matr(Vett4<tipo> *quat, Vett3<tipo> *trasl, Matr4x4<tipo> *matr) {
    tipo xx = quat->getX() * quat->getX() ;
    tipo xy = quat->getX() * quat->getY() ;
    tipo xz = quat->getX() * quat->getZ() ;
    tipo xw = quat->getX() * quat->getW() ;
    tipo yy = quat->getY() * quat->getY() ;
    tipo yz = quat->getY() * quat->getZ() ;
    tipo yw = quat->getY() * quat->getW() ;
    tipo zz = quat->getZ() * quat->getZ() ;
    tipo zw = quat->getZ() * quat->getW() ;
    matr->set11( 1-2*(yy+zz) );
    matr->set21(   2*(xy-zw) );
    matr->set31(   2*(xz+yw) );
    matr->set12(   2*(xy+zw) );
    matr->set22( 1-2*(xx+zz) );
    matr->set32(   2*(yz-xw) );
    matr->set13(   2*(xz-yw) );
    matr->set23(   2*(yz+xw) );
    matr->set33( 1-2*(xx+yy) );
    matr->set14( 0 );
    matr->set24( 0 );
    matr->set34( 0 );
    matr->set44( 1 );
    //Traslazione
    if(trasl!=NULL) {
        matr->set41( trasl->getX() );
        matr->set42( trasl->getY() );
        matr->set43( trasl->getZ() );
    }
    else {
        matr->set41( 0 );
        matr->set42( 0 );
        matr->set43( 0 );
    }
}


/* Calcola il coniugato del quaternione
 *  - equivale a cambiare segno alla rotazione
 */
template<class tipo> void conjQuat(Vett4<tipo> *quatIn, Vett4<tipo> *quatOut) {
    if(quatOut==NULL) {
        //Modifica il quaternione in ingresso
        quatIn->setX( - quatIn->getX() );
        quatIn->setY( - quatIn->getY() );
        quatIn->setZ( - quatIn->getZ() );
    }
    else {
        //Lascia inalterato il quaternione in ingresso
        quatOut->setX( - quatIn->getX() );
        quatOut->setY( - quatIn->getY() );
        quatOut->setZ( - quatIn->getZ() );
    }
}


/* Moltiplicazione tra quaternioni:
 *  - esegue c=b*a, che equivale ad eseguire prima la rotazione a e poi la rotazione b.
 */
template<class tipo> void prodQuat(Vett4<tipo> *a, Vett4<tipo> *b, Vett4<tipo> *c) {
    tipo cx, cy, cz, cw;
    //Vettore
    cx = b->getW() * a->getX() + b->getX() * a->getW() + b->getY() * a->getZ() - b->getZ() * a->getY() ;
    cy = b->getW() * a->getY() - b->getX() * a->getZ() + b->getY() * a->getW() + b->getZ() * a->getX() ;
    cz = b->getW() * a->getZ() + b->getX() * a->getY() - b->getY() * a->getX() + b->getZ() * a->getW() ;
    //Scalare
    cw = b->getW() * a->getW() - b->getX() * a->getX() - b->getY() * a->getY() - b->getZ() * a->getZ() ;
    //Assegnazione
    c->set( cx, cy, cz, cw );
}


/* Ricava il vettore eaa dal quaternione
 */
template<class tipo> void quat2eaa(Vett4<tipo> *quat, Vett4<tipo> *eaa) {
    //normalizza(quat);
    tipo cosT=quat->getW();
    if(abs(cosT)>1) cosT=1; //Evita l'errore di dominio dell'acos causato da errori numerici
    tipo sinT=sqrt(1-cosT*cosT);
    tipo teta=2*acos(cosT);
    if(abs(sinT)<1e-5) sinT=1; //Evita l'errore divByZero. NB: sinT==0 => teta=0 o 180
    eaa->setX( quat->getX() / sinT );
    eaa->setY( quat->getY() / sinT );
    eaa->setZ( quat->getZ() / sinT );
    eaa->setW( teta );
}


/* Ricava il quaternione dal vettore eaa
 */
template<class tipo> void eaa2quat(Vett4<tipo> *eaa, Vett4<tipo> *quat) {
    //normalizza(eea);
    tipo x=eaa->getX();
    tipo y=eaa->getY();
    tipo z=eaa->getZ();
    tipo norma=sqrt( x*x+y*y+z*z );
    tipo angolo=eaa->getW();
    tipo sinT=sin( angolo/2 ) / norma;
    tipo cosT=cos( angolo/2 );
    x/=sinT;
    y/=sinT;
    z/=sinT;
    norma=sqrt( x*x+y*y+z*z+cosT*cosT );
    quat->setX( x / norma );
    quat->setY( y / norma );
    quat->setZ( z / norma );
    quat->setW( cosT / norma);
}


#endif // MATHUTILS_CPP_INCLUDED
