#include "Mesh.h"



/** *** *** *** *** *** *** *** *** *** *** *** *** *** ***
 **
 **                    CLASSE Material
 **
 ** *** *** *** *** *** *** *** *** *** *** *** *** *** ***
 **/


/** CLASSE Material: COSTRUTTORE & DISTRUTTORE **/

/* Costruttore:
 *  - inizializza le variabili.
 */
Material::Material(void) {
    alphaMode=0;
    zBuffMode=0;
    matShining=0;
    mipMapLevel=0.0f;
}


/* Distruttore: (per memoria)
 *  - nothing to do.
 */



/** CLASSE Material: ASSEGNAZIONE **/

/* Salva le variabili del materiale
 */
void Material::setMatName(string _matName) {
    matName=_matName;
}

void Material::setImageName(string _imageName) {
    imageName=_imageName;
}

void Material::setFilterName(string _filterName) {
    filterName=_filterName;
}

void Material::setShaderName(string _shaderName) {
    shaderName=_shaderName;
}

void Material::setAlphaMode(u8 _alphaMode) {
    alphaMode=_alphaMode;
}

void Material::setZBuffMode(u8 _zBuffMode) {
    zBuffMode=_zBuffMode;
}

void Material::setMatShining(s8 _matShining) {
    matShining=_matShining;
}

void Material::setMipMapLevel(f32 _mipMapLevel) {
    mipMapLevel=_mipMapLevel;
}

void Material::setColor(u8 _r, u8 _g, u8 _b) {
    r=_r;
    g=_g;
    b=_b;
}


/** CLASSE MATERIAL: INTERROGAZIONE **/

/* Restituisce i dati privati
 */
string Material::getMatName(void) {
    return matName;
}

string Material::getImageName(void) {
    return imageName;
}

string Material::getFilterName(void) {
    return filterName;
}

string Material::getShaderName(void) {
    return shaderName;
}

u8 Material::getAlphaMode(void) {
    return alphaMode;
}

u8 Material::getZBuffMode(void)  {
    return zBuffMode;
}

s8 Material::getMatShining(void) {
    return matShining;
}

f32 Material::getMipMapLevel(void) {
    return mipMapLevel;
}

u8 Material::getColorR(void) {
    return r;
}

u8 Material::getColorG(void) {
    return g;
}

u8 Material::getColorB(void) {
    return b;
}






/** *** *** *** *** *** *** *** *** *** *** *** *** *** ***
 **
 **                    CLASSE SubObject
 **
 ** *** *** *** *** *** *** *** *** *** *** *** *** *** ***
 **/



/** CLASSE SubObject: COSTRUTTORE & DISTRUTTORE **/

/* Costruttore:
 *  - inizializza i codici di errore delle liste.
 */
SubObject::SubObject(void) {
    faces.setErrCode(ERR_CL_ME_FACE);
    normals.setErrCode(ERR_CL_ME_NORMAL);
}


/* Distruttore: (per memoria)
 *  - nothing to do.
 */



/** CLASSE SubObject: ASSEGNAZIONE **/

/* Salva il numero del materiale
 */
void SubObject::setNumMat(u16 _numMat) {
    numMat=_numMat;
}


/* Cambia l'n-esimo vertexIdx della k-esima faccia
 */
void SubObject::setVertIdx(u16 faccia, u8 n, u16 vertIdx) {
    Vett3<u16> *oneFace=faces.getElem(faccia);
    switch(n) {
        case 1:
            return oneFace->setX(vertIdx);
            break;
        case 2:
            return oneFace->setY(vertIdx);
            break;
        case 3:
            return oneFace->setZ(vertIdx);
            break;
        default:
            throw Errore(faces.getErrCode(), ERR_OP_IMPORT_VTX, ERR_TP_UNEXPECTED_NUM);
    }
}



/** CLASSE SubObject: INTERROGAZIONE **/

/* Restituisce il numero del materiale
 */
u16 SubObject::getNumMat(void) {
    return numMat;
}


/* Restituisce il numero di facce salvate nel subobject
 */
u16 SubObject::getNumFaces(void) {
    return faces.getLung();
}


/* Restituisce l'n-esimo indice (n=1-3) del vertice della faccia
 */
u16 SubObject::getVertIdx(u16 faccia, u8 n) {
    Vett3<u16> *oneFace=faces.getElem(faccia);
    switch(n) {
        case 1:
            return oneFace->getX();
            break;
        case 2:
            return oneFace->getY();
            break;
        case 3:
            return oneFace->getZ();
            break;
        default:
            throw Errore(faces.getErrCode(), ERR_OP_IMPORT_VTX, ERR_TP_UNEXPECTED_NUM_2);
    }
}


/* Restituisce l'indirizzo di un elemento delle liste private
 */
Vett3<f32> *SubObject::getNormal(u16 faccia) {
    return normals.getElem(faccia);
}



/** CLASSE SubObject: NUOVO ELEMENTO **/

/* Aggiunge una faccia
 */
void SubObject::addFace(u16 vertexIdx[]) {
    faces.addElem()->set(vertexIdx[0], vertexIdx[1], vertexIdx[2]);
}


/* Aggiunge una normale
 */
void SubObject::addNormal(f32 x, f32 y, f32 z) {
    normals.addElem()->set(x, y, z);
}



/** CLASSE Subobject: CANCELLAZIONE LISTE **/

/* Cancella la lista delle facce
 */
void SubObject::eraseFaces(void) {
    faces.eraseAll();
}


/* Cancella la lista delle normali
 */
void SubObject::eraseNormals(void) {
    normals.eraseAll();
}





/** *** *** *** *** *** *** *** *** *** *** *** *** *** ***
 **
 **                    CLASSE Key
 **
 ** *** *** *** *** *** *** *** *** *** *** *** *** *** ***
 **/


/** CLASSE Key: COSTRUTTORE & DISTRUTTORE **/

/* Costruttore: (per memoria)
 *  - nothing to do.
 */

/* Distruttore: (per memoria)
 *  - nothing to do.
 */



/** CLASSE Key: ASSEGNAZIONE **/

/* Salva il numero del keyframe
 */
void Key::setNumKey(u8 n) {
    numKey=n;
}



/** CLASSE Key: INTERROGAZIONE **/

/* Restituisce il numero del keyframe
 */
u8 Key::getNumKey(void) {
    return numKey;
}






/** *** *** *** *** *** *** *** *** *** *** *** *** *** ***
 **
 **                     CLASSE Object
 **
 ** *** *** *** *** *** *** *** *** *** *** *** *** *** ***
 **/



/** CLASSE Object: COSTRUTTORE & DISTRUTTORE **/

/* Costruttore:
 *  - inizializza i codici di errore delle liste.
 */
Object::Object(void) {
    vertices.setErrCode(ERR_CL_ME_VERTEX);
    uvVertices.setErrCode(ERR_CL_ME_UVVERTEX);
    subObjects.setErrCode(ERR_CL_ME_SUBOBJECT);
    rotKeys.setErrCode(ERR_CL_ME_ROTKEY);
    linKeys.setErrCode(ERR_CL_ME_LINKEY);
}


/* Distruttore: (per memoria)
 *  - nothing to do.
 */



/** CLASSE Object: ASSEGNAZIONE **/

/* Salva il nome dell'oggetto
 */
void Object::setObjName(string _objName) {
    objName=_objName;
}


/* Salva il nome dell'oggetto
 */
void Object::setParent(s16 _parent) {
    parent=_parent;
}



/** AGGIUNGE UN ELEMENTO ALLE LISTE PRIVATE **/
/* Aggiunge un subobject
 */
SubObject *Object::addSubObj(void) {
    try {
        return subObjects.addElem();
    }
    catch(Errore err) {
        //Aggiunge le informazioni e poi rilancia l'errore
        err.setErrClass(subObjects.getErrCode());
        throw err;
    }
}

/* Aggiunge un vertice
 */
void Object::addPoint(f32 x, f32 y, f32 z) {
    try {
        vertices.addElem()->set(x, y, z);
    }
        catch(Errore err) {
        //Aggiunge le informazioni e poi rilancia l'errore
        err.setErrClass(vertices.getErrCode());
        throw err;
    }
}

/* Aggiunge un vertice uv
 */
void Object::addUvPoint(f32 u, f32 v) {
    try {
        uvVertices.addElem()->set(u, v);
    }
        catch(Errore err) {
        //Aggiunge le informazioni e poi rilancia l'errore
        err.setErrClass(uvVertices.getErrCode());
        throw err;
    }
}

/* Aggiunge un linKey
 */
void Object::addLinKey(u8 n, Vett3<f32> *trasl) {
    try {
        LinKey *key=linKeys.addElem();
        key->setNumKey(n);
        key->set(trasl->getX(), trasl->getY(), trasl->getZ());
    }
        catch(Errore err) {
        //Aggiunge le informazioni e poi rilancia l'errore
        err.setErrClass(linKeys.getErrCode());
        throw err;
    }
}

/* Aggiunge un rotKey
 */
void Object::addRotKey(u8 n, Vett4<f32> *quat) {
    try {
        RotKey *key=rotKeys.addElem();
        key->setNumKey(n);
        key->set(quat->getX(), quat->getY(), quat->getZ(), quat->getW());
    }
        catch(Errore err) {
        //Aggiunge le informazioni e poi rilancia l'errore
        err.setErrClass(rotKeys.getErrCode());
        throw err;
    }
}



/** CLASSE Object: INTERROGAZIONE LISTE PRIVATE **/

/* Restituisce l'indirizzo di un elemento delle liste private
 */
SubObject *Object::getSubObj(u8 n) {
    return subObjects.getElem(n);
}

Vett3<f32> *Object::getPoint(u16 n) {
    return vertices.getElem(n);
}

Vett2<f32> *Object::getUvPoint(u16 n) {
    return uvVertices.getElem(n);
}

RotKey *Object::getRotKey(u8 n) {
    return rotKeys.getElem(n);
}

LinKey *Object::getLinKey(u8 n) {
    return linKeys.getElem(n);
}


/* Restituisce la lunghezza delle liste private
 */
u8 Object::getNumSubObjs(void) {
    return subObjects.getLung();
}

u16 Object::getNumPoints(void) {
    return vertices.getLung();
}

u16 Object::getNumUvPoints(void) {
    return uvVertices.getLung();
}

u8 Object::getNumRotKeys(void) {
    return rotKeys.getLung();
}

u8 Object::getNumLinKeys(void) {
    return linKeys.getLung();
}



/** CLASSE Object: INTERROGAZIONE **/

/* Restituisce il numero dell'oggetto parent
 */
s16 Object::getParent(void) {
    return parent;
}


/* Restituisce il nome dell'oggetto
 */
string Object::getObjName(void) {
    return objName;
}


/* Restituisce l'indirizzo della matrice di trasformazione
 */
Matr4x4<f32> *Object::getAssMatr(void) {
    return &assMatr;
}

Matr4x4<f32> *Object::getRelMatr(void) {
    return &relMatr;
}



/** CLASSE Object: CANCELLAZIONE LISTE **/

/* Cancella le liste
 */
void Object::eraseSubObjs(void) {
    subObjects.eraseAll();
}

void Object::erasePoints(void) {
    vertices.eraseAll();
}

void Object::eraseUvPoints(void) {
    uvVertices.eraseAll();
}

void Object::eraseRotKeys(void) {
    rotKeys.eraseAll();
}

void Object::eraseLinKeys(void) {
    linKeys.eraseAll();
}



/** *** *** *** *** *** *** *** *** *** *** *** *** *** ***
 **
 **                     CLASSE Mesh
 **
 ** *** *** *** *** *** *** *** *** *** *** *** *** *** ***
 **/


/** CLASSE Mesh: COSTRUTTORE & DISTRUTTORE **/

/* Costruttore:
 *  - inizializza i codici di errore delle liste.
 */
#ifdef CONSOLE
Mesh::Mesh(TSshape &_tsShape, ostream &_log, bool _logFile) : tsShape(_tsShape), log(_log)
#else
Mesh::Mesh(TSshape &_tsShape, ostream &_log, bool _logFile, ProgressDialog &_pgsDlg) : tsShape(_tsShape), log(_log), pgsDlg(_pgsDlg)
#endif
{
    materials.setErrCode(ERR_CL_ME_MATERIAL);
    objects.setErrCode(ERR_CL_ME_OBJECT);
    logFile=_logFile;
}

/* Distruttore: (per memoria)
 *  - nothing to do.
 */


/** CLASSE MESH: CONVERSIONE TS <-> MESH - dati comuni a tutti i LODs **/

/* Importa la lista dei materiali dalla TSshape:
 *  - crea l'elenco dei materiali a partire dall'elenco dei prim_states;
 *  - scarta i materiali doppi.
 */
void Mesh::importMat(void) {
    try {
        u16 numPSs=0;
        u16 numMat=0;
        TSprimState *tsPScurr=NULL, *tsPScheck=NULL;
        Material *nuovoMat=NULL;
        //Numero di prim_states
        numPSs=tsShape.getTSprimStates()->getLung();
        /* --------------------------- *
         *  SALVATAGGIO DEI MATERIALI  *
         * --------------------------- */
        //Percorre la lista dei prim_states
        for(u16 i=0; i<numPSs; i++) {
            //Prende l'indirizzo dell'i-esimo prim_state
            tsPScurr=tsShape.getTSprimStates()->getElem(i);
            //Controlla se nell'elenco dei prim_states, tra quelli gi letti (condizione
            // j<i), ce n' un altro uguale. Due prim_states sono uguali se hanno tutti i
            // campi uguali tranne nome e vtxStateIdx. Il vtx_state infatti contiene anche
            // il riferimento al numero dell'oggetto; occorre quindi verificare se tutte
            // le voci dei due vtx_states sono uguali tranne il numero dell'oggetto.
            // Anche il nome pu essere differente (ma sarebbe una condizione un po'
            // strana e anomala)
            bool isEqual=false;
            //Cerca tra i prim_states precedenti
            for(u16 j=0; j<i; j++) {
                //Prende l'indirizzo di uno dei prim_state precedenti l'i-esimo
                tsPScheck=tsShape.getTSprimStates()->getElem(j);
                //Controlla i campi dei due prim_states
                if(tsPScurr->getShaderIdx()==tsPScheck->getShaderIdx()) {
                    if(tsPScurr->getTextureIdx()==tsPScheck->getTextureIdx()) {
                        if(tsPScurr->getAlphaMode()==tsPScheck->getAlphaMode()) {
                            if(tsPScurr->getLightCfgIdx()==tsPScheck->getLightCfgIdx()) {
                                if(tsPScurr->getZBuffMode()==tsPScheck->getZBuffMode()) {
                                    //Controlla i campi 'matShining' dei due vtxStateIdx
                                    u8 vsiCurr=0, vsiCheck=0;
                                    TSvtxState *tsVScurr=NULL, *tsVScheck=NULL;
                                    //Numero dei due vtx_states
                                    vsiCurr=tsPScurr->getVtxStateIdx();
                                    vsiCheck=tsPScheck->getVtxStateIdx();
                                    //Puntatori ai vtx_states
                                    tsVScurr=tsShape.getTSvtxStates()->getElem(vsiCurr);
                                    tsVScheck=tsShape.getTSvtxStates()->getElem(vsiCheck);
                                    //Controllo
                                    if(tsVScurr->getMatShining()==tsVScheck->getMatShining()) {
                                        isEqual=true;
                                    }
                                }
                            }
                        }
                    }
                } //Fine degli if
            } //Fine ciclo controllo prim_states
            //Se tra i prim_states precedenti non ne trova uno uguale, crea un nuovo materiale
            if(isEqual==false) {
                //Copia le caratteristiche: variabili
                u8 shaderIdx=0, textureIdx=0, alphaMode=0, lightCfgIdx=0, zBuffMode=0, vtxStateIdx=0;
                u8 imageIdx=0, filterModeIdx=0;
                s8 matShining=0;
                f32 mipMapLevel=0.0f;
                string  matName, imageName, filterName, shaderName;
                //Copia le caratteristiche: tsPrimStates
                matName=tsPScurr->getMatName();
                shaderIdx=tsPScurr->getShaderIdx();
                textureIdx=tsPScurr->getTextureIdx();
                alphaMode=tsPScurr->getAlphaMode();
                lightCfgIdx=tsPScurr->getLightCfgIdx();
                zBuffMode=tsPScurr->getZBuffMode();
                vtxStateIdx=tsPScurr->getVtxStateIdx();
                //Copia le caratteristiche: tsVtxStates
                matShining=tsShape.getTSvtxStates()->getElem(vtxStateIdx)->getMatShining();
                //Copia le caratteristiche: tsTextures
                if(tsPScurr->getHasTexture()) {
                    imageIdx=tsShape.getTStextures()->getElem(textureIdx)->getImageIdx();
                    filterModeIdx=tsShape.getTStextures()->getElem(textureIdx)->getFilterIdx();
                    mipMapLevel=tsShape.getTStextures()->getElem(textureIdx)->getMipLevel();
                    //Copia le caratteristiche: tsImages
                    imageName=tsShape.getTSimages()->getElem(imageIdx)->getString();
                    //Copia le caratteristiche: tsFilters
                    filterName=tsShape.getTSfilters()->getElem(filterModeIdx)->getString();
                }
                else {
                    //Materiale senza texture
                    imageName="";
                    filterName="";
                    shaderName="";
                    mipMapLevel=0;
                }
                //Copia le caratteristiche: tsShaders
                shaderName=tsShape.getTSshaders()->getElem(shaderIdx)->getString();
                //Crea un nuovo materiale nella lista
                nuovoMat=materials.addElem();
                nuovoMat->setMatName(matName);
                nuovoMat->setImageName(imageName);
                nuovoMat->setFilterName(filterName);
                nuovoMat->setShaderName(shaderName);
                nuovoMat->setAlphaMode(alphaMode);
                nuovoMat->setZBuffMode(zBuffMode);
                nuovoMat->setMatShining(matShining);
                nuovoMat->setMipMapLevel(mipMapLevel);
                //Assegna il colore casuale al materiale
                u8 color[3];
                color[0]=(rand()%255);
                color[1]=(rand()%255);
                color[2]=(rand()%255);
                nuovoMat->setColor(color[0], color[1], color[2]);
                //Il numero del materiale appena salvato corrrisponde con la lunghezza della lista -1
                numMat=materials.getLung()-1;
                //Salva il numero del materiale nel primstate
                tsPScurr->setNumMat(numMat);
            }
            else {
                //Se ha trovato un prim_state uguale, copia la corrispondenza con il materiale
                //NB: il ciclo trova il primo prim_state uguale che  anche l'unico usato per
                // creare il materiale e quindi contiene NumMat.
                tsPScurr->setNumMat(tsPScheck->getNumMat());
            }
        } //Fine ciclo for dei tsPrimStates
        //Controlla la lista dei materiali appena creata
        u16 mem=0;
        mem=materials.check();
        //cout<<">>>>>> Numero materiali: "<<materials.getLung();
    } //End of try
    catch(Errore err) {
        //Aggiunge le informazioni (se assenti) e poi rilancia l'errore
        if(err.getErrClass()==ERR_CL_NULL) {
            err.setErrClass(objects.getErrCode());
        }
        throw err;
    }
}


/* Importa la lista degli oggetti con gerarchia dalla TSshape
 */
void Mesh::importObj(void) {
    try {
        u8 numTsObjs=0;
        TSobject *tsObject=NULL;
        /* --------- *
         *  OGGETTI  *
         * --------- */
        numTsObjs=tsShape.getTSobjects()->getLung();
        for(u8 i=0; i<numTsObjs; i++) {
            //Prende l'indirizzo dell'i-esimo oggetto
            tsObject=tsShape.getTSobjects()->getElem(i);
            //Prende il nome dell'oggetto
            string name;
            name=tsObject->getObjName();
            objects.addElem()->setObjName(name);
        } //Fine ciclo tsObjects
        //Controlla la lista degli oggetti appena creata
        u8 mem=0;
        mem=objects.check();
        /* ----------- *
         *  GERARCHIA  *
         * ----------- */
        u8 numHiers=0;
        s8 parent=0;
        TSlod *tsFirstLod=NULL;
        //Prende l'indirizzo del primo Lod (che contiene sicuramente la gerarchia di tutti gli oggetti)
        tsFirstLod=tsShape.getTSlods()->getElem(1);
        numHiers=tsFirstLod->getHierarchy()->getLung();
        if(numTsObjs!=numHiers) {
            //Devono essere uguali perch entrambi rappresentano il numero di oggetti
            throw Errore(ERR_OP_IMPORT_OBJ, ERR_TP_UNEXPECTED_NUM);
        }
        for(u8 i=0; i<numTsObjs; i++) {
            //Prende il numero dell'oggetto genitore
            parent=tsFirstLod->getHierarchy(i);
            //Salva nell'i-esimo oggetto
            objects.getElem(i)->setParent(parent);
        } //Fine ciclo tsObjects
        //Controlla la lista degli oggetti appena creata
    } //End of try
    catch(Errore err) {
        //Aggiunge le informazioni (se assenti) e poi rilancia l'errore
        if(err.getErrClass()==ERR_CL_NULL) {
            err.setErrClass(ERR_CL_ME_MESH);
        }
        throw err;
    }
}


/* Importa le matrici rotaz e trasl degli oggetti dalla TSshape:
 *  - rispetta la gerarchia degli oggetti.
 */
void Mesh::importMatrices(void) {
    try {
        //Matrice di trasformazione per inversione y <-> z: solo ROTAZIONE, 90 in senso antiorario su x
        // NB: l'inversione lungo x  messa nel SCL_TRACK_KEY
        Matr4x4<f32> trasf;
        /*trasf.set(1.0f, 0.0f, 0.0f, 0.0f,
                  0.0f, 0.0f, 1.0f, 0.0f,
                  0.0f, 1.0f, 0.0f, 0.0f,
                  0.0f, 0.0f, 0.0f, 1.0f);*/
        trasf.set(1.0f,  0.0f,  0.0f, 0.0f,
                  0.0f,  0.0f, -1.0f, 0.0f,
                  0.0f,  1.0f,  0.0f, 0.0f,
                  0.0f,  0.0f,  0.0f, 1.0f);
        /*trasf.set(1.0f,  0.0f,  0.0f, 0.0f, //Matrice identit (nessuna trasformazione)
                  0.0f,  1.0f,  0.0f, 0.0f,
                  0.0f,  0.0f,  1.0f, 0.0f,
                  0.0f,  0.0f,  0.0f, 1.0f);*/
        //Objects mesh (destinazione)
        u8 numObjs=objects.getLung();
        for(u8 i=0; i<numObjs; i++) {
            //Prende l'indirizzo dell'i-esimo object di mesh e di TS
            Object *oneObj=objects.getElem(i);
            TSobject *oneTSobj=tsShape.getTSobjects()->getElem(i);
            //Controlla se ha parents
            s8 parent=oneObj->getParent();
            if(parent>=0) {
                /*
                //Ha uno o pi parents; NB: poich nel file shape sono elencati in modo che tutti i figli seguano i padri
                //  sufficiente considerare solo la matrice dell'oggetto indicato da 'Parent', la quale, a sua volta, 
                // gi stata aggiornata in precedenza se figlia di altri oggetti
                //Prende l'indirizzo dell'oggetto TS parent dell'i-esimo object
                //TSobject *tsParent=tsShape.getTSobjects()->getElem(parent);
                //Prende l'indirizzo dell'oggetto TS child (uguale all'i-esimo object)
                TSobject *tsChild=tsShape.getTSobjects()->getElem(i);
                //NB: la rel non va ruotata perch max tiene conto della gerarchia
                copyMatr( tsChild->getMatr(), oneObj->getRelMatr() );
                //prodRxC( tsChild->getMatr(), &trasf, oneObj->getRelMatr() );
                //Esegue il prodotto delle due matrici NOTA: non dovrebbe essere al contrario? Chil*Par
                //prodRxC(tsParent->getMatr(), tsChild->getMatr(), oneObj->getMatr());
                //prodRxC(tsChild->getMatr(), tsParent->getMatr(), oneObj->getMatr());
                Object *parentObj=objects.getElem(parent);
                //NB: la ass non va ruotata perch la rotazione  gi compresa nella ass parent
                prodRxC( tsChild->getMatr(), parentObj->getAssMatr(), oneObj->getAssMatr() );
                //cout<<"Object: "<<oneObj->getObjName()<<endl;
                //copyMatr(tsChild->getMatr(), oneObj->getMatr());
                */
                copyMatr( oneTSobj->getMatr(), oneObj->getRelMatr() );
                Object *parentObj=objects.getElem(parent);
                prodRxC( oneTSobj->getMatr(), parentObj->getAssMatr(), oneObj->getAssMatr() );
            }
            else {
                /*//L'i-esimo object  la root
                //Prende l'indirizzo dell'oggetto TS child
                TSobject *tsRoot=tsShape.getTSobjects()->getElem(i);
                //Copia la matrice
                //copyMatr( tsRoot->getMatr(), oneObj->getAssMatr() );
                prodRxC( tsRoot->getMatr(), &trasf, oneObj->getAssMatr() );
                //copyMatr( oneObj->getAssMatr(), oneObj->getRelMatr() );
                prodRxC( tsRoot->getMatr(), &trasf, oneObj->getRelMatr() );
                */
                prodRxC( oneTSobj->getMatr(), &trasf, oneObj->getAssMatr() );
                prodRxC( oneTSobj->getMatr(), &trasf, oneObj->getRelMatr() );
            }
        } //Fine ciclo objects
    } //End of try
    catch(Errore err) {
        //Aggiunge le informazioni (se assenti) e poi rilancia l'errore
        if(err.getErrClass()==ERR_CL_NULL) {
            err.setErrClass(ERR_CL_ME_MATR);
        }
        throw err;
    }
}


/* Importa i keyframes delle animazioni di TS se presenti:
 *  - se l'oggetto non  animato inserisce un key per posizionare correttamente gli oggetti child.
 */
void Mesh::importKeys(void) {
    try {
        //Questa variabile contiene l'indice pi alto tra tutti i keys; determina il numero di keys del file 3ds
        u8 maxKeyNum=0;
        //Numero di animNodes TS
        u8 numANs=tsShape.getTSanimNodes()->getLung();
        //Numero di oggetti contenuti in mesh
        u8 numObjs=objects.getLung();
        for(u8 i=0; i<numObjs; i++) {
            //Prende l'indirizzo dell'i-esimo object
            Object *oneObj=objects.getElem(i);
            //Cerca tra gli animNodes di TS (se esistono) quello (se esiste) con nome uguale all'oggetto
            s16 nObj=-1;
            string tsObjName, objName=oneObj->getObjName();
            //cout<<endl<<"Nome: "<<objName<<endl;
            TSanimNode *oneAN=NULL;
            for(u8 j=0; j<numANs; j++) {
                //Prende l'indirizzo del j-esimo animNode
                oneAN=tsShape.getTSanimNodes()->getElem(j);
                tsObjName=oneAN->getObjName();
                if(objName==tsObjName) {
                    nObj=j;
                    break;
                }
            }
            //Se ha trovato l'animNode corrispondente all'i-esimo oggetto, copia (se esistono) rotKeys e linKeys;
            // Se non l'ha trovato o mancano lin o rot keys deve salvare il key corrispondente alla matrice
            // relativa, quindi ora converte la matrice in quaternione e vettore traslazione.
            // Se nObj>0, oneAN contiene gi l'indirizzo dell'animNode da usare!
            Vett4<f32> quatPivot;
            Vett3<f32> traslPivot;
            matr2quat( oneObj->getRelMatr(), &quatPivot, &traslPivot );
            if(nObj>0 ) {
                //Controlla se l'animNode contiene keys
                u8 numRotKeys=oneAN->getNumRotKeys(), numLinKeys=oneAN->getNumLinKeys();
                //cout<<"numRK: "<<(u16)numRotKeys<<", NumLK: "<<(u16)numLinKeys<<endl;
                //RotKeys
                if(numRotKeys>0) {
                    //Ci sono rotKeys
                    for(u8 j=0; j<numRotKeys; j++) {
                        //Indirizzo del j-esimo rotKey
                        TSrotKey *oneRK=oneAN->getRotKey(j);
                        //Quaternione e numero del j-esimo rotKey
                        u8 nRK=oneRK->getNumKey();
                        Vett4<f32> *currQuat=oneRK->getQuat();
                        //Elaborazione dei dati letti dal file shape per renderli compatibili con 3ds
                        Vett4<f32> quat, *prevQuat=NULL;
                        if(j!=0) {
                            //Nei RK successivi occorre salvare il RK corrispondente alla rotazione relativa
                            // tra i due RK
                            prevQuat=oneAN->getRotKey(j-1)->getQuat();
                            conjQuat(prevQuat);
                            //cout<<" => Quaternione precedente coniugato:"<<endl;
                            //cout<<'\t'<<prevQuat->getX()<<'\t'<<prevQuat->getY()<<'\t'<<prevQuat->getZ()<<'\t'<<prevQuat->getW()<<endl;
                            //Calcola il coniugato del RK corrente
                            //conjQuat(currQuat);
                            //cout<<" => Quaternione attuale:"<<endl;
                            //cout<<'\t'<<currQuat->getX()<<'\t'<<currQuat->getY()<<'\t'<<currQuat->getZ()<<'\t'<<currQuat->getW()<<endl;
                            //Prodotto tra il quat del RK precedente e il coniugato del RK corrente
                            //prodQuat(currQuat, prevQuat, &quat);
                            prodQuat(prevQuat, currQuat, &quat);
                        }
                        else {
                            //E' il primo RK, non serve fare elaborazioni
                            quat.set( currQuat->getX(), currQuat->getY(), currQuat->getZ(), currQuat->getW() );
                        }
                        //Salva il quaternione
                        //cout<<" => Quaternione salvato:"<<endl;
                        //cout<<'\t'<<quat.getX()<<'\t'<<quat.getY()<<'\t'<<quat.getZ()<<'\t'<<quat.getW()<<endl;
                        oneObj->addRotKey(nRK, &quat);
                        //cout<<"    Corrispondente al vettore eaa:"<<endl;
                        Vett4<f32> eaa;
                        quat2eaa(&quat, &eaa);
                        //cout<<'\t'<<eaa.getW()<<";\t"<<eaa.getX()<<'\t'<<eaa.getY()<<'\t'<<eaa.getZ()<<endl;
                        //cout<<" + Salvato RK no."<<(u16)nRK<<" letto da file."<<endl;
                        //Controlla ed aggiorna eventualmente il numero massimo di keys
                        if(maxKeyNum<nRK) maxKeyNum=nRK;
                    }
                }
                else {
                    //Non ci sono rotKeys => salva il quaternione corrispondente alla matrice
                    oneObj->addRotKey(0, &quatPivot);
                    //cout<<" + Salvato RK corrispondente alla matrice."<<endl;
                }
                //LinKeys
                if(numLinKeys>0) {
                    //Ci sono linKeys
                    for(u8 j=0; j<numLinKeys; j++) {
                        //Indirizzo del j-esimo rotKey
                        TSlinKey *oneLK=oneAN->getLinKey(j);
                        //Vettore e numero del j-esimo linKey
                        Vett3<f32> *trasl=oneLK->getTrasl();
                        u8 nLK=oneLK->getNumKey();
                        //Salva il vettore
                        oneObj->addLinKey(nLK, trasl);
                        //cout<<" + Salvato LK no."<<(u16)nLK<<" letto da file."<<endl;
                        //Controlla ed aggiorna eventualmente il numero massimo di keys
                        if(maxKeyNum<nLK) maxKeyNum=nLK;
                    }
                }
                else {
                    //Non ci sono linKeys => salva il vettore corrispondente alla matrice
                    oneObj->addLinKey(0, &traslPivot);
                    //cout<<" + Salvato LK corrispondente alla matrice."<<endl;
                }
            }
            else {
                //Salva i keys corrispondenti alla matrice relativa dell'oggetto
                oneObj->addRotKey(0, &quatPivot);
                oneObj->addLinKey(0, &traslPivot);
                //cout<<" => Quaternione salvato:"<<endl;
                //cout<<'\t'<<quatPivot.getX()<<'\t'<<quatPivot.getY()<<'\t'<<quatPivot.getZ()<<'\t'<<quatPivot.getW()<<endl;
                //cout<<" + Salvati RK e LK corrispondenti alla matrice."<<endl;
            }
            //int y; cin>>y;
        } //Fine ciclo objects
    } //End of try
    catch(Errore err) {
        //Aggiunge le informazioni (se assenti) e poi rilancia l'errore
        if(err.getErrClass()==ERR_CL_NULL) {
            err.setErrClass(ERR_CL_ME_KEY);
        }
        throw err;
    }
}



/** CLASSE MESH: CONVERSIONE TS <-> MESH - dati relativi a un LOD **/

/* Importa la lista delle facce dalla TSshape:
 *  - importa le facce suddividendole per materiale e per oggetto.
 */
void Mesh::importFaces(u8 numLod) {
    try {
        //Prende l'indirizzo dell'i-esimo lod
        TSlod *oneLod=NULL;
        oneLod=tsShape.getTSlods()->getElem(numLod);
        //Prende l'indirizzo della lista 'tsFaces' del lod, per comodit
        Lista<TSface> *tsFaces=NULL;
        tsFaces=oneLod->getTSfaces();
        //Prende l'indirizzo della lista 'tsNormals' della shape, per comodit
        Lista<TSvector> *tsNormals=NULL;
        tsNormals=tsShape.getTSnormals();
        //Lunghezza della lista
        u16 numFaces=0;
        numFaces=tsFaces->getLung();
        //Ora per ogni faccia legge il codice del prim_state che fornisce direttamente il
        // numero del materiale da applicare (e quindi il subobject della mesh cui appartiene)
        // e indiretamente il numero dell'oggetto cui la faccia appartieene (tramite vtx_state
        // index).
        u8 numPS=255, numMat=0, numVS=0, numObj=0, numSubObjs=0;
        TSface *oneFace=NULL; //L'indirizzo della faccia corrispondente a i
        TSvector *oneNormal=NULL; //L'indirizzo della normale della faccia oneFace
        TSprimState *onePS=NULL; //L'indirizzo del primState corrispondente a numPS
        TSvtxState *oneVS=NULL; //L'indirizzo del vtxState corrispondente a numVS
        Object *theObj=NULL;  //L'indirizzo dell'oggetto di mesh da usare per salvare la faccia
        SubObject *theSubObj=NULL; //L'indirizzo del subobject di mesh da usare per salvare la faccia
        #ifndef CONSOLE
        //Inizializza la barra
        pgsDlg.startOperation(20.0f/numFaces, NULL, false);
        #endif
        for(u16 i=0; i<numFaces; i++) {
            //Indirizzo dell'i-esima faccia
            oneFace=tsFaces->getElem(i);
            //Codice del primState della faccia: se  diverso cambia i codici mat e obj e sceglie
            // l'oggetto e il subobject corretti.
            if(numPS!=oneFace->getPrimStateIdx()) {
                //Il codice del primState  diverso da quello della faccia precedente
                numPS=oneFace->getPrimStateIdx();
                //Indirizzo del primState
                onePS=tsShape.getTSprimStates()->getElem(numPS);
                //Numero del materiale
                numMat=onePS->getNumMat();
                //Codice del vtxState
                numVS=onePS->getVtxStateIdx();
                //Indirizzo del vtxState
                oneVS=tsShape.getTSvtxStates()->getElem(numVS);
                //Infine, numero dell'oggetto
                numObj=oneVS->getObjectIdx();
                //Prende l'indirizzo dell'oggeto di mesh da utilizzare
                theObj=objects.getElem(numObj);
                //Cerca tra i subobjects di theObj se ce n' uno con mat==numMat, altrimenti
                // ne crea uno nuovo
                numSubObjs=theObj->getNumSubObjs();
                theSubObj=NULL; //Inizializza il puntatore altrimenti non funziona il controllo 'if' dopo il ciclo
                for(u8 j=0; j<numSubObjs; j++) {
                    theSubObj=theObj->getSubObj(j);
                    if(theSubObj->getNumMat()==numMat) break;
                    else theSubObj=NULL;
                }
                if(theSubObj==NULL) {
                    //Deve creare un nuovo subobject
                    theSubObj=theObj->addSubObj();
                    //Assegna il numero del materiale al subobject
                    theSubObj->setNumMat(numMat);
                }
                //Adesso pu salvare la faccia nel subobject => esce dall'if
            }
            //A questo punto  possibile salvare la faccia (se  passato dall'if ha cercato il
            // subobject corretto, altrimenti quello del passo precedente era gi adatto) nel
            // subobject puntato da theSubObj.
            theSubObj->addFace( oneFace->getVerticesIdx() );
            //Adesso salva la normale della faccia
            //Prende la normale di TS
            oneNormal=tsNormals->getElem( oneFace->getNormalIdx() );
            /*//Trasforma la normale secondo la matrice assoluta dell'oggetto
            Vett3<f32> normal;
            trasfVett( theObj->getAssMatr(), oneNormal, &normal );*/
            //Salva la normale
            //theSubObj->addNormal( normal.getX(), normal.getY(), normal.getZ() ); //Trasformata
            theSubObj->addNormal( oneNormal->getX(), oneNormal->getY(), oneNormal->getZ() );
            #ifndef CONSOLE
            //Incrementa la progress bar
            pgsDlg.incrementStep();
            #endif
        } //Fine ciclo for per le tsFaces
        //Controllo
        u16 mem=0;
        u8 numObjs=0;
        numObjs=objects.getLung();
        for(u8 i=0; i<numObjs; i++) {
            theObj=objects.getElem(i);
            numSubObjs=theObj->getNumSubObjs();
            for(u8 j=0; j<numSubObjs; j++) {
                theSubObj=theObj->getSubObj(j);
                mem+=theSubObj->getNumFaces();
            }
        }
        if(mem!=numFaces) {
            //Devono essere uguali perch entrambi rappresentano il numero di oggetti
            throw Errore(ERR_OP_IMPORT_FACES, ERR_TP_DIFF_LENGTH);
        }
        //Fine importazione facce
    } //End of try
    catch(Errore err) {
        //Aggiunge le informazioni (se assenti) e poi rilancia l'errore
        if(err.getErrClass()==ERR_CL_NULL) {
            err.setErrClass(ERR_CL_ME_MESH);
        }
        throw err;
    }
}


/* Importa la lista dei vertici dalla TSshape:
 *  - importa i vertici necessari per le facce importate.
 */
void Mesh::importVertices(u8 numLod) {
    try {
        //Prende l'indirizzo dell'i-esimo lod
        TSlod *oneLod=NULL;
        oneLod=tsShape.getTSlods()->getElem(numLod);
        //Prende l'indirizzo della lista tsVertices del lod corrente, per comodit
        Lista<TSvertex> *tsVertices=NULL;
        tsVertices=oneLod->getTSvertices();
        //Prende gli indirizzi delle liste dei punti e punti uv della shape
        Lista<TSvector> *tsPoints=NULL;
        Lista<TSuvPoint> *tsUvPoints=NULL;
        tsPoints=tsShape.getTSpoints();
        tsUvPoints=tsShape.getTSuvPoints();
        //Lunghezza lista TSvertices e puntatore TSvertex
        u16 numTSvertices=0;
        numTSvertices=tsVertices->getLung();
        TSvertex *oneTSvertex=NULL;
        //Puntatori oggetti TS:
        TSvector *oneTSpoint=NULL;
        TSuvPoint *oneTSuvPoint=NULL;
        //Variabili per scorrere la lista degli objects
        u8 numObjs=0, numSubObjs=0;
        u16 numFaces=0, vertexIdx=0, numVertIdxs=0, conta=0;
        Object *oneObj=NULL;
        SubObject *oneSubObj=NULL;
        //Lista dei vertex da salvare per questo oggetto
        Lista<u16> verticesIdxs(ERR_CL_ME_VERTIDXS);
        bool isNew=true;
        //Scorre la lista degli oggetti
        numObjs=objects.getLung();
        for(u8 i=0; i<numObjs; i++) {
            oneObj=objects.getElem(i);
            //cout<<endl<<endl<<"=> Object no."<<(int)i;
            //Scorre la lista dei subobjects dell'i-esimo object
            numSubObjs=oneObj->getNumSubObjs();
            for(u8 j=0; j<numSubObjs; j++) {
                //cout<<endl<<"\t=> Subobject no."<<(int)j;
                oneSubObj=oneObj->getSubObj(j);
                //Scorre la lista delle facce del j-esimo subobject
                numFaces=oneSubObj->getNumFaces();
                //cout<<endl<<"\t + Contiene "<<numFaces<<" facce.";
                #ifndef CONSOLE
                //Inizializza la barra
                pgsDlg.startOperation(20.0f/(numObjs*numSubObjs*numFaces), NULL, false);
                #endif
                for(u16 k=0; k<numFaces; k++) {
                    //Scorre i 3 vertici della k-esima faccia. NB: i vertici della faccia sono gi stati
                    // depurati dai doppioni durante la lettura.
                    for(u8 l=1; l<4; l++) {
                        vertexIdx=oneSubObj->getVertIdx(k, l);
                        //Scorre l'elenco dei vertex da salvare per l'i-esimo object
                        numVertIdxs=verticesIdxs.getLung();
                        isNew=true;
                        //cout<<endl<<"Faccia no."<<k<<" vert."<<(int)l<<"\t";
                        for(u16 m=numVertIdxs; m!=0; m--) {
                            //Cerca se il vertex richiesto  gi in elenco (NB: m=lung lista!)
                            if( vertexIdx == *verticesIdxs.getElem(m-1) ) {
                                //Il vertice  gi presente
                                isNew=false;
                                //cout<<"Cambiato "<<vertexIdx<<" con m-1="<<m-1<<endl;
                                //int p; cin>>p;
                                //Cambia il vertex salvato nella faccia ( l'(m-1)-esimo)
                                oneSubObj->setVertIdx(k, l, m-1);
                                //Salta al prossimo vertice (l+1 oppure k+1)
                                conta++;
                                break;
                            }
                        }
                        if(isNew==true) {
                            //Il vertice non  presente in elenco
                            //Nuovo vertexIdx da usare per il vertice l della faccia k
                            // NB: si mette getLung() e non getLung()-1 perch la lunghezza della lista
                            //  valutata prima di aggiungere il vertexIdx di questo passo.
                            u16 newVertexIdx=0;
                            newVertexIdx=verticesIdxs.getLung();
                            //Aggiunge il vertexIdx letto nella faccia alla lista
                            *verticesIdxs.addElem()=vertexIdx;
                            //Cambia il vertex salvato nella faccia
                            oneSubObj->setVertIdx(k, l, newVertexIdx);
                            //cout<<"Cambiato "<<vertexIdx<<" con "<<newVertexIdx<<endl;
                            //cout<<"\t\tSalvato "<< *verticesIdxs.getElem(newVertexIdx);
                            //cout<<" in posizione "<<newVertexIdx<<endl;
                        }
                        //cout<<"controllo faccia: "<<oneSubObj->getVertIdx(k, l)<<endl;
                    } //Fine for vertici faccia
                    //if(k==308) { int p; cin>>p; }
                    #ifndef CONSOLE
                    //Incrementa la progress bar
                    pgsDlg.incrementStep();
                    #endif
                } //Fine for faces
                //cout<<endl<<"\t + Vertices doppi: "<<conta<<" su "<<3*numFaces;
                conta=0;
            } //Fine for subObjects
            //Ha esaminato tutte le facce dell'object i-esimo => copia i vertici necessari
            numVertIdxs=verticesIdxs.getLung();
            //cout<<endl<<" + Richiesti: "<<numVertIdxs<<" vertices."<<endl;
            for(u16 j=0; j<numVertIdxs; j++) {
                //VertexIdx j-esimo
                u16 vertexIdx= *verticesIdxs.getElem(j);
                //Indirzzo del vertex
                oneTSvertex=tsVertices->getElem(vertexIdx);
                //Prende l'indirizzo del punto da copiare
                oneTSpoint=tsPoints->getElem( oneTSvertex->getPointIdx() );
                //cout<<endl<<"Richiesto vertex no."<<vertexIdx;
                //cout<<endl<<"\tPoint:\t"<<"no."<<oneTSvertex->getPointIdx()<<": ";
                //cout<<oneTSpoint->getX()<<" ; "<<oneTSpoint->getY()<<" ; "<<oneTSpoint->getZ()<<endl;
                oneTSuvPoint=tsUvPoints->getElem( oneTSvertex->getUvPointIdx() );
                //Aggiunge negli elenchi dell'object i punti
                /*Vett3<f32> point;
                trasfVett(oneObj->getAssMatr(), oneTSpoint, &point); //Trasformazione in coord globali
                oneObj->addPoint( point.getX(), point.getY(), point.getZ() ); //Trasformato!*/
                oneObj->addPoint( oneTSpoint->getX(), oneTSpoint->getY(), oneTSpoint->getZ() );
                oneObj->addUvPoint( oneTSuvPoint->getX(), 1-oneTSuvPoint->getY() ); //Trasformato!
                //if(j==0) { int p; cin>>p; }
            }
            //Cancella il contenuto di verticesIdxs per il prossimo object
            verticesIdxs.eraseAll();
        } //Fine for objects
        //Fine importazione vertici
    } //End of try
    catch(Errore err) {
        //Aggiunge le informazioni (se assenti) e poi rilancia l'errore
        if(err.getErrClass()==ERR_CL_NULL) {
            err.setErrClass(ERR_CL_ME_MESH);
        }
        throw err;
    }
}


/* Controlla l'orientamento della normale delle facce:
 *  - .
 */
void Mesh::checkNormals(void) {
    try {
        u8 numObjs=0, numSubObjs=0;
        u16 numFaces=0;
        u16 uguale=0, opposto=0, errore=0;
        //Scorre la lista degli objects
        numObjs=objects.getLung();
        for(u8 i=0; i<numObjs; i++) {
            Object *oneObj=objects.getElem(i);
            //Scorre la lista dei subobjects dell'i-esimo object
            numSubObjs=oneObj->getNumSubObjs();
            for(u8 j=0; j<numSubObjs; j++) {
                SubObject *oneSubObj=oneObj->getSubObj(j);
                //Scorre la lista delle facce del j-esimo subobject
                numFaces=oneSubObj->getNumFaces();
                #ifndef CONSOLE
                //Inizializza la barra
                pgsDlg.startOperation(5.0f/(numObjs*numSubObjs*numFaces), NULL, false);
                #endif
                for(u16 k=0; k<numFaces; k++) {
                    //Prende il vettore del primo vertice
                    u16 v1Idx=oneSubObj->getVertIdx(k, 1);
                    Vett3<f32> *v1=oneObj->getPoint(v1Idx);
                    //cout<<endl<<endl<<"v1:\t"<<"no."<<v1Idx<<": "<<v1->getX()<<" ; "<<v1->getY()<<" ; "<<v1->getZ()<<endl;
                    //Prende il vettore del secondo vertice
                    u16 v2Idx=oneSubObj->getVertIdx(k, 2);
                    Vett3<f32> *v2=oneObj->getPoint(v2Idx);
                    //cout<<"v2:\t"<<"no."<<v2Idx<<": "<<v2->getX()<<" ; "<<v2->getY()<<" ; "<<v2->getZ()<<endl;
                    //Prende il vettore del terzo vertice
                    u16 v3Idx=oneSubObj->getVertIdx(k, 3);
                    Vett3<f32> *v3=oneObj->getPoint(v3Idx);
                    //cout<<"v3:\t"<<"no."<<v3Idx<<": "<<v3->getX()<<" ; "<<v3->getY()<<" ; "<<v3->getZ()<<endl;
                    //Prende il vettore della normale della k-esima faccia
                    Vett3<f32> *norm=oneSubObj->getNormal(k);
                    //cout<<"nlet:\t"<<norm->getX()<<" ; "<<norm->getY()<<" ; "<<norm->getZ()<<endl;
                    //Calcola il vettori corrispondentl ai lato 1-2
                    f32 Dx=v2->getX() - v1->getX();
                    f32 Dy=v2->getY() - v1->getY();
                    f32 Dz=v2->getZ() - v1->getZ();
                    Vett3<f32> lato1(Dx, Dy, Dz);
                    //Calcola il vettori corrispondentl ai lato 2-3
                    Dx=v3->getX() - v2->getX();
                    Dy=v3->getY() - v2->getY();
                    Dz=v3->getZ() - v2->getZ();
                    Vett3<f32> lato2(Dx, Dy, Dz);
                    //Calcola la normale (prodotto vettoriale)
                    Vett3<f32> normCalc(0.0f, 0.0f, 0.0f);
                    prodVett(&lato1, &lato2, &normCalc);
                    //cout<<"ncal:\t"<<normCalc.getX()<<" ; "<<normCalc.getY()<<" ; "<<normCalc.getZ()<<endl;
                    //Confronta la normale calcolata con quella memorizzata (prodotto scalare);
                    // - se l'angolo  'circa' 0 (cos=1) => tutto OK;
                    // - se l'angolo  'circa' 180 (cos=-1) => invertire la sequenza vertici (es 2 con 1);
                    // - se l'alngolo  differente (cos!=+/-1) => errore!
                    // NB: prodotto scalare normalizzato fornisce il coseno dell'angolo
                    f32 cosAngolo=prodScalNorm(norm, &normCalc);
                    //cout<<"Coseno dell'angolo:"<<cosAngolo<<" => ";
                    if(cosAngolo>0.9888f) uguale++; //cout<<"uguale"<<endl;
                    else if(cosAngolo<-0.9888f) opposto++; //cout<<"opposti"<<endl;
                    else errore++; // { cout<<"Errore normale faccia "<<k<<", subobj no."<<(int)j<<"; cos="<<cosAngolo<<endl; errore++;}
                    //if(k==32 && i==10) {int y; cin>>y;}
                    if(cosAngolo<-0.9888f) {
                        //La normale  opposta => scambia due vertici
                        oneSubObj->setVertIdx(k, 1, v2Idx);
                        oneSubObj->setVertIdx(k, 2, v1Idx);
                    }
                    #ifndef CONSOLE
                    //Incrementa la progress bar
                    pgsDlg.incrementStep();
                    #endif
                }
            }
            //cout<<endl<<"Oggetto no."<<(int)i<<";\tuguale="<<uguale<<";\topposto="<<opposto<<";\terrore="<<errore;
            if(logFile) {
                log<<"\t\tObject no."<<(int)i<<", "<<(uguale+opposto+errore)<<" faces:"<<endl;
                log<<"\t\t\t- "<<uguale<<" normals ok"<<endl;
                log<<"\t\t\t- "<<opposto<<" normals flipped"<<endl;
                log<<"\t\t\t- Unable to check "<<errore<<" normals."<<endl;
            }
            uguale=0; opposto=0; errore=0;
            //int y; cin>>y;
        }
    } //End of try
    catch(Errore err) {
        //Aggiunge le informazioni (se assenti) e poi rilancia l'errore
        if(err.getErrClass()==ERR_CL_NULL) {
            err.setErrClass(ERR_CL_ME_MESH);
        }
        throw err;
    }
}




/** CLASSE MESH: PREPARAZIONE PER L'ESPORTAZIONE **/

/* Controlla i nomi dei materiali e delle immagini:
 *  - no comment.
 */
void Mesh::checkMatNames(void) {
    //Carattere da aggiungere come contatore ai nomi troncati
    u8 contaMat=0, senzaNome=0, contaImg=0;
    //Numero di materiali presenti
    u8 numMats=materials.getLung();
    for(u8 i=0; i<numMats; i++) {
        //Prende l'indirizzo dell'i-esimo materiale
        Material *oneMat=materials.getElem(i);
        //Nome del materiale
        string matName=oneMat->getMatName();
        if(matName.length()>15) {
            //Nome troppo lungo per il 3ds: va troncato
            if(logFile) log<<"\tMaterial no."<<(int)i<<", name '"<<matName<<"' too long. ";
            matName.resize(12);
            matName+="~";
            matName+=int2string(++contaMat, true);
            oneMat->setMatName(matName);
            if(logFile) log<<" New name: '"<<matName<<"'."<<endl;
        }
        else if(matName.length()==0) {
            //Se il nome  vuoto
            if(logFile) log<<"\tMaterial no."<<(int)i<<", unknown name. ";
            matName="Mat#"+int2string(++senzaNome, true);
            oneMat->setMatName(matName);
            if(logFile) log<<" New name: '"<<matName<<"'."<<endl;
        }
        //Controlla se il nome  univoco
        u8 j=0;
        while(j<i) {
            string anotherMatName=materials.getElem(j)->getMatName();
            if(anotherMatName==matName) {
                //Se l'i-esimo ha lo stesso nome del j-esimo, cambia il nome del primo
                if(logFile) log<<"\tMaterial no."<<(int)i<<", name duplicated. ";
                matName="Mat#"+int2string(++senzaNome, true);
                oneMat->setMatName(matName);
                if(logFile) log<<"New name: '"<<matName<<"'."<<endl;
                //Riparte a controllare dall'inizio dell'elenco
                j=0;
            }
            else j++;
        }
        //Nome dell'immagine
        string imageName=oneMat->getImageName();
        if(imageName.length()>11) {
            //Nome troppo lungo per il 3ds: va troncato
            if(logFile) log<<"\tMaterial no."<<(int)i<<", image name '"<<imageName<<"' too long. ";
            imageName.resize(5);
            imageName+="~";
            imageName+=int2string(++contaImg, true);
            imageName+=".ace";
            oneMat->setImageName(imageName);
            if(logFile) log<<" New name: '"<<imageName<<"'."<<endl;
        }
    }
}


/* Controlla i nomi degli oggetti:
 *  - no comment.
 */
void Mesh::checkObjNames(void) {
    //Carattere da aggiungere come contatore ai nomi troncati
    u8 contaObj=0, senzaNome=0;
    //Numero di oggetti presenti
    u8 numObjs=objects.getLung();
    for(u8 i=0; i<numObjs; i++) {
        //Prende l'indirizzo dell'i-esimo object
        Object *oneObj=objects.getElem(i);
        //Nome dell'oggetto
        string objName=oneObj->getObjName();
        if(objName.length()>10) {
            //Nome troppo lungo per il 3ds: va troncato
            if(logFile) log<<"\tObject no."<<(int)i<<", name '"<<objName<<"' too long. ";
            objName.resize(7);
            objName+="~";
            objName+=int2string(++contaObj, true);
            oneObj->setObjName(objName);
            if(logFile) log<<" New name: '"<<objName<<"'."<<endl;
        }
        //Controlla se il nome  univoco
        u8 j=0;
        while(j<i) {
            string anotherObjName=objects.getElem(j)->getObjName();
            if(anotherObjName==objName) {
                //Se l'i-esimo ha lo stesso nome del j-esimo, cambia il nome del primo
                if(logFile) log<<"\tObject no."<<(int)i<<", name duplicated. ";
                objName="Obj#"+int2string(++senzaNome, true);
                oneObj->setObjName(objName);
                if(logFile) log<<" New name: '"<<objName<<"'."<<endl;
                //Riparte a controllare dall'inizio dell'elenco
                j=0;
            }
            else j++;
        }
    }
}




/** CLASSE MESH: SALVA L'ELENCO DEI MATERIALI **/

/* Salva su file l'elenco dei materiali:
 *  - no comment.
 */
void Mesh::writeMatFile(const char filename[]) {
    //Apre il file
    ofstream file(filename);
    if(!file.is_open()) return;
    //Numero di materiali presenti
    u8 numMats=materials.getLung();
    for(u8 i=0; i<numMats; i++) {
        //Prende l'indirizzo dell'i-esimo materiale
        Material *oneMat=materials.getElem(i);
        file<<(int)i+1<<") Material '"<<oneMat->getMatName()<<"':"<<endl;
        file<<"\t- Texture image name: '"<<oneMat->getImageName()<<"';"<<endl;
        file<<"\t- Texture filter mode name: '"<<oneMat->getFilterName()<<"';"<<endl;
        file<<"\t- Shader name: '"<<oneMat->getShaderName()<<"';"<<endl;
        file<<"\t- Alpha test mode: '";
        switch(oneMat->getAlphaMode()) {
            case 0:
                file<<"None";
                break;
            case 1:
                file<<"Trans";
                break;
            default:
                file<<"?";
        }
        file<<"';"<<endl;
        file<<"\t- Z-buffer mode: '";
        switch(oneMat->getZBuffMode()) {
            case 0:
                file<<"None";
                break;
            case 1:
                file<<"Normal";
                break;
            case 2:
                file<<"Write only";
                break;
            case 3:
                file<<"Test only";
                break;
            default:
                file<<"?";
        }
        file<<"';"<<endl;
        file<<"\t- Material shining: '";
        switch(oneMat->getMatShining()) {
            case -5:
                file<<"OptSpecular0, normal";
                break;
            case -6:
                file<<"OptSpecular25, low specular shine";
                break;
            case -7:
                file<<"OptSpecular750, high specular shine";
                break;
            case -8:
                file<<"OptFullBright, always full bright";
                break;
            case -9:
                file<<"Cruciform";
                break;
            case -10:
                file<<"CruciformLong";
                break;
            case -11:
                file<<"OptHalfbright";
                break;
            case -12:
                file<<"DarkShade - always dark";
                break;
            default:
                file<<"?";
        }
        file<<"';"<<endl;
        file<<"\t- MipMap LOD bias: "<<oneMat->getMipMapLevel()<<";"<<endl<<endl;
    }
    //Chiude il file
    file.close();
}




/** CLASSE MESH: INTERROGAZIONE LISTE **/

/* Lunghezza della lista
 */
u16 Mesh::getNumMats(void) {
    return materials.getLung();
}

u16 Mesh::getNumObjs(void) {
    return objects.getLung();
}


/* Un elemento della lista
 */
Material *Mesh::getMat(u16 n) {
    return materials.getElem(n);
}

Object *Mesh::getObj(u16 n) {
    return objects.getElem(n);
}



/** CLASSE MESH: INTERROGAZIONE LISTE **/

/* Restituisce il numero massimo di keys
 */
u8 Mesh::getMaxKeyNum(void) {
    return maxKeyNum;
}


/** CLASSE MESH: CANCELLAZIONE LISTE **/

/* Cancella le liste di geometria contenute in mesh al termine dell'esportazione
 *  - permette di riutilizzare lo stesso oggetto per pi LODs;
 *  - non cancella la lista dei materiali.
 *  - non cancella la lista degli oggetti.
 *  - non cancella le animazioni
 */
void Mesh::eraseGeom(void) {
    u8 numObjs=objects.getLung();
    for(u8 i=0; i<numObjs; i++) {
        Object *oneObj=objects.getElem(i);
        u8 numSubObjs=oneObj->getNumSubObjs();
        for(u8 j=0; j<numSubObjs; j++) {
            SubObject *oneSubObj=oneObj->getSubObj(j);
            //Cancella facce e normali del subobject
            oneSubObj->eraseFaces();
            oneSubObj->eraseNormals();
        }
        //Cancella i subobjects
        oneObj->eraseSubObjs();
        //Cancella l'elenco dei vertici
        oneObj->erasePoints();
        //Cancella l'elenco dei vertici uv
    }
}


/* Cancella la lista dei materiali
 */
void Mesh::eraseMat(void) {
    materials.eraseAll();
}


/* Cancella tutte le liste
 */
void Mesh::eraseAll(void) {
    //Cancella gli oggetti
    u8 numObjs=objects.getLung();
    for(u8 i=0; i<numObjs; i++) {
        Object *oneObj=objects.getElem(i);
        u8 numSubObjs=oneObj->getNumSubObjs();
        for(u8 j=0; j<numSubObjs; j++) {
            SubObject *oneSubObj=oneObj->getSubObj(j);
            //Cancella facce e normali del subobject
            oneSubObj->eraseFaces();
            oneSubObj->eraseNormals();
        }
        //Cancella i subobjects
        oneObj->eraseSubObjs();
        //Cancella l'elenco dei vertici
        oneObj->erasePoints();
        //Cancella l'elenco dei vertici uv
        oneObj->eraseUvPoints();
        //Cancella gli elenchi delle animazioni
        oneObj->eraseRotKeys();
        oneObj->eraseLinKeys();
    }
    //Cancella i materiali
    materials.eraseAll();
}

