#ifndef MESH_H_INCLUDED
#define MESH_H_INCLUDED


#include <iostream>
#include <fstream>
#include <cstdlib>
#include <string>
#include "Tipi.h"
#include "Liste.h"
#include "TSshape.h"


using namespace std;



/** CLASSE MATERIAL **/

/* Questa classe rappresenta una materiale da applicare alle facce:
 * -
 */
class Material {

    public:
        //Costruttore e distruttore
        Material(void);
        ~Material(void) { };
        //Assegnazione
        void setMatName(string _matName);
        void setImageName(string _imageName);
        void setFilterName(string _filterName);
        void setShaderName(string _shaderName);
        void setAlphaMode(u8 _alphaMode);
        void setZBuffMode(u8 _zBuffMode);
        void setMatShining(s8 _matShining);
        void setMipMapLevel(f32 _mipMapLevel);
        void setColor(u8 _r, u8 _g, u8 _b);
        //Interrogazione
        string getMatName(void);
        string getImageName(void);
        string getFilterName(void);
        string getShaderName(void);
        u8 getAlphaMode(void);
        u8 getZBuffMode(void);
        s8 getMatShining(void);
        f32 getMipMapLevel(void);
        u8 getColorR(void);
        u8 getColorG(void);
        u8 getColorB(void);

    private:
        string matName; //Nome del materiale
        string imageName; //Nome dell'immagine
        string filterName; //Nome del filtro texture
        string shaderName; //Nome dello shader
        u8 alphaMode; //AlphaTestMode
        u8 zBuffMode; //ZBuffMode
        s8 matShining; //Shining
        f32 mipMapLevel; //MipMapLod Bias
        u8 r, g, b; //Colore
};





/** CLASSE SUBOBJECT **/

/* Questa classe rappresenta un subobject della mesh:
 * -
 */
class SubObject {

    public:
        //Costruttore e distruttore
        SubObject(void);
        ~SubObject(void) { };
        //Assegnazione
        void setNumMat(u16 _numMat);
        void setVertIdx(u16 face, u8 n, u16 vertIdx);
        //Interrogazione
        u16 getNumMat(void);
        u16 getNumFaces(void);
        u16 getVertIdx(u16 face, u8 n);
        //Aggunge un elemento
        void addFace(u16 vertexIdx[]);
        void addNormal(f32 x, f32 y, f32 z);
        //Restituisce l'indirizzo di un elemento delle liste private
        Vett3<f32> *getNormal(u16 faccia);
        //Svuota le liste
        void eraseFaces(void);
        void eraseNormals(void);

    private:
        //Elenco delle facce (tre idx vertici)
        Lista< Vett3<u16> > faces;
        //Elenco normali (punto 3D), in corrispondenza 1 a 1 con le facce (faccia n => normale n)
        Lista< Vett3<f32> > normals;
        //Numero del materiale da applicare alle facce elencate
        u16 numMat;
        //todo
};




/** CLASSE KEY **/

/* Questa classe  la classe base per i lin e rot keys
 * -
 */
class Key {

    public:
        //Costruttore e distruttore
        Key(void) { };
        ~Key(void) { };
        //Assegnazione
        void setNumKey(u8 n);
        //Interrogazione
        u8 getNumKey(void);

    private:
        //Numero fotogramma
        u8 numKey;
};




/** CLASSE ROTKEY **/

/* Questa classe rappresenta un fotogramma di rotazione:
 * -
 */
class RotKey : public Vett4<f32>, public Key {

    public:
        //Costruttore e distruttore
        RotKey(void) { };
        ~RotKey(void) { };
        //Nothing else

    private:
        //Nothing
};




/** CLASSE LINKEY **/

/* Questa classe rappresenta un fotogramma di traslazione:
 * -
 */
class LinKey : public Vett3<f32>, public Key {

    public:
        //Costruttore e distruttore
        LinKey(void) { };
        ~LinKey(void) { };
        //Nothing else

    private:
        //Nothing
};




/** CLASSE OBJECT **/

/* Questa classe rappresenta un oggetto:
 * -
 */
class Object {

    public:
        //Costruttore e distruttore
        Object(void);
        ~Object(void) { };
        //Assegnazione
        void setObjName(string _objName);
        void setParent(s16 _parent);
        //Aggiunge un elemento alle liste private
        SubObject *addSubObj(void);
        void addPoint(f32 x, f32 y, f32 z);
        void addUvPoint(f32 u, f32 v);
        void addLinKey(u8 n, Vett3<f32> *trasl);
        void addRotKey(u8 n, Vett4<f32> *quat);
        //Interrogazione
        s16 getParent(void);
        string getObjName(void);
        Matr4x4<f32> *getAssMatr(void);
        Matr4x4<f32> *getRelMatr(void);
        //Restituisce l'indirizzo di un elemento delle liste private
        SubObject *getSubObj(u8 n);
        Vett3<f32> *getPoint(u16 n);
        Vett2<f32> *getUvPoint(u16 n);
        RotKey *getRotKey(u8 n);
        LinKey *getLinKey(u8 n);
        //Restituisce la lunghezza delle liste private
        u8 getNumSubObjs(void);
        u16 getNumPoints(void);
        u16 getNumUvPoints(void);
        u8 getNumRotKeys(void);
        u8 getNumLinKeys(void);
        //Cancellazione liste private
        void eraseSubObjs(void);
        void erasePoints(void);
        void eraseUvPoints(void);
        void eraseRotKeys(void);
        void eraseLinKeys(void);

    private:
        //Nome dell'oggetto
        string objName;
        //Numero dell'oggetto parent
        s16 parent;
        //Matrice trasformazione coordinate (compone rot e trasl)
        Matr4x4<f32> assMatr; //In termini assoluti
        Matr4x4<f32> relMatr; //Relativa al parent object
        //Elenco dei vertici
        Lista< Vett3<f32> > vertices;
        //Elenco dei vertici uv
        Lista< Vett2<f32> > uvVertices;
        //Elenco dei subobject (che contengono le liste delle facce suddivise per materiale)
        Lista<SubObject> subObjects;
        //Elenco fotogrammi animazione
        Lista<RotKey> rotKeys;
        Lista<LinKey> linKeys;
};




/** CLASSE MESH **/

/* Questa classe rappresenta una mesh corrispondente ad un LOD di TS:
 * -
 */
class Mesh {

    public:
        //Costruttore e distruttore
        #ifdef CONSOLE
        Mesh(TSshape &_tsShape, ostream &_log, bool _logFile);
        #else
        Mesh(TSshape &_tsShape, ostream &_log, bool _logFile, ProgressDialog &_pgsDlg);
        #endif
        ~Mesh(void) { };
        //Conversione ts <-> mesh: dati indipendenti dai lods
        void importMat(void);
        void importObj(void);
        void importMatrices(void);
        void importKeys(void);
        //Conversione ts <-> mesh: dati dipendenti dai lods
        void importFaces(u8 numLod);
        void importVertices(u8 numLod);
        void checkNormals(void);
        //Preparazione mesh per l'esportazione
        void checkMatNames(void);
        void checkObjNames(void);
        //Salva su file l'elenco dei materiali
        void writeMatFile(const char filename[]);
        //Interrogazione liste private: lunghezza
        u16 getNumMats(void);
        u16 getNumObjs(void);
        //Interrogazione liste private: un elemento
        Material *getMat(u16 n);
        Object *getObj(u16 n);
        //Interrogazione dati
        u8 getMaxKeyNum(void);
        //Cancellazione liste private
        void eraseGeom(void);
        void eraseMat(void);
        void eraseAll(void);

    private:
        //Reference all'oggetto tsShape
        TSshape &tsShape;
        //Stream per file log
        ostream &log;
        bool logFile;
        #ifndef CONSOLE
        //Finestra dialogo attesa
        ProgressDialog &pgsDlg;
        #endif
        //Contenuto
        Lista<Material> materials;
        Lista<Object> objects;
        //Il key con il numero maggiore
        u8 maxKeyNum;
};



#endif // MESH_H_INCLUDED
