#ifndef TSCLASS_H_INCLUDED
#define TSCLASS_H_INCLUDED

#include <iostream>
#include <string>
#include "Tipi.h"
#include "Errore.h"
#include "Liste.cpp"
#include "Vettori.cpp" //Per evitare errore 'undefined reference' includere il cpp invece dell'h
#include "MathUtils.cpp" //Per evitare errore 'undefined reference' includere il cpp invece dell'h
#include "FileUtf16in.h"

#ifndef CONSOLE
    #include "../ProgressDialog.h"
#endif


using namespace std;



/** CLASSI PER DATI DEL FILE SHAPE: TSvector **/

/* Questa classe corrisponde al contenitore 'point' o 'vector' e contiene:
 * - un punto 3D (vettore con tre componenti f32);
 * - classe derivata dal vettore generico con tipo f32 per aggiungere la funzione 'save'.
 */
class TSvector : public Vett3<f32> {
    public:
        //Salva i dati dal file shape
        void save(FileUtf16in *file, char delim[]);

    private:
        //nothing
};



/** CLASSI PER DATI DEL FILE SHAPE: TSuvPoint **/

/* Questa classe corrisponde al contenitore 'point' e contiene:
 * - un punto 2D (vettore con due componenti f32);
 * - classe derivata dal vettore generico con tipo f32 per aggiungere la funzione 'save'.
 */
class TSuvPoint : public Vett2<f32> {
    public:
        //Salva i dati dal file shape
        void save(FileUtf16in *file);

    private:
        //nothing
};



/** CLASSI PER DATI DEL FILE SHAPE: TSstring **/

/* Questa classe corrisponde ad una stringa:
 * - (NO)classe derivata dalla classe string per aggiungere la funzione 'save'.
 */
//class TSstring : public string {
class TSstring {
    public:
        //Costruttore e distruttore
        TSstring(void) { };
        ~TSstring(void) { };
        //Restituisce la stringa
        string getString(void);
        //Salva i dati dal file shape
        void save(FileUtf16in *file, char delim[]);

    private:
        string str;
};



/** CLASSI PER DATI DEL FILE SHAPE: TSobject **/

/* Questa classe corrisponde al contenitore 'matrix' e contiene:
 * - il nome dell'oggetto;
 * - la matrice di rotazione;
 * - il vettore di traslazione.
 */
class TSobject {
    public:
        TSobject(void) { };
        ~TSobject(void) { };
        //Assegna il valore (per memoria)
        //Restituisce il valore
        string getObjName(void); //Solo objName
        Matr4x4<f32> *getMatr(void); //Solo matrice (puntatore)
        //Salva i dati dal file shape
        void save(FileUtf16in *file);

    private:
        string objName;
        Matr4x4<f32> matr;
};



/** CLASSI PER DATI DEL FILE SHAPE: TStexture **/

/* Questa classe corrisponde al contenitore 'texture' e contiene:
 * - il numero dell'immagine usata dalla texture (riferito alla lista 'images');
 * - il numero del 'filtro' della texture (riferito alla lista 'named_filters');
 * - il valore di mipMapLodBias (valore scelto per il materiale).
 */
class TStexture {
    public:
        //Costruttore e distruttore
        TStexture(void);
        ~TStexture(void) { };
        //Assegna il valore
        void set(u8 _imageIdx, u8 _filterModeIdx, s8 _mipMapLevel); //Tutte le componenti
        void setImageIdx(u8 _imageIdx); //Solo imageIdx
        void setFilterIdx(u8 _filterModeIdx); //Solo filterModeIdx
        void setMipLevel(f32 _mipMapLevel); //Solo mipMapLevel
        //Restituisce il valore
        u8 getImageIdx(void); //Solo imageIdx
        u8 getFilterIdx(void); //Solo filterModeIdx
        f32 getMipLevel(void); //Solo mipMapLevel
        //Salva i dati dal file shape
        void save(FileUtf16in *file);

    private:
        u8 imageIdx;
        u8 filterModeIdx;
        f32 mipMapLevel;
};



/** CLASSI PER DATI DEL FILE SHAPE: TSvtxState **/

/* Questa classe corrisponde al contenitore 'vtx_state' e contiene:
 * - il numero dell'oggetto cui si applicano le propriet (riferito alla lista 'matrices');
 * - il valore della riflettivit della superficie (valore scelto per il materiale).
 */
class TSvtxState {
    public:
        //Costruttore e distruttore
        TSvtxState(void);
        ~TSvtxState(void) { };
        //Assegna il valore
        void set(u8 _objectIdx, s8 _matShining); //Tutte e tre le componenti
        void setObjectIdx(u8 _objectIdx); //Solo objectIdx
        void setMatShining(s8 _matShining); //Solo matShining
        //Restituisce il valore
        u8 getObjectIdx(void); //Solo objectIdx
        s8 getMatShining(void); //Solo matShining
        //Salva i dati dal file shape
        void save(FileUtf16in *file);

    private:
        u8 objectIdx;
        s8 matShining;
};



/** CLASSI PER DATI DEL FILE SHAPE: TSprimState **/

/* Questa classe corrisponde al contenitore 'prim_state' e contiene:
 * - il nome del materiale (si trova prima della parentesi; pu non esistere);
 * - il numero dello shader utilizzato (riferito alla lista 'shader_names');
 * - il numero della texture utilizzata (riferito alla lista 'textures');
 * - il numero del vtx_state utilizzato (riferito alla lista 'vtx_states'):
 * - il valore di alphaTestMode (valore scelto per il materiale);
 * - il numero del light_cfg utilizzato (riferito alla lista 'ligth_model_cfg'?);
 * - il valore di zBuffMode (valore scelto per il materiale).
 */
class TSprimState {
    public:
        //Costruttore e distruttore
        TSprimState(void);
        ~TSprimState(void) { };
        //Assegna il valore
        void set(string name, u8 shaderIdx, u8 textureIdx, u8 vtxStateIdx,
                    u8 alphaMode, u8 lightCfgIdx, u8 zBuffMode); //Tutte e tre le componenti
        void setNumMat(u16 _numMat);
        //Restituisce il valore
        string getMatName(void); //Solo il nome
        u8 getShaderIdx(void); //Solo shaderIdx
        u8 getTextureIdx(void); //Solo textureIdx
        u8 getVtxStateIdx(void); //Solo
        u8 getAlphaMode(void); //Solo
        u8 getLightCfgIdx(void); //Solo
        u8 getZBuffMode(void); //Solo
        u16 getNumMat(void);
        bool getHasTexture(void);
        //Salva i dati dal file shape
        void save(FileUtf16in *file);

    private:
        string matName;
        u8 shaderIdx;
        u8 textureIdx;
        u8 vtxStateIdx;
        u8 alphaMode;
        u8 lightCfgIdx;
        u8 zBuffMode;
        u16 numMat; //Usato nella conversione primstate-materiale
        bool hasTexture;
};



/** CLASSI PER DATI DEL FILE SHAPE: TSvertex **/

/* Questa classe corrisponde al contenitore 'vertices' e contiene:
 * - il numero del vertice (punto 3D) (riferito alla lista 'points');
 * - il numero della normale (punto 3D) (riferito alla lista 'normals');
 * - il numero del vertice uv della mappa (punto 2D) (riferito alla lista 'uv_points').
 * - il colore (valore scelto per il materiale). --> todo
 */
class TSvertex {
    public:
        //Costruttore e distruttore
        TSvertex(void);
        ~TSvertex(void) { };
        //Assegna il valore
        void set(u16 _pointIdx,u16 _normalIdx, u16 _uvPointIdx); //Tutte le componenti
        void setPointIdx(u16 _pointIdx); //Solo pointIdx
        void setNormalIdx(u16 _normalIdx); //Solo normalIdx
        void setUvPointIdx(u16 _uvpointIdx); //Solo uvPointIdx
        //Restituisce il valore
        u16 getPointIdx(void); //Solo pointIdx
        u16 getNormalIdx(void); //Solo normalIdx
        u16 getUvPointIdx(void); //Solo uvPointIdx
        //Salva i dati dal file shape
        void save(FileUtf16in *file);
        //Organizzazione vertex
        void setIsDupl(bool _isDupl);
        void setEqVertexIdx(u16 _eqVertexIdx);
        bool getIsDupl(void);
        u16 getEqVertexIdx(void);

    private:
        u16 pointIdx;
        u16 normalIdx;
        u16 uvPointIdx;
        bool isDupl; //Indica se in elenco c' un vertex uguale (tranne che per la normale)
        u16 eqVertexIdx; //Se isDupl=true indica il vertex da usare (il primo uguale in lista)
        //Vett4<u8> colour;
};



/** CLASSI PER DATI DEL FILE SHAPE: TSface **/

/* Questa classe corrisponde al contenitore 'primitives' e contiene:
 * - il numero del vertice 1,2,3 (riferito alla lista 'vertices');
 * - il numero della normale (punto 3D) (riferito alla lista 'normals').
 * - il numero del prim_state da applicare (riferito alla lista 'prim_states').
 */
class TSface {
    public:
        //Costruttore e distruttore
        TSface(void);
        ~TSface(void) { };
        //Assegna il valore
        void set(u16 _verticesIdx[], u16 _normalIdx, u8 _primStateIdx); //Tutte e tre le componenti
        void setVerticesIdx(u16 _verticesIdx[]); //Solo verticesIdx
        void setVerticesIdx(u8 n, u16 vertexIdx); //Solo 1 verticesIdx
        void setNormalIdx(u16 _normalIdx); //Solo normalIdx
        void setPrimStateIdx(u8 _primStateIdx); //Solo primstateIdx
        //Restituisce il valore
        u16 *getVerticesIdx(void); //Solo verticesIdx (tutti)
        u16 getVerticesIdx(u8 n); //Solo verticesIdx (uno solo, con n=1-3)
        u16 getNormalIdx(void); //Solo normalIdx
        u8 getPrimStateIdx(void); //Solo primStateIdx
        //Salva i dati dal file shape: non esiste (per memoria)

    private:
        u16 verticesIdx[3];
        u16 normalIdx;
        u8 primStateIdx;
};



/** CLASSI PER DATI DEL FILE SHAPE: TSlod **/

/* Questa classe corrisponde al contenitore 'distance_level' e contiene:
 * - l'elenco dei 'vertices';
 * - l'elenco delle 'faces';
 * - la distanza di attivazione (valore scelto per il LOD).
 */
class TSlod {
    public:
        //Costruttore e distruttore
        TSlod(void);
        ~TSlod(void) { };
        //Assegna il valore (assente; per memoria)
        //Restituisce il valore
        TSvertex *getPVertex(u8 n); //Solo un puntatore a TSvertex
        TSface *getPFace(u8 n); //Solo un puntatore a TSface
        s8 getHierarchy(u8 n); //Solo un valore di hierarchy
        f32 getDistance(void); //Solo il valore di distance
        //Restituisce i puntatori alle liste
        Lista<TSvertex> *getTSvertices(void);
        Lista<TSface> *getTSfaces(void);
        Lista<s8> *getHierarchy(void);
        //Salva i dati dal file shape
        #ifdef CONSOLE
        void save(FileUtf16in *file);
        #else
        void save(FileUtf16in *file, ProgressDialog &pgsDlg);
        #endif
        //Cancella le liste
        void eraseAll(void);

    private:
        Lista<TSvertex> tsVertices;
        Lista<TSface> tsFaces;
        Lista<s8> hierarchy;
        f32 distance;
};



/** CLASSI PER DATI DEL FILE SHAPE: TSrotKey **/

/* Questa classe corrisponde al contenitore 'tcb_key' oppure 'slerp_rot' e contiene:
 * - il numero del fotogramma;
 * - un quaternione di rotazione (Vett4<f32>).
 */
class TSrotKey {
    public:
        //Costruttore e distruttore
        TSrotKey(void);
        ~TSrotKey(void) { };
        //Assegna il valore (assente; per memoria)
        //Restituisce il valore
        u8 getNumKey(void); //Solo il numKey
        Vett4<f32> *getQuat(void); //Solo il quaternione
        //Salva i dati dal file shape
        void save(FileUtf16in *file);

    private:
        u8 numKey;
        Vett4<f32> quat;
};



/** CLASSI PER DATI DEL FILE SHAPE: TSlinKey **/

/* Questa classe corrisponde al contenitore 'linear_pos' e contiene:
 * - il numero del fotogramma;
 * - un vettore di spostamento (Vett3<f32>).
 */
class TSlinKey {
    public:
        //Costruttore e distruttore
        TSlinKey(void);
        ~TSlinKey(void) { };
        //Assegna il valore (assente; per memoria)
        //Restituisce il valore
        u8 getNumKey(void); //Solo il numKey
        Vett3<f32> *getTrasl(void); //Solo il quaternione
        //Salva i dati dal file shape
        void save(FileUtf16in *file);

    private:
        u8 numKey;
        Vett3<f32> trasl;
};



/** CLASSI PER DATI DEL FILE SHAPE: TSanimNode **/

/* Questa classe corrisponde al contenitore 'animNode' e contiene:
 * - il nome dell'oggetto;
 * - la lista dei TSrotKey;
 * - la lista dei TSlinKey.
 */
class TSanimNode {
    public:
        //Costruttore e distruttore
        TSanimNode(void);
        ~TSanimNode(void) { };
        //Assegna il valore (assente; per memoria)
        //Restituisce il valore
        string getObjName(void); //Solo objName
        TSrotKey *getRotKey(u8 n); //Solo un puntatore a TSrotKey
        TSlinKey *getLinKey(u8 n); //Solo un puntatore a TSlinKey
        //Numero di keys
        u8 getNumRotKeys(void);
        u8 getNumLinKeys(void);
        //Salva i dati dal file shape
        void save(FileUtf16in *file);
        //Cancella le liste
        void eraseAll(void);

    private:
        string objName;
        Lista<TSrotKey> tsRotKeys;
        Lista<TSlinKey> tsLinKeys;
};


/** CLASSI PER DATI DEL FILE SHAPE: - **/

/* Questa classe corrisponde al contenitore 'vtx_state' e contiene:
 * - il numero dell'oggetto cui si applicano le propriet (riferito alla lista 'matrix');
 * - il valore della riflettivit della superficie (valore scelto per il materiale).
 */
/*class TStexture {
    public:
        //Costruttore e distruttore
        TStexture(void);
        ~TStexture(void) { };
        //Assegna il valore
        void set(); //Tutte e tre le componenti
        void set(); //Solo
        //Restituisce il valore
        tipo get(void); //Solo
        //Salva i dati dal file shape
        void save(FileUtf16in *file);

    private:
        //Cont
};*/

#endif // TSCLASS_H_INCLUDED
