#ifndef TSSHAPE_CPP_INCLUDED
#define TSSHAPE_CPP_INCLUDED

#include "TSshape.h"


/** FUNZIONI MEMBRO --- COSTRUTTORE & DISTRUTTORE **/

/* Costruttore:
 * - inizializza i codici di errore delle liste.
 */
#ifdef CONSOLE
TSshape::TSshape(ostream &_log, bool _logFile) : log(_log)
#else
TSshape::TSshape(ostream &_log, bool _logFile, ProgressDialog &_pgsDlg) : log(_log), pgsDlg(_pgsDlg)
#endif
{
    tsShaders.setErrCode(ERR_CL_TS_SHADER);
    tsFilters.setErrCode(ERR_CL_TS_FILTER);
    tsPoints.setErrCode(ERR_CL_TS_POINT);
    tsUVPoints.setErrCode(ERR_CL_TS_UVPOINT);
    tsNormals.setErrCode(ERR_CL_TS_NORMAL);
    tsObjects.setErrCode(ERR_CL_TS_OBJECT);
    tsImages.setErrCode(ERR_CL_TS_IMAGE);
    tsTextures.setErrCode(ERR_CL_TS_TEXTURE);
    tsVtxStates.setErrCode(ERR_CL_TS_VTXSTATE);
    tsPrimStates.setErrCode(ERR_CL_TS_PRIMSTATE);
    tsLods.setErrCode(ERR_CL_TS_LOD);
    tsAnimNodes.setErrCode(ERR_CL_TS_ANIM);
    logFile=_logFile;
}


/* Costruttore: (per memoria)
 * - nothing to do.
 */




/** FUNZIONI MEMBRO --- GESTIONE FILE **/

/* Apre il file shape:
 * - richiama la funzione apposita di FileUtf16in.
 */
void TSshape::openFile(const char filename[]) {
    file.open(filename);
}


/* Chiude il file shape:
 * - richiama la funzione apposita di FileUtf16in.
 */
void TSshape::closeFile(void) {
    file.close();
}


/* Controlla il formato del file shape:
 * -
 */
s8 TSshape::checkType(void) {
    //todo
    return 0;
}


/* Converte il file shape da binario a unicode:
 * -
 */
void TSshape::bin2utf(void) {
    //todo
}




/** FUNZIONI MEMBRO --- GESTIONE FILE **/

/* Lettura della parte generale del file shape:
 * -
 */
void TSshape::readGeneral(void) {
    pgsDlg.setProgress(0);
    salvaLista(&tsShaders       ,   "shader_names ( "           ,   "named_shader ( "       );
    salvaLista(&tsFilters       ,   "texture_filter_names ( "   ,   "named_filter_mode ( "  );
    salvaLista(&tsPoints        ,   "points ( "                 ,   "point ( "              );
    salvaLista(&tsUVPoints      ,   "uv_points ( "              );
    salvaLista(&tsNormals       ,   "normals ( "                ,   "vector ( "             );
    salvaLista(&tsObjects       ,   "matrices ( "               );
    salvaLista(&tsImages        ,   "images ( "                 ,   "image ( "              );
    salvaLista(&tsTextures      ,   "textures ( "               );
    salvaLista(&tsVtxStates     ,   "vtx_states ( "             );
    salvaLista(&tsPrimStates    ,   "prim_states ( "            );
}


/* Lettura della parte del file shape relativa ai LODs:
 * -
 */
void TSshape::readLods(bool readAllLods) {
    if(file.findString("distance_levels ( ")!=1) {
        //Non ha trovato il delimitatore
        throw Errore(ERR_OP_SAVE_DATA, ERR_TP_MISS_DELIM_1);
    }
    u16 lung=0, mem=0;
    //Legge la lunghezza della lista
    file.readInt(&lung);
    if(lung==0) {
        //Lunghezza==0
        throw Errore(tsLods.getErrCode(), ERR_OP_SAVE_DATA, ERR_TP_NULL_LENGTH);
    }
    //Lunghezza della lista ammissibile
    tsLods.eraseAll(); //Svuota la lista (o fa nulla se  gi vuota)
    if(!readAllLods) lung=1; //Se vuole convertire solo il primo LOD
    for(u16 i=0; i<lung; i++) {
        try {
            #ifdef CONSOLE
            tsLods.addElem()->save(&file);
            #else
            pgsDlg.setProgress(0); //Azzera ad ogni LOD
            tsLods.addElem()->save(&file, pgsDlg);
            #endif
        }
        catch(Errore err) {
            //Aggiunge le informazioni (se non sono gi presenti) e poi rilancia l'errore
            if(err.getErrClass()==ERR_CL_NULL) {
                err.setErrClass(tsLods.getErrCode());
                err.setNumElem(i+1);
            }
            throw err;
        }
    }
    //Una volta completata la lista, la controlla
    mem=tsLods.check();
    if(mem!=lung) {
        //Ha memorizzato un numero di oggetti differente da quato previsto
        throw Errore(tsLods.getErrCode(), ERR_OP_SAVE_DATA, ERR_TP_DIFF_LENGTH);
    }
}


/* Lettura della parte del file shape relativa ai LODs:
 * -
 */
void TSshape::readAnims(void) {
    if(file.findString("anim_nodes ( ")==0) {
        //Non ha trovato il delimitatore
        //throw Errore(ERR_OP_SAVE_DATA, ERR_TP_MISS_DELIM_1);
        //Non  un errore perch in questo caso l'oggetto non  animato!
        return;
    }
    u16 lung=0, mem=0;
    //Legge la lunghezza della lista
    file.readInt(&lung);
    if(lung==0) {
        throw Errore(tsAnimNodes.getErrCode(), ERR_OP_SAVE_DATA, ERR_TP_NULL_LENGTH);
    }
    //Lunghezza della lista ammissibile
    tsAnimNodes.eraseAll(); //Svuota la lista (o fa nulla se  gi vuota)
    for(u16 i=0; i<lung; i++) {
        try {
            tsAnimNodes.addElem()->save(&file);
        }
        catch(Errore err) {
            //Aggiunge le informazioni (se non sono gi presenti) e poi rilancia l'errore
            if(err.getErrClass()==ERR_CL_NULL) {
                err.setErrClass(tsAnimNodes.getErrCode());
                err.setNumElem(i+1);
            }
            throw err;
        }
    }
    //Una volta completata la lista, la controlla
    mem=tsAnimNodes.check();
    if(mem!=lung) {
        //Ha memorizzato un numero di oggetti differente da quato previsto
        throw Errore(tsAnimNodes.getErrCode(), ERR_OP_SAVE_DATA, ERR_TP_DIFF_LENGTH);
    }
}



/** LETTURA DI UNA LISTA GENERICA */

/* Legge un elenco da file e riempie la lista:
 * - un solo delimitatore in input (il/i altro/i sono definiti nella funzione save dell'oggetto in lista);
 * - funzione PRIVATA.
 */
template<class tipo>
void TSshape::salvaLista(Lista<tipo> *lista, char delim1[]) {
    if(file.findString(delim1)!=1) {
        //Non ha trovato il delimitatore
        throw Errore(ERR_OP_SAVE_DATA, ERR_TP_MISS_DELIM_1);
    }
    u16 lung=0, mem=0;
    //Legge la lunghezza della lista
    file.readInt(&lung);
    if(lung==0) {
        //Lunghezza==0
        throw Errore(lista->getErrCode(), ERR_OP_SAVE_DATA, ERR_TP_NULL_LENGTH);
    }
    //La lunghezza della lista  ammissibile
    lista->eraseAll(); //Svuota la lista (o fa nulla se  gi vuota)
    for(u16 i=0; i<lung; i++) {
        try {
            lista->addElem()->save(&file);
        }
        catch(Errore err) {
            //Aggiunge le informazioni e poi rilancia l'errore
            err.setErrClass(lista->getErrCode());
            err.setNumElem(i+1);
            throw err;
        }
    }
    //Una volta completata la lista, la controlla
    mem=lista->check();
    if(mem!=lung) {
        //Lunghezza==0
        throw Errore(lista->getErrCode(), ERR_OP_SAVE_DATA, ERR_TP_DIFF_LENGTH);
    }
}


/* Legge un elenco da file e riempie la lista:
 * - due delimitatori in input (per oggetti che hanno pi possibilit d'uso, es TSstring);
 * - funzione PRIVATA.
 */
template<class tipo>
void TSshape::salvaLista(Lista<tipo> *lista, char delim1[], char delim2[]) {
    if(file.findString(delim1)!=1) {
        //Non ha trovato il delimitatore
        throw Errore(ERR_OP_SAVE_DATA, ERR_TP_MISS_DELIM_1);
    }
    u16 lung=0, mem=0;
    //Legge la lunghezza della lista
    file.readInt(&lung);
    if(lung==0) {
        //Lunghezza==0
        throw Errore(lista->getErrCode(), ERR_OP_SAVE_DATA, ERR_TP_NULL_LENGTH);
    }
    //La lunghezza della lista  ammissibile
    lista->eraseAll(); //Svuota la lista (o fa nulla se  gi vuota)
    for(u16 i=0; i<lung; i++) {
        try {
            lista->addElem()->save(&file, delim2);
        }
        catch(Errore err) {
            //Aggiunge le informazioni e poi rilancia l'errore
            err.setErrClass(lista->getErrCode());
            err.setNumElem(i+1);
            throw err;
        }
    }
    //Una volta completata la lista, la controlla
    mem=lista->check();
    if(mem!=lung) {
        //Lunghezza==0
        throw Errore(lista->getErrCode(), ERR_OP_SAVE_DATA, ERR_TP_DIFF_LENGTH);
    }
}



/** FUNZIONI MEMBRO --- RICHIESTA INDIRIZZI LISTE **/

/* Richiesta indirizzi liste private:
 */
Lista<TSstring> *TSshape::getTSshaders(void) {
    return &tsShaders;
}

Lista<TSstring> *TSshape::getTSfilters(void) {
    return &tsFilters;
}

Lista<TSvector> *TSshape::getTSpoints(void) {
    return &tsPoints;
}

Lista<TSuvPoint> *TSshape::getTSuvPoints(void) {
    return &tsUVPoints;
}

Lista<TSvector> *TSshape::getTSnormals(void) {
    return &tsNormals;
}

Lista<TSobject> *TSshape::getTSobjects(void) {
    return &tsObjects;
}

Lista<TSstring> *TSshape::getTSimages(void) {
    return &tsImages;
}

Lista<TStexture> *TSshape::getTStextures(void) {
    return &tsTextures;
}

Lista<TSvtxState> *TSshape::getTSvtxStates(void) {
    return &tsVtxStates;
}

Lista<TSprimState> *TSshape::getTSprimStates(void) {
    return &tsPrimStates;
}

Lista<TSlod> *TSshape::getTSlods(void) {
    return &tsLods;
}

Lista<TSanimNode> *TSshape::getTSanimNodes(void) {
    return &tsAnimNodes;
}



/** FUNZIONI MEMBRO --- CANCELLAZIONE LISTE **/

/* Cancella tutte le liste
 */
void TSshape::eraseAll(void) {
    //Cancella le liste
    tsShaders.eraseAll();
    tsFilters.eraseAll();
    tsPoints.eraseAll();
    tsUVPoints.eraseAll();
    tsNormals.eraseAll();
    tsObjects.eraseAll();
    tsImages.eraseAll();
    tsTextures.eraseAll();
    tsVtxStates.eraseAll();
    tsPrimStates.eraseAll();
    //La lista lod contiene delle sottoliste
    u16 numLods=tsLods.getLung();
    for(u16 i=0; i<numLods; i++) {
        //Cancella le sottoliste
        tsLods.getElem(i)->eraseAll();
    }
    tsLods.eraseAll();
    //La lista animNodes contiene delle sottoliste
    u16 numAnimNodes=tsAnimNodes.getLung();
    for(u16 i=0; i<numAnimNodes; i++) {
        //Cancella le sottoliste
        tsAnimNodes.getElem(i)->eraseAll();
    }
    tsAnimNodes.eraseAll();
}

#endif // TSSHAPE_CPP_INCLUDED
