#ifndef TSSHAPE_H_INCLUDED
#define TSSHAPE_H_INCLUDED

#include <iostream>
#include "Tipi.h"
#include "FileUtf16in.h"
#include "Liste.cpp" //Per evitare errore 'undefined reference' includere il cpp invece dell'h
#include "TSclass.h"

#ifndef CONSOLE
    #include "../ProgressDialog.h"
#endif


using namespace std;



/** CLASSE PER LA GESTIONE DEL FILE SHAPE DI MSTS **/
/* Nota: per le liste, derivare dalla classe template "base" Lista<tipo>
    ogni lista, specificando il tipo dell'elenco e definendo la funzione "leggi" virtuale
    presente (eventualmente) nella lista base. In questo modo per leggere non occorre definire
    tutte le funzioni in TSshape.
*/

class TSshape {

    public:
        //Costruttore e distruttore
        #ifdef CONSOLE
        TSshape(ostream &_log, bool _logFile);
        #else
        TSshape(ostream &_log, bool _logFile, ProgressDialog &_pgsDlg);
        #endif
        ~TSshape(void) { };
        //Apertura, chiusura e gestione formato
        void openFile(const char filename[]);
        void closeFile(void);
        s8 checkType(void);
        void bin2utf(void);
        //Lettura file
        void readGeneral(void);
        void readLods(bool readAllLods);
        void readAnims(void);
        //Richiesta indirizi liste
        Lista<TSstring> *getTSshaders(void);
        Lista<TSstring> *getTSfilters(void);
        Lista<TSvector> *getTSpoints(void);
        Lista<TSuvPoint> *getTSuvPoints(void);
        Lista<TSvector> *getTSnormals(void);
        Lista<TSobject> *getTSobjects(void);
        Lista<TSstring> *getTSimages(void);
        Lista<TStexture> *getTStextures(void);
        Lista<TSvtxState> *getTSvtxStates(void);
        Lista<TSprimState> *getTSprimStates(void);
        Lista<TSlod> *getTSlods(void);
        Lista<TSanimNode> *getTSanimNodes(void);
        //Cancella le liste
        void eraseAll(void);

    private:
        //Stream per log
        ostream &log;
        bool logFile;
        #ifndef CONSOLE
        //Finestra dialogo attesa
        ProgressDialog &pgsDlg;
        #endif
        //File shape
        FileUtf16in file;
        //Liste del contenuto
        Lista<TSstring> tsShaders;
        Lista<TSstring> tsFilters;
        Lista<TSvector> tsPoints;
        Lista<TSuvPoint> tsUVPoints;
        Lista<TSvector> tsNormals;
        Lista<TSobject> tsObjects;
        Lista<TSstring> tsImages;
        Lista<TStexture> tsTextures;
        Lista<TSvtxState> tsVtxStates;
        Lista<TSprimState> tsPrimStates;
        Lista<TSlod> tsLods;
        Lista<TSanimNode> tsAnimNodes;
        //Funzioni generiche per comporre una lista
        template<class tipo> void salvaLista(Lista<tipo> *lista, char delim1[]);
        template<class tipo> void salvaLista(Lista<tipo> *lista, char delim1[], char delim2[]);
        //todo
};


#endif // TSSHAPE_H_INCLUDED
