#ifndef TIPI_H_INCLUDED
#define TIPI_H_INCLUDED


/** TIPEDEF **/
                                                              //byte //sign //cont  //name      //from           //to
typedef unsigned char u8; //un byte senza segno                                 1     n       int    u8               0            +255
typedef signed char s8; //un byte con segno                                     1     y       int    s8            -127            +127
typedef unsigned short int u16; //due byte senza segno (== unsigned int)        2     n       int    u16              0         +65,535
typedef signed short int s16; //due byte con segno  (== int)                    2     y       int    s16        -32,767         +32,767
typedef unsigned long int u32; //quattro byte senza segno                       4     n       int    u32              0  +4,294,967,295
typedef signed long int s32; //quattro byte con segno  (== long int)            4     y       int    s32 -2,147,483,647  +2,147,483,647
typedef float f32; //foating point quattro byte (6 cifre di precisione)         4     y     float    f32
typedef double f64; //foating point otto byte (10 cifre di precisione)          8     y     float    f64
typedef long double f96; //foating point dodici byte (14 cifre di precisione)  12     y     float    f96



/** CARATTERE SEPARATORE NEI FILEPATH **/

#define PATH_SEP '\\'



/** ENUMERAZIONI - CODICI ERRORE **/
//Il valore 0 indica campo non definito, 1 errore default

//Indica a quale classe appartiene la funzione (Who)
enum ErrClass {
    ERR_CL_NULL=0,
    ERR_CL_DEFAULT=1,
    ERR_CL_TS_SHADER,
    ERR_CL_TS_FILTER,
    ERR_CL_TS_POINT,
    ERR_CL_TS_UVPOINT,
    ERR_CL_TS_NORMAL,
    ERR_CL_TS_OBJECT,
    ERR_CL_TS_IMAGE,
    ERR_CL_TS_TEXTURE,
    ERR_CL_TS_VTXSTATE,//10
    ERR_CL_TS_PRIMSTATE,
    ERR_CL_TS_VERTEX,
    ERR_CL_TS_FACE,
    ERR_CL_TS_LOD,
    ERR_CL_TS_ANIM,
    ERR_CL_TS_LIN_KEY,
    ERR_CL_TS_ROT_KEY,
    ERR_CL_FILE_UTF,
    ERR_CL_ME_MATERIAL,
    ERR_CL_ME_OBJECT, //20
    ERR_CL_ME_MATR,
    ERR_CL_ME_KEY,
    ERR_CL_ME_VERTEX,
    ERR_CL_ME_UVVERTEX,
    ERR_CL_ME_SUBOBJECT,
    ERR_CL_ME_ROTKEY,
    ERR_CL_ME_LINKEY,
    ERR_CL_ME_FACE,
    ERR_CL_ME_NORMAL,
    ERR_CL_ME_VERTIDXS, //30
    ERR_CL_ME_MESH,
    ERR_CL_3DS
};

//Descrive l'operazione che stava compiendo (When)
enum ErrOper {
    ERR_OP_NULL=0,
    ERR_OP_DEFAULT=1,
    ERR_OP_LIST_ADD,
    ERR_OP_MEM_ALLOC,
    ERR_OP_POINTER_CHECK,
    ERR_OP_LIST_CHECK,
    ERR_OP_INT_READ,
    ERR_OP_INT_CONV,
    ERR_OP_RETURN_VAL,
    ERR_OP_SET_VAL,
    ERR_OP_SAVE_DATA, //10
    ERR_OP_IMPORT_OBJ,
    ERR_OP_IMPORT_FACES,
    ERR_OP_IMPORT_VTX,
    ERR_OP_IMPORT_KEYS,
    ERR_OP_FILE_OPENING,
    ERR_OP_SIZE_CALC,
};


//Descrive l'errore (Why)
enum ErrType {
    ERR_TP_NULL=0,
    ERR_TP_DEFAULT=1,
    ERR_TP_BAD_ALLOC,
    ERR_TP_NULL_POINTER,
    ERR_TP_FORWARD_FAIL,
    ERR_TP_BACKWARD_FAIL,
    ERR_TP_CHECK_FAIL,
    ERR_TP_CHECK_FAIL_2,
    ERR_TP_EXCEED_ARRAY,
    ERR_TP_EXCEED_UNITS,
    ERR_TP_UNEXPECTED_CHAR, //10
    ERR_TP_MISS_DELIM_1,
    ERR_TP_MISS_DELIM_2,
    ERR_TP_MISS_DELIM_3,
    ERR_TP_MISS_DELIM_4,
    ERR_TP_MISS_DELIM_5,
    ERR_TP_MISS_DELIM_6,
    ERR_TP_MISS_DELIM_7,
    ERR_TP_MISS_DELIM_8,
    ERR_TP_MISS_DELIM_9,
    ERR_TP_MISS_DELIM_10, //20
    ERR_TP_MISS_DELIM_11,
    ERR_TP_MISS_DELIM_12,
    ERR_TP_UNEXP_DELIM,
    ERR_TP_MISS_SECTION,
    ERR_TP_UNEXPECTED_NUM,
    ERR_TP_UNEXPECTED_NUM_2,
    ERR_TP_UNEXPECTED_NUM_3,
    ERR_TP_UNEXPECTED_NUM_4,
    ERR_TP_UNEXPECTED_NUM_5,
    ERR_TP_UNEXPECTED_NUM_6, //30
    ERR_TP_UNEXPECTED_NUM_7,
    ERR_TP_NULL_LENGTH,
    ERR_TP_DIFF_LENGTH,
    ERR_TP_MISS_OBJ
};

//Indica lo stato del file (se non viene usato vale 0) (Why)
enum FileStat {
    FILE_ST_OK=0,
    FILE_ST_DEFAULT=1,
    FILE_ST_CLOSED,
    FILE_ST_OPENED,
    FILE_ST_EOF
};

//Indica lo stato del file (se non viene usato vale 0) (When)
enum ErrFileOp {
    ERR_FO_NULL=0,
    ERR_FO_DEFAULT=1,
    ERR_FO_OPEN_FAIL,
    ERR_FO_READ_CHAR,
    ERR_FO_READ_STR,
    ERR_FO_READ_SINT,
    ERR_FO_READ_UINT,
    ERR_FO_READ_FLOAT,
    ERR_FO_SKIP_BYTES,
    ERR_FO_1STR_SEARCH,
    ERR_FO_2STR_SEARCH, //10
    ERR_FO_3STR_SEARCH
};



/* TIPI PER SHAPECONVERTER */ /*
enum Formato {
    MAX3DS,
    BLENDER,
};

struct Vettore3 {
    float x;
    float y;
    float z;
};

struct Vettore2 {
    float u;
    float v;
};

struct RotKey {
    int numKey;
    float quaternion[4];
    float angolo;
    float asse[3];
};

struct LinKey {
    int numKey;
    float posizione[3];
};

struct Object {
    string nome;
    //float rotazione[3][3];
    //float traslazione[3];
    float matrix[4][4];
    int parent;
    //Riservato all'animazione
    Lista<RotKey> rotKey;
    Lista<LinKey> linKey;
};

struct Texture {
    int immagine;
    int texfilter;
    int miplevel;
};

struct VtxState {
    int oggetto;
    int shine;
    //in futuro: light_model_cfg number ( sempre zero nei files standard)
};

struct PrimState {
    string nome;
    int shader;
    int texture;
    int vtxstate;
    int materiale;
};

struct Vertice {
    int punto;
    int uv;
    Vettore3 *coord;
    Vettore2 *coorduv;
};

struct Faccia {
    int v1;
    int v2;
    int v3;
    int normale;
    Vettore3 *coordnorm;
    int oggetto;
    int materiale;
    ElementoLista<Vertice> *vertici[3];
};
struct FacciaMat {
    int numMat;
    int totFacce;
};

struct Oggetto {
    string nome;
    int parent;
    //float rotazione[3][3];
    //float traslazione[3];
    //float rotazioneAss[3][3];
    //float traslazioneAss[3];
    float matrix[4][4];
    float matrixAss[4][4];
    Lista<Vertice> vertici;
    Lista<Faccia> facce;
    Lista<FacciaMat> elencoMat;
    Lista<RotKey> *rotKey;
    Lista<LinKey> *linKey;
};

struct Materiale {
    string nome;
    string immagine;
    int miplevel;
    int texfilter;
    int shine;
    int shader;
};

struct SubObject {
    Lista<Vertice> vertici;
    Lista<Faccia> facce;
};

struct DistLev {
    int distanza;
    Lista<SubObject> subobjects;
    Lista<Oggetto> oggetti;
};

struct Shape {
    Lista<string> shaders;
    Lista<string> texfilters;
    Lista<Vettore3> punti;
    Lista<Vettore2> punti_uv;
    Lista<Vettore3> normali;
    Lista<Object> objects;
    Lista<string> immagini;
    Lista<Texture> texture;
    Lista<VtxState> vtxstate;
    Lista<PrimState> primstate;
    Lista<DistLev> lod;
    Lista<Materiale> materiali;
    bool hasAnimation;
};
*/

#endif // TIPI_H_INCLUDED
